Package: latticeExtra
Version: 0.6-30
Date: 2022-06-30
Title: Extra Graphical Utilities Based on Lattice
Author: Deepayan Sarkar <deepayan.sarkar@r-project.org>, Felix Andrews <felix@nfrac.org>
Maintainer: Deepayan Sarkar <deepayan.sarkar@r-project.org>
Description: Building on the infrastructure provided by the lattice
	     package, this package provides several new high-level
	     functions and methods, as well as additional utilities
	     such as panel and axis annotation functions.
Depends: R (>= 3.6.0), lattice
Imports: grid, stats, utils, grDevices, png, jpeg, RColorBrewer,
        interp, MASS
Suggests: maps, mapproj, deldir, quantreg, zoo, mgcv
URL: http://latticeextra.r-forge.r-project.org/
LazyLoad: yes
LazyData: yes
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2022-06-30 07:04:00 UTC; deepayan
Repository: CRAN
Date/Publication: 2022-07-04 16:50:02 UTC
Built: R 4.4.0; ; 2026-05-12 12:14:04 UTC; unix

Package: latticeExtra
Version: 0.6-31
Date: 2025-09-08
Title: Extra Graphical Utilities Based on Lattice
Authors@R: c(person("Deepayan", "Sarkar", role = c("aut", "cre"),
	            email = "deepayan.sarkar@r-project.org",
		    comment = c(ORCID = "0000-0003-4107-1553")),
              person("Felix", "Andrews", role = "aut", email = "felix@nfrac.org"))
Author: Deepayan Sarkar [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4107-1553>),
  Felix Andrews [aut]
Maintainer: Deepayan Sarkar <deepayan.sarkar@r-project.org>
Description: Building on the infrastructure provided by the lattice
	     package, this package provides several new high-level
	     functions and methods, as well as additional utilities
	     such as panel and axis annotation functions.
Depends: R (>= 3.6.0), lattice
Imports: grid, stats, utils, grDevices, png, jpeg, RColorBrewer,
        interp, MASS
Suggests: maps, mapproj, deldir, quantreg, zoo, mgcv
URL: http://latticeextra.r-forge.r-project.org/
BugReports: https://github.com/deepayan/latticeExtra/issues
LazyLoad: yes
LazyData: yes
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2025-09-09 05:30:05 UTC; deepayan
Repository: CRAN
Date/Publication: 2025-09-10 06:40:45 UTC
Built: R 4.6.0; ; 2026-04-27 06:06:32 UTC; unix

import(grid)
import(lattice)

import(stats)
import(grDevices)

importFrom(utils, modifyList, head, tail, download.file)
importFrom(RColorBrewer, brewer.pal)
importFrom(png, readPNG)
importFrom(jpeg, readJPEG)
importFrom(interp, voronoi.polygons, voronoi.mosaic)



export(dendrogramGrob) ## FIXME: add some variant of heatmap

export(rootogram, panel.rootogram, prepanel.rootogram)
export(ecdfplot, panel.ecdfplot, prepanel.ecdfplot)
export(mapplot, panel.mapplot, prepanel.mapplot)
export(segplot, prepanel.segplot, panel.segplot)

S3method(rootogram, formula)
S3method(ecdfplot, formula)
S3method(ecdfplot, numeric)
S3method(segplot, formula)

S3method(mapplot, formula)

S3method(xyplot, stl)


export(panel.3dbars, panel.3dpolygon, panel.3dtext)
export(useOuterStrips, resizePanels, combineLimits)
export(custom.theme)
export(custom.theme.2)

export(xscale.components.logpower)
export(yscale.components.logpower)
export(xscale.components.fractions)
export(yscale.components.fractions)
export(xscale.components.log10ticks)
export(yscale.components.log10ticks)
export(xscale.components.log)
export(yscale.components.log)
export(xscale.components.log10.3)
export(yscale.components.log10.3)
export(xscale.components.subticks)
export(yscale.components.subticks)

## contributions from Felix Andrews

export(mergedTrellisLegendGrob)
export(panel.key)

export(horizonplot)
S3method(horizonplot, default)
export(panel.horizonplot, prepanel.horizonplot)

export(panel.smoother)
export(panel.quantile)
export(panel.tskernel)
export(simpleSmoothTs)
S3method(simpleSmoothTs, default)
S3method(simpleSmoothTs, zoo)

export(panel.2dsmoother)

export(tileplot, panel.voronoi, panel.levelplot.points)

export(panel.xblocks)
S3method(panel.xblocks, default)
S3method(panel.xblocks, ts)
S3method(panel.xblocks, zoo)
export(panel.xyarea)
S3method(panel.xyarea, default)
S3method(panel.xyarea, ts)
S3method(panel.xyarea, zoo)
export(panel.qqmath.xyarea)

export(panel.qqmath.tails) ## Deprecated as of lattice 0.18-4

export(panel.lmlineq, panel.ablineq)
export(panel.scaleArrow)

export(marginal.plot)
S3method(c, trellis)
export(xyplot.list) ## Note: not just an S3 method

export(layer, layer_, glayer, glayer_)
export(drawLayer, flattenPanel)
S3method("+", trellis)
S3method("[", layer)
S3method(print, layer)

export(as.layer)
S3method(as.layer, layer)
S3method(as.layer, trellis)
export(doubleYScale)

export(ggplot2like)
export(ggplot2like.opts)
export(axis.grid)

export(theEconomist.theme)
export(theEconomist.opts)
export(theEconomist.axis)
export(asTheEconomist)


export(panel.ellipse)
export(panel.xyimage)


### Local variables:
### mode: R
### End:

# latticeExtra

This package extends the functionality of the
[lattice](https://github.com/deepayan/lattice) package by providing
several new high-level functions and methods, as well as additional
utilities such as panel and axis annotation functions.

It also implements a layering mechanism similar to ggplot2, and allows
trellis plots to be combined easily using `c()`.

## Tests for non-standard evaluation in layer()

library(lattice)
library(latticeExtra)
oopt <- lattice.options(panel.error = NULL)

xx <- 1970
xyplot(nhtemp) + layer(panel.abline(v = xx))

layer.test1 <- function() {
    xyplot(nhtemp) + layer(panel.abline(v = xx))
}
layer.test1()

layer.test2 <- function() {
    vline <- 1920
    xyplot(nhtemp) + layer(panel.abline(v = vline))
}
try(plot(layer.test2()), silent = TRUE) # fails

layer.test3 <- function() {
    zz <- 1930
    ls.str(environment())
    xyplot(nhtemp) + layer(panel.abline(v = vline),
                           data = list(vline = zz))
}
layer.test3()

layer.test4 <- function() {
    zz <- 1940
    xyplot(nhtemp) + layer(panel.abline(v = c(zz, xx)),
                           data = environment())
}
layer.test4()

do.call(lattice.options, oopt)

prepanel.ecdfplot <-
    function(x, f.value = NULL, ...)
{
    ans <-
        prepanel.default.qqmath(x,
                                f.value = f.value,
                                distribution = qunif)
    with(ans, list(xlim = ylim, ylim = c(0, 1),
                   dx = dy, dy = dx))
}



panel.ecdfplot <-
    function(x, f.value = NULL, type = "s",
             groups = NULL, qtype = 7,
             ref = TRUE,
             ...)
{
    if (ref)
    {
        reference.line <- trellis.par.get("reference.line")
        do.call(panel.abline, c(list(h = c(0, 1)), reference.line))
    }
    x <- as.numeric(x)
    distribution <- qunif
    nobs <- sum(!is.na(x))
    if (!is.null(groups))
    {
        panel.superpose(x, y = NULL,
                        f.value = f.value, type = type,
                        distribution = distribution, 
                        qtype = qtype,
                        groups = groups,
                        panel.groups = panel.ecdfplot,
                        ...)
    }
    else if (nobs)
    {
        if (is.null(f.value))
        {
            panel.xyplot(x = sort(x),
                         y = seq_len(nobs) / nobs,
                         type = type, 
                         ...)
        }
        else
        {
            p <- if (is.numeric(f.value)) f.value else f.value(nobs)
            panel.xyplot(x = quantile(x, p, names = FALSE, type = qtype, na.rm = TRUE),
                         y = distribution(p),
                         type = type, 
                         ...)
        }
    }
}



ecdfplot <- 
    function (x, data, ...) 
    UseMethod("ecdfplot")


ecdfplot.formula <- 
    function (x, data = NULL,
              prepanel = "prepanel.ecdfplot", 
              panel = "panel.ecdfplot",
              ylab = gettext("Empirical CDF"),
              ...) 
{
    ccall <- match.call()
    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(ecdfplot) ## for nice $call
    ccall$data <- data
    ccall$prepanel <- prepanel
    ccall$panel <- panel
    ccall$ylab <- ylab
    ccall[[1]] <- quote(lattice::densityplot)
    ans <- eval.parent(ccall)
    ans$call <- ocall
    ans
}


ecdfplot.numeric <- 
    function (x, data = NULL, xlab = deparse(substitute(x)), ...)
{
    ccall <- match.call()
    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(ecdfplot) ## for nice $call
    if (!is.null(ccall$data))
        warning("explicit 'data' specification ignored")
    ccall$data <- list(x = x)
    ccall$xlab <- xlab
    ccall$x <- ~x
    ccall[[1]] <- quote(latticeExtra::ecdfplot)
    ans <- eval.parent(ccall)
    ans$call <- ocall
    ans
}

##
## Copyright (c) 2008 Felix Andrews <felix@nfrac.org>
## Copyright (c) 2022 Deepayan Sarkar <deepayan.sarkar@r-project.org>
## GPL version 2 or newer

tileplot <-
    function(x, data = NULL, aspect = "iso",
             prepanel = "prepanel.default.xyplot",
             panel = "panel.voronoi", ...)
{
    foo <- levelplot(x, data = data, aspect = aspect,
                     panel = panel, prepanel = prepanel, ...)
    foo$call <- sys.call(sys.parent())
    foo
}

## panel function to draw Voronoi mosaic
panel.voronoi <-
    function(x, y, z, subscripts = TRUE, at = pretty(z),
             points = TRUE, border = "transparent",
             na.rm = FALSE, win.expand = 0.07,
             use.tripack = FALSE,
             backend = c("interp", "deldir"),
             ...,
             col.regions = regions$col, alpha.regions = regions$alpha)
{
    backend <- match.arg(backend)
    ## latticeExtra <= 0.6-29 offered the choice of using the non-free
    ## ACM licensed 'tripack' package, which was faster than the
    ## 'deldir' implementation. Later versions use the FOSS
    ## replacements in 'interp', and deprecates the 'use.tripack'
    ## argument. 'deldir' can still be used using 'backend = "deldir"'.
    if (!missing(use.tripack)) {
        warning("The 'use.tripack' argument is deprecated and ignored. See ?panel.voronoi")
    }
    ## find subset of points to use
    x0 <- x[subscripts]
    y0 <- y[subscripts]
    z0 <- z[subscripts]
    ## throw away NAs, but keep originals for panel.xyplot()
    ok <- complete.cases(x0, y0)
    if (na.rm) ok <- ok & !is.na(z0)
    x <- x0[ok]
    y <- y0[ok]
    z <- z0[ok]
    if (!any(is.finite(z))) return()
    ## strip duplicated locations, with warning
    dup <- duplicated(cbind(x, y))
    if (any(dup)) {
        warning(paste("Ignoring", sum(dup),
                      "cases of duplicated locations"))
        x <- x[!dup]
        y <- y[!dup]
        z <- z[!dup]
    }
    ## compute bounds
    data.rg <- list(x = extendrange(x, f = win.expand),
                    y = extendrange(y, f = win.expand))
    bounds <- c(data.rg$x, data.rg$y)
    #panel.rg <- lapply(current.panel.limits(), sort)
    #bounds <- c(max(panel.rg$x[1], data.rg$x[1]),
    #            min(panel.rg$x[2], data.rg$x[2]),
    #            max(panel.rg$y[1], data.rg$y[1]),
    #            min(panel.rg$y[2], data.rg$y[2]))
    ## check if any points in visible plot region
    #if (is.unsorted(bounds[1:2]))
    #    bounds[1:2] <- panel.rg$x
    #if (is.unsorted(bounds[3:4]))
    #    bounds[3:4] <- panel.rg$y
    if (backend == "interp") {
        xy <- data.frame(x = x, y = y)
        ## add dummy points to ensure that voronoi polygons are finite
        dummies <- data.frame(x = c(-1,-1,1,1), y = c(-1,1,-1,1)) * 10 * max(abs(xy))
        xy <- rbind(xy, dummies)
        tiles <- voronoi.polygons(voronoi.mosaic(xy, duplicate = "error"))
    } else {
        if (!requireNamespace("deldir", quietly = TRUE))
            stop("The 'deldir' backend requires the 'deldir' package to be installed.")
        ## NB: the 'rw' argument as subset of data is bad because
        ## need to take corresponding subset of z !
        ## (but not easy to work out what that is)

        #set <- ((bounds[1] < x) & (x < bounds[2]) &
        #        (bounds[3] < y) & (y < bounds[4]))
        #x <- x[set]
        #y <- y[set]
        #z <- z[set]
        tiles <- deldir::tile.list(deldir::deldir(x, y, rw = bounds))
        tiles <- lapply(tiles, function(p) as.data.frame(p[c("x", "y")]))
    }
    ## draw it as one composite polygon
    polydata <- do.call("rbind", tiles)
    regions <- trellis.par.get("regions")
    zcol <- level.colors(z, at, col.regions, colors = TRUE)
    grid.polygon(polydata[,1], polydata[,2],
                 id.lengths = sapply(tiles, nrow),
                 default.units = "native",
                 gp = gpar(fill = zcol, col = border,
                 alpha = alpha.regions))
    if (points) {
        panel.xyplot(x0, y0, ...)
    }
}

panel.levelplot.points <-
    function(x, y, z, subscripts = TRUE, at = pretty(z),
             shrink, labels, label.style, contour, region, ## (all ignored)
             pch = 21, col.symbol = "#00000044",
             ...,
             col.regions = regions$col,
             fill = NULL) ## (ignored)
{
    regions <- trellis.par.get("regions")
    zcol <- level.colors(z, at, col.regions, colors = TRUE)
    x <- x[subscripts]
    y <- y[subscripts]
    zcol <- zcol[subscripts]
    ## panel.xyplot does the work (can handle 'type' argument, etc)
    panel.xyplot(x, y, fill = zcol, pch = pch,
                 col.symbol = col.symbol, ...)
}

##
## Copyright (c) 2008 Felix Andrews <felix@nfrac.org>
##


panel.xblocks <- function(x, ...)
    UseMethod("panel.xblocks")

panel.xblocks.default <-
    function (x, y, ..., col = NULL, border = NA, 
              height = unit(1, "npc"),
              block.y = unit(0, "npc"), vjust = 0,
              name = "xblocks", gaps = FALSE,
              last.step = median(diff(tail(x))))
{
    if (is.function(y))
        y <- y(x)
    x <- as.numeric(x)
    if (length(x) == 0) return()
    if (is.unsorted(x, na.rm = TRUE))
        stop("'x' should be ordered (increasing)")
    if (is.na(last.step))
        last.step <- 0
    if (gaps) {
        .Deprecated(msg = "The 'gaps' argument is deprecated; use panel.xblocks(time(z), is.na(z))")
        y <- is.na(y)
    }
    ## Three cases:
    ## (1) If y is character, assume it gives the block colours
    ## -- unless 'col' is given, which over-rides it.
    ## (2) If y is logical, show blocks of TRUE values.
    ## (3) If y is numeric, show blocks of non-NA values.
    if (is.logical(y)) {
        y <- y
    } else if (is.numeric(y)) {
        y <- !is.na(y)
    } else {
        ## this will convert factor, Date, etc to character:
        y <- as.character(y)
    }
    ## Note: rle treats each NA as unique (does not combine runs of NAs)
    ## so we need to replace NAs with a temporary value.
    NAval <-
        if (is.character(y)) "" else FALSE
    y[is.na(y)] <- NAval
    ## find blocks (runs of constant values)
    yrle <- rle(y)
    ## substitute NA values back in
    blockCol <- yrle$values
    blockCol[blockCol == NAval] <- NA
    ## for logical series, col default comes from current theme
    if (is.logical(y) && is.null(col))
        col <- trellis.par.get("plot.line")$col
    ## set block colours if 'col' given
    if (length(col) > 0) {
        if (is.character(col))
            col[col == ""] <- NA
        ok <- !is.na(blockCol)
        blockCol[ok] <- rep(col, length = sum(ok)) ## rep to avoid warnings
    }
    ## work out block geometry
    idxBounds <- cumsum(c(1, yrle$lengths))
    idxStart <- head(idxBounds, -1)
    idxEnd <- tail(idxBounds, -1)
    idxEnd[length(idxEnd)] <- length(y)
    blockStart <- x[idxStart]
    blockEnd <- x[idxEnd]
    blockEnd[length(blockEnd)] <- tail(blockEnd, 1) + last.step
    blockWidth <- blockEnd - blockStart
    ## draw it
    grid::grid.rect(x = blockStart, width = blockWidth,
                    y = block.y, height = height,
                    hjust = 0, vjust = vjust,
                    default.units = "native", name = name,
                    gp = gpar(fill = blockCol, col = border, ...))
}

panel.xblocks.ts <-
    function(x, y = x, ...)
{
    if (!is.function(y)) y <- as.vector(y)
    panel.xblocks(as.vector(time(x)), y, ...)
}

panel.xblocks.zoo <-
    function(x, y = x, ...)
{
    if (!is.function(y)) y <- zoo::coredata(y)
    panel.xblocks(zoo::index(x), y, ...)
}

## Similar to panel.xyplot, except that (x,y) data are indicated by an
## image rather than standard plotting character. The image URLs must
## be provided by 


## Support only png and jpeg for now.

url2raster <- function(src)
{
    ext <- tail(strsplit(src, split = ".", fixed = TRUE)[[1]], 1)
    if (file.exists(src)) file <- src
    else 
    {
        file <- tempfile(fileext = paste0(".", ext))
        download.file(src, destfile = file, mode = "wb", quiet = TRUE)
        on.exit(unlink(file))
    }
    readWith <- switch(ext,
                       png = list(readPNG, readJPEG),
                       list(readJPEG, readPNG))
    ## Try best guess first. If it fails, try the other
    r <- try(readWith[[1]](file, native = TRUE), silent = TRUE)
    if (inherits(r, "try-error"))
        r <- try(readWith[[2]](file, native = TRUE), silent = TRUE)
    if (inherits(r, "try-error"))
        stop("'%s' does not appear to be a PNG or JPEG file.", src)
    r
}


panel.xyimage <-
    function(x, y, 
             subscripts,
             groups = NULL,
             pch = NULL,
             cex = 1,
             ...,
             grid = FALSE, abline = NULL)
{
    if (all(is.na(x) | is.na(y))) return()
    if (!is.character(pch))
        stop("'pch' must be a character vector giving path(s) or URL(s) of PNG or JPEG files.")
    if (!identical(grid, FALSE))
    {
        if (!is.list(grid))
            grid <- switch(as.character(grid),
                           "TRUE" = list(h = -1, v = -1, x = x, y = y),
                           "h" = list(h = -1, v = 0, y = y),
                           "v" = list(h = 0, v = -1, x = x),
                           list(h = 0, v = 0))
        do.call(panel.grid, grid)
    }
    if (!is.null(abline))
    {
        if (is.numeric(abline)) abline <- as.list(abline)
        do.call(panel.abline, abline)
    }
    if (is.null(groups))
    {
        pch.raster <- url2raster(pch[1])
        grid.raster(x, y, image = pch.raster,
                    width = unit(cex * 10, "mm"),
                    height = unit(cex * 10, "mm"),
                    default.units = "native")
    }
    else
    {
        groups <- as.factor(groups)
        cex <- rep(cex, length = nlevels(groups))
        pch <- rep(pch, length = nlevels(groups))
        pch.raster <- lapply(pch, url2raster)
        g <- as.numeric(groups)[subscripts]
        ug <- unique(g)
        for (i in ug)
        {
            w <- (g == i)
            grid.raster(x[w], y[w], image = pch.raster[[i]],
                        width = unit(cex[i] * 8, "mm"),
                        height = unit(cex[i] * 8, "mm"),
                        default.units = "native")
        }
    }
}

## not exported; for use in c.trellis and doubleYScale
mergeTrellisLegends <- function(legend, legend2, vertical = NULL)
{
    legend <- as.list(legend)
    legend2 <- as.list(legend2)
    for (space in c("top", "bottom", "left", "right")) {
        if (!is.null(legend2[[space]])) {
            if (is.null(legend[[space]])) {
                ## no conflict
                legend[[space]] <- legend2[[space]]
            } else {
                v <- vertical
                if (is.null(v))
                    v <- space %in% c("left", "right")
                legend[[space]] <-
                    list(fun = "mergedTrellisLegendGrob",
                         args = list(a = legend[[space]], b = legend2[[space]],
                         vertical = v))
            }
        }
    }
    legend <- c(legend, legend2[names(legend2) == "inside"])
    legend
}

## exported, to be called at plot time from 'legend'
mergedTrellisLegendGrob <-
    function(a, b, vertical = FALSE, border = NULL)
{
    if (is.null(a))
        return(b)
    if (is.null(b))
        return(a)
    if (!inherits(a$fun, "grob")) {
        ## fun <- a$fun
        if (is.character(a$fun)) a$fun <- as.symbol(a$fun)
        a$fun <- eval(as.call(c(a$fun, a$args)), getNamespace("lattice"))
    }
    if (!inherits(b$fun, "grob")) {
        if (is.character(b$fun)) b$fun <- as.symbol(b$fun)
        b$fun <- eval(as.call(c(b$fun, b$args)), getNamespace("lattice"))
    }
    g <- frameGrob(name = "mergedLegend")
    g <- packGrob(g, a$fun, side = if (vertical) "top" else "left", border = border)
    g <- packGrob(g, b$fun, side = if (vertical) "bottom" else "right", border = border)
    g
}

## based on the stat_quantile() function in ggplot2 package.

panel.quantile <-
    function(x, y, form = y ~ x, method = "rq", ...,
             tau = 0.5,
             ci = FALSE, ci.type = "default", level = 0.95,
             n = 100,
             col = plot.line$col, col.se = col,
             lty = plot.line$lty, lwd = plot.line$lwd,
             alpha = plot.line$alpha,
             alpha.se = 0.25, border = NA,
             superpose = FALSE,
             ## ignored (do not pass to method()):
             subscripts, group.number, group.value,
             type, col.line, col.symbol, fill,
             pch, cex, font, fontface, fontfamily)
{
    if (method %in% c("rq", "rqss") &&
        !requireNamespace("quantreg", quietly=TRUE))
        stop("The 'quantreg' package is required for methods \"rq\" and \"rqss\".")
    methodFun <- switch(method,
                        rq = quantreg::rq,
                        rqss = quantreg::rqss,
                        method)
    plot.line <- trellis.par.get("plot.line")
    if (!missing(col.line)) col <- col.line
    ## allow 'form' to be passed as the first argument
    missing.x <- missing(x)
    if (!missing.x && inherits(x, "formula")) {
        form <- x
        missing.x <- TRUE
    }
    ## use 'x' and 'y' if given
    ## otherwise try to find them in the formula environment
    if (missing.x) x <- environment(form)$x
    if (missing(y)) y <- environment(form)$y
    ok <- is.finite(x) & is.finite(y)
    if (sum(ok) < 1) return()
    x <- as.numeric(x)[ok]
    y <- as.numeric(y)[ok]
    mod <- do.call(method,
                   c(alist(form, tau = tau, data = list(x = x, y = y)),
                     list(...)))
    xseq <- seq(min(x), max(x), length = n)
    pred <- predict(mod, data.frame(x = xseq),
                    interval = if (ci) "confidence" else "none",
                    type = ci.type, level = level)
    pred <- as.matrix(pred)
    if (ci && ncol(pred) > 1) {
        panel.polygon(x = c(xseq, rev(xseq)),
                      y = c(pred[,"lower"], rev(pred[,"higher"])),
                      col = col.se, alpha = alpha.se, border = border)
        pred <- pred[, "fit", drop = FALSE]
    }
    if (superpose) {
        for (i in 1:NCOL(pred)) {
            line <- Rows(trellis.par.get("superpose.line"), i)
            panel.lines(xseq, pred[,i], col = line$col, alpha = line$alpha,
                        lty = line$lty, lwd = line$lwd)
        }
    } else {
        apply(pred, 2, panel.lines, x = xseq, col = col, alpha = alpha,
              lty = lty, lwd = lwd)
    }
}

## moving quantiles
#L.rollquantile <- function(probs = c(0.05, 0.5, 0.95), width,
#                           alpha = 0.25, ...)
#{
#    stopifnot(require("zoo"))
#    z <- zoo(y, x)
#    pred <- rollapply(z, width = width, quantile, probs = probs,
#                      na.rm = TRUE)
#    apply(pred, 2, panel.lines, x = time(pred), col = col)
#}

## quantile regression with smoothness by mgcv
#  L.quantile <- function(probs = c(0.05, 0.5, 0.95), n = 100,
#    alpha = 0.25, ss = FALSE, lambda = NULL, ...)
#  {
#      mod <- rqss(y ~ rqss(x), tau = probs, lambda = lambda)

### Copyright 2000-2004 Deepayan Sarkar <deepayan@stat.wisc.edu>,
###
### This file is part of the lattice library for R.  It is made
### available under the terms of the GNU General Public License,
### version 2, or at your option, any later version, incorporated
### herein by reference.
###
### This program is distributed in the hope that it will be
### useful, but WITHOUT ANY WARRANTY; without even the implied
### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
### PURPOSE.  See the GNU General Public License for more
### details.
###
### You should have received a copy of the GNU General Public
### License along with this program; if not, write to the Free
### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
### MA 02111-1307, USA


## .noGenerics <- TRUE
.noGenerics <- FALSE

## .First.lib can be used if the NAMESPACE file is missing.  This is
## useful during development, thanks to C-c C-l in Emacs/ESS. It won't
## be used if NAMESPACE is present.


## .First.lib <- function(lib, pkg) 
## {
##     require(lattice)
## }

## also available in lattice, but not exported
is.characterOrExpression <- function (x) is.character(x) || is.expression(x)

.arrayIndices <- function(d, i)
    ## Suppose we have an array 'x' with dimension 'd'.  We can index
    ## 'x' in two different ways: x[i] or x[i_1, i_2, ..., i_d].
    ## Here, we are given 'i', and want to compute i_1, i_2, ..., i_d.
{
    ## Here's what we are doing:
    ## For length(d) == 3, note that (for 0-based indexing)
    ## 
    ## i1 = (i mod d[1])
    ## i2 = (i mod d[1] * d[2]) div d[1]
    ## i3 = (i mod d[1] * d[2] * d[3]) div d[1] * d[2]
    n <- length(d)
    ans <- vector(mode = "list", length = n)
    for (k in seq_along(ans))
    {
        ans[[k]] <- 1L + (((i-1L) %% prod(head(d, k)))
                          %/%
                          prod(head(d, k-1L)))
    }
    ans
}


combineLimits <-
    function(x, margin.x = 2L, margin.y = 1L,
             extend = TRUE, adjust.labels = TRUE)
{
    if (length(dim(x)) == 1L)
        warning("Only one conditioning variable; nothing interesting will happen.")
    indices <- .arrayIndices(dim(x), seq_len(prod(dim(x))))
    ## For regular `numeric' scales, all we need is to modify
    ## $[xy].scales.  But for `factor' scales, we need to leave
    ## $[xy].scales alone, and instead modify $[xy].used.at and $[xy].num.limit
    modifyLimits <- function(limits, margin, ext)
    {
        limits <- array(limits, dim = dim(x))
        for (i in seq_len(prod(dim(x))))
        {
            ## index.combine <- index.entry <- Rows(indices, i)
            index.combine <- Rows(indices, i)
            index.combine[margin] <- list(TRUE)
            ## limits[[i]] <-
            ##     range(do.call("[", c(list(limits), index.combine)), finite = TRUE)
            
            li <- unlist(do.call("[", c(list(limits), index.combine)))
            limits[[i]] <- if(all(is.na(li))) li else range(li, finite = TRUE)
        }
        if (ext) lapply(limits, lattice:::extend.limits)
        else limits
    }
    modifyUsed <- function(used.at, margin)
    {
        used.at <- array(used.at, dim = dim(x))
        for (i in seq_len(prod(dim(x))))
        {
            index.combine <- Rows(indices, i)
            index.combine[margin] <- list(TRUE)
            li <- unlist(do.call("[", c(list(used.at), index.combine)))
            used.at[[i]] <- sort(unique(li))
        }
        used.at
    }
    if (x$x.scales$relation != "free" && x$y.scales$relation != "free")
        warning("Function only has effect for scales with 'relation=\"free\"'.")
    if (x$x.scales$relation == "free" && length(margin.x))
    {
        if (is.characterOrExpression(x$x.limits[[1]]))
        {
            x$x.used.at <- modifyUsed(x$x.used.at, margin.x)
            x$x.num.limit <- modifyLimits(x$x.num.limit, margin.x, ext = FALSE)
        }
        else
            x$x.limits <- modifyLimits(x$x.limits, margin.x, ext = extend)
    }
    if (x$y.scales$relation == "free" && length(margin.y))
    {
        if (is.characterOrExpression(x$y.limits[[1]]))
        {
            x$y.used.at <- modifyUsed(x$y.used.at, margin.y)
            x$y.num.limit <- modifyLimits(x$y.num.limit, margin.y, ext = FALSE)
        }
        else
            x$y.limits <- modifyLimits(x$y.limits, margin.y, ext = extend)
    }
    if (adjust.labels)
    {
        ## Drop all but left/bottom-most labels, and set space to 0
        ## for those.  Needs to know layout, and will set it unless
        ## already set.
        npackets <- prod(dim(x))
        par.settings <- if (is.null(x$par.settings)) list() else x$par.settings
        if (is.null(x$layout))
            x$layout <-
                if (length(dim(x)) == 1L) c(dim(x), 1)
                else dim(x)[1:2]
        else if (!isTRUE(all.equal(x$layout[1:2], dim(x)[1:2])))
        {
            warning("'layout' does not match dimensions; displayed scales may be wrong.")
        }
        if (any(is.na(x$layout) | x$layout == 0))
            stop("'layout' must explicitly determine number of rows and columns")
        if (x$x.scales$relation == "free" && length(margin.x))
        {
            ## change x-scales
            if (is.list(x$x.scales$at))
            {
                warning("Explicit per-panel tick mark locations ignored")
                x$x.scales$at <- FALSE
            }
            page.at <- 
                if (x$as.table)
                    rep(list(NULL, x$x.scales$at),
                        c(x$layout[1] * (x$layout[2]-1), x$layout[1]))
                else
                    rep(list(x$x.scales$at, NULL),
                        c(x$layout[1], x$layout[1] * (x$layout[2]-1)))
            x$x.scales$at <- rep(page.at, length.out = npackets)
            par.settings <-
                if (x$as.table)
                    modifyList(par.settings,
                               list(layout.heights =
                                    list(axis.panel = rep(c(0, 1), c(x$layout[2]-1, 1)))))
                else 
                    modifyList(par.settings,
                               list(layout.heights =
                                    list(axis.panel = rep(c(1, 0), c(1, x$layout[2]-1)))))
        }
        if (x$y.scales$relation == "free" && length(margin.y))
        {
            ## change y-scales
            if (is.list(x$y.scales$at))
            {
                warning("Explicit per-panel tick mark locations ignored")
                x$y.scales$at <- FALSE
            }
            page.at <- rep(list(TRUE, NULL), c(1, x$layout[1]-1))
            x$y.scales$at <- rep(page.at, length.out = npackets)
            par.settings <-
                modifyList(par.settings,
                           list(layout.widths =
                                list(axis.panel = rep(c(1, 0), c(1, x$layout[1]-1)))))
        }
        x$par.settings <- par.settings
    }
    x
}

##
## Copyright (c) 2007 Felix Andrews <felix@nfrac.org>
## GPL version 2 or newer

is.categorical <- function (x)
{
    is.factor(x) || is.shingle(x) || is.character(x) || is.logical(x)
}

marginal.plot <-
    function(x,
             data = NULL,
             groups = NULL,
             reorder = !is.table(x),
             plot.points = FALSE,
             ref = TRUE, cut = 0,
             origin = 0, #ylim = c(0, NA), this only supported in R >= 2.11
             xlab = NULL, ylab = NULL,
             type = c("p", if (is.null(groups)) "h"),
             ...,
             subset = TRUE,
             as.table = TRUE,
             subscripts = TRUE,
             default.scales = list(
               relation = "free",
               abbreviate = TRUE, minlength = 5,
               rot = 30, cex = 0.75, tick.number = 3,
               y = list(draw = FALSE)),
             layout = NULL,
             lattice.options = list(
               layout.heights = list(
                 axis.xlab.padding = list(x = 0),
                 xlab.key.padding = list(x = 0))))
{
    if (is.table(data))
        data <- as.data.frame(data)
    ## assume first term of formula is the data object; ignore rest
    if (inherits(x, "formula"))
        x <- eval(x[[2]], data, environment(x))
    ## x must be either a data.frame or a table
    if (!is.data.frame(x) && !is.table(x))
        x <- as.data.frame(x)
    ## groups and subset are subject to non-standard evaluation:
    groups <- eval(substitute(groups), data, parent.frame())
    ## note unusual cases e.g.
    ## evalq(marginal.plot(dat, subset = complete.cases(dat)), myEnv)
    subset <- eval(substitute(subset), data, parent.frame())
    ## apply subset
    if ((length(subset) > 0) && !isTRUE(subset)) {
        x <- x[subset,]
        if (!is.null(groups))
            groups <- groups[subset]
    }
    ## divide into categoricals and numerics
    if (is.table(x)) {
        iscat <- TRUE
    } else {
        iscat <- sapply(x, is.categorical)
    }
    ## reorder factor levels
    if (reorder) {
        if (is.table(x)) {
            x <- reorderTableByFreq(x)
        } else {
            for (nm in names(x)[iscat]) {
                val <- x[[nm]]
                if (is.character(val))
                    x[[nm]] <- factor(val)
                if (!is.ordered(val) &&
                    !is.shingle(val) &&
                    nlevels(val) > 1)
                {
                    x[[nm]] <- reorder(val, val, function(z) -length(z))
                }
            }
        }
    }
    if (any(iscat)) {
        ## handle categorical variables
        ## make a list of dotplot trellis objects
        if (is.table(x)) {
            margins <- seq(length = length(dim(x)))
            names(margins) <- names(dimnames(x))
        } else {
            margins <- which(iscat)
            names(margins) <- colnames(x)[iscat]
        }
        dotobjs <-
            lapply(margins,
                   function(i)
               {
                   if (is.table(x)) {
                       nm <- names(dimnames(x))[i]
                       nm <- deparse(as.symbol(nm), backtick = TRUE)
                       form <- paste("Freq ~", nm)
                       if (!is.null(groups))
                           form <- paste(form, "+ groups")
                       tab <- xtabs(as.formula(form), x)
                   } else {
                       if (!is.null(groups)) {
                           tab <- table(Value = x[[i]], groups = groups)
                       } else {
                           tab <- table(Value = x[[i]])
                       }
                   }
                   dotplot(tab, horizontal = FALSE,
                           groups = !is.null(groups),
                           subscripts = TRUE,
                           ...,
                           type = type,
                           origin = origin, #ylim = ylim,
                           as.table = as.table,
                           default.scales = default.scales,
                           lattice.options = lattice.options,
                           xlab = xlab, ylab = ylab)
               })
        ## merge the list of trellis objects into one
        catobj <- do.call("c", c(dotobjs, merge.legends = FALSE))
        catobj$layout <- layout
        catobj$call <- match.call()
    }
    if (any(!iscat)) {
        ## handle numeric variables
        ## construct formula with all numeric variables
        nms <- names(x)[!iscat]
        symbolStr <- function(nm)
            deparse(as.symbol(nm), backtick = TRUE)
        nms <- sapply(nms, symbolStr)
        numform <- paste("~", paste(nms, collapse = " + "))
        numobj <-
            densityplot(as.formula(numform), x, outer = TRUE,
                        subscripts = TRUE,
                        groups = groups,
                        ...,
                        plot.points = plot.points,
                        ref = ref, cut = cut, #ylim = ylim,
                        as.table = as.table,
                        default.scales = default.scales,
                        lattice.options = lattice.options,
                        xlab = xlab, ylab = ylab)
        ## set strip name if only one panel
        if (prod(dim(numobj)) == 1)
            rownames(numobj) <- names(x)[!iscat]
        numobj$call <- match.call()
        numobj$layout <- layout
    }
    if (all(iscat)) {
        obj <- catobj
    } else if (all(!iscat)) {
        obj <- numobj
    } else {
        ## if there are both categoricals and numerics,
        ## merge the trellis objects; keep original var order
        reIndex <- order(c(which(iscat), which(!iscat)))
        obj <- update(c(catobj, numobj, merge.legends = FALSE),
                      index.cond = list(reIndex), layout = layout)
        ## force strips when only one panel in each object
        if (identical(obj$strip, FALSE))
            obj$strip <- "strip.default"
    }
    obj$call <- sys.call(sys.parent())
    obj
}

reorderTableByFreq <- function(x)
{
    stopifnot(is.table(x))
    df <- as.data.frame(x)
    i <- which(names(df) == "Freq")
    df[-i] <- lapply(df[-i], reorder, - df$Freq)
    xtabs(Freq ~ ., df)
}

##
## Copyright (c) 2009 Felix Andrews <felix@nfrac.org>
## GPL version 2 or newer

panel.ablineq <-
    function(a = NULL, b = 0,
             h = NULL, v = NULL,
             reg = NULL, coef = NULL,
             pos = if (rotate) 1 else NULL,
             offset = 0.5, adj = NULL,
             at = 0.5,
             x = NULL, y = NULL,
             rotate = FALSE, srt = 0,
             label = NULL,
             varNames = alist(y = y, x = x),
             varStyle = "italic",
             fontfamily = "serif",
             digits = 3,
             r.squared = FALSE, sep = ", ", sep.end = "",
             col, col.text = add.text$col,
             col.line = add.line$col,
             ..., reference = FALSE)
{
    if (!is.null(label)) varStyle <- NULL
    ## work out colours for text and line
    add.text <- trellis.par.get("add.text")
    add.line <- if (reference)
        trellis.par.get("reference.line")
    else trellis.par.get("add.line")
    if (!missing(col) && missing(col.line))
        col.line <- col
    if (!missing(col) && missing(col.text))
        col.text <- col
    ## draw the line
    panel.abline(a = a, b = b, h = h, v = v, reg = reg, coef = coef,
                 col = col.line, ..., reference = reference)
    ## extract r.squared from model object if any
    if (!is.null(reg)) {
        a <- reg
    }
    if (isTRUE(r.squared)) {
        if (is.object(a) || is.list(a)) {
            r.squared <- round(summary(a)$r.squared, digits)
        } else {
            warning("r.squared = TRUE requires a model object")
        }
    }
    ## work out equation coefficients
    ## the following copied from lattice::panel.abline
    if (is.object(a) || is.list(a)) {
        p <- length(coefa <- as.vector(coef(a)))
        if (p > 2)
            warning("only using the first two of ", p, "regression coefficients")
        islm <- inherits(a, "lm")
        noInt <- if (islm)
            !as.logical(attr(stats::terms(a), "intercept"))
        else p == 1
        if (noInt) {
            a <- 0
            b <- coefa[1]
        }
        else {
            a <- coefa[1]
            b <- if (p >= 2)
                coefa[2]
            else 0
        }
    }
    if (!is.null(coef)) {
        if (!is.null(a))
            warning("'a' and 'b' are overridden by 'coef'")
        a <- coef[1]
        b <- coef[2]
    }
    if (length(h <- as.numeric(h)) > 0) {
        if (!is.null(a))
            warning("'a' and 'b' are overridden by 'h'")
        a <- h[1]
        b <- 0
    }
    if (length(a) > 1) {
        b <- a[2]
        a <- a[1]
    }
    ## construct the equation label
    if (length(as.numeric(v)) > 0) {
        ## vertical line (special case)
        if (!is.null(a))
            warning("'a' and 'b' are overridden by 'v'")
        if (is.null(x))
            x <- v[1]
        if (is.null(y))
            y <- convertY(unit(at, "npc"), "native", TRUE)
        if (is.null(label)) {
            v <- signif(v[1], digits)
            varNames <- c(as.list(varNames), v = v)
            label <- substitute(x == v, varNames)
        }
    } else {
        ## normal a+bx line
        if (is.null(x)) {
            ## work out start and end x values of visible line
            xlim <- current.panel.limits()$xlim
            ylim <- current.panel.limits()$ylim
            if (b == 0) {
                xx <- xlim
            } else {
                xx <- range((ylim - a) / b)
                xx <- pmin(max(xlim), xx)
                xx <- pmax(min(xlim), xx)
            }
            ## x position as fractional distance along line
            x <- min(xx) + at * abs(diff(xx))
        }
        if (is.null(y))
            y <- a + b * x
        if (is.null(label)) {
            a <- round(a, digits)
            b <- round(b, digits)
            varNames <- c(as.list(varNames), a = a, b = b)
            if (b == 0) {
                label <- substitute(y == a, varNames)
            } else if (a == 0) {
                label <- substitute(y == b * x, varNames)
            } else if (b > 0) {
                label <- substitute(y == a + b * x, varNames)
            } else {
                varNames$b <- abs(b)
                label <- substitute(y == a - b * x, varNames)
            }
        }
    }
    if (rotate) {
        if (length(as.numeric(v)) > 0) {
            srt <- 90
        } else {
            ## aspect ratio with respect to native coordinates
            asp <- with(lapply(current.panel.limits(), diff), ylim / xlim)
            ## aspect ratio of panel at *current* device size
            asp.cm <- with(lapply(current.panel.limits("cm"), diff), ylim / xlim)
            grad <- b * (asp.cm / asp)
            srt <- 180 * atan(grad) / pi
        }
    }
    if (is.numeric(r.squared)) {
        ## add R^2 = ... to label
        rsq.expr <- substitute(italic(R)^2 == z,
                            list(z = r.squared))
        label <- call("paste", label, sep, rsq.expr, sep.end)
    }
    ## wrap 'varStyle' function (or multiple functions) around label expression
    if (!is.null(varStyle)) {
        while (length(varStyle) > 0) {
            label <- call(varStyle[1], label)
            varStyle <- varStyle[-1]
        }
    }
    panel.text(x = x, y = y, labels = label,
               pos = pos, offset = offset, adj = adj,
               fontfamily = fontfamily, srt = srt,
               col = col.text, ...)
}

panel.lmlineq <-
    function(x, y, ...)
{
    if (length(x) > 1)
        panel.ablineq(lm(as.numeric(y) ~ as.numeric(x)), ...)
}

## Copyright (C) 2007 Felix Andrews <felix@nfrac.org>
## GPL version 2 or newer


xyplot.list <-
    function(x, data = NULL, ..., FUN = xyplot,
             y.same = TRUE, x.same = NA, layout = NULL,
             merge.legends = FALSE)
{
    if (length(x) == 0) return(NULL)
    ## NOTE lapply here causes problems with eval.parent and `...` later.
    #objs <- lapply(x, FUN, data = data, ...)
    objs <- vector(mode = "list", length = length(x))
    for (i in as.numeric(seq_along(x))) {
        ## this is what we had previously, but it seemed to cause failures
        ## in complex call structures (e.g. pch=pch ==> object 'pch' not found)
        ## (use substitute to get reasonable ylab)
        #objs[[i]] <- eval.parent(substitute(FUN(x[[i]], data = data, ...)))
        ## check for 'data' to avoid warnings in e.g. qqmath.numeric
        objs[[i]] <-
            if (!is.null(data)) FUN(x[[i]], data = data, ...) else FUN(x[[i]], ...)
    }
    names(objs) <- names(x)
    ok <- unlist(lapply(objs, inherits, "trellis"))
    if (any(!ok))
        stop("FUN returned object of class ",
             toString(class(objs[[ which(!ok)[1] ]])),
             ", not trellis.")
    ans <- do.call("c", c(objs,
                   list(x.same = x.same, y.same = y.same,
                        layout = layout, merge.legends = merge.legends)))
    ans$call <- match.call()
    ans
}

c.trellis <-
    function(..., x.same = NA, y.same = NA,
             layout = NULL, merge.legends = FALSE,
             recursive = FALSE)
{
    objs <- list(...)
    if (length(objs) == 0) return(NULL)
    if (length(objs) == 1) {
        ## only one object
        obj <- objs[[1]]
        ## set dimnames if given and only one panel
        if (!is.null(names(objs)) && (prod(dim(obj)) == 1))
        {
            rownames(obj) <- names(objs)
            if (identical(obj$strip, FALSE)) obj$strip <- "strip.default"
        }
        return(obj)
    }
    if (length(objs) > 2) {
        ## merge first two objects, and call again
        first2Merged <-
            do.call("c.trellis", c(objs[1:2],
                           list(x.same = x.same, y.same = y.same,
                                merge.legends = merge.legends)))
        return(do.call("c.trellis", c(list(first2Merged), objs[-(1:2)],
                              list(x.same = x.same, y.same = y.same,
                                   layout = layout, merge.legends = merge.legends))))
    }
    ## now exactly 2 objects
    obj1 <- objs[[1]]
    obj2 <- objs[[2]]
    ## number of packets in object, i.e. offset
    NPACK1 <- prod(dim(obj1))
    NPACK2 <- prod(dim(obj2))
    ## first panel function
    panel <- obj1$panel
    PANEL1 <- if (is.function(panel)) panel
    else if (is.character(panel)) get(panel)
    else eval(panel)
    ## second panel function
    panel <- obj2$panel
    PANEL2 <- if (is.function(panel)) panel
    else if (is.character(panel)) get(panel)
    else eval(panel)
    obj1$panel <- function(...) {
        if (packet.number() <= NPACK1)
            PANEL1(...)
        else PANEL2(...)
    }
    ## TODO: treat 'prepanel' the same way as 'panel'?
    ## flatten the trellis objects (make 1 dimensional)
    flatIC <- function(index.cond) {
        dim <- sapply(index.cond, length)
        ic <- do.call(expand.grid, index.cond)
        if (length(dim) >= 2)
            ic[,2] <- (ic[,2] - 1) * dim[1]
        if (length(dim) >= 3)
            ic[,3] <- (ic[,3] - 1) * prod(dim[1:2])
        rowSums(ic)
    }
    flatCL <- function(condlevels, newname=NULL) {
        ## paste names of variables to their values (for strips)
        #for (i in seq_along(condlevels))
        #    condlevels[[i]] <- paste(names(condlevels)[i], ## may be NULL
        #                             condlevels[[i]], sep=" = ")
        ## convert shingle levels to character strings
        condlevels <- lapply(condlevels, as.character)
        cl <- do.call(expand.grid, condlevels)
        cl <- apply(cl, 1, paste, sep=" / ")
        if (!is.null(newname) && (nchar(newname) > 0)) {
            if (length(cl) == 1) cl <- newname
            else cl <- paste(newname, cl, sep=": ")
        }
        cl
    }
    obj1$index.cond <- list(c(flatIC(obj1$index.cond),
                              flatIC(obj2$index.cond) + NPACK1))
    obj1$condlevels <- list(c(flatCL(obj1$condlevels, names(objs)[1]),
                              flatCL(obj2$condlevels, names(objs)[2])))
    obj1$perm.cond <- 1

    ## make scales nominally "free", so they look like original objects
    makeFreeScales <- function(obj, npack, x.y)
    {
        obj[[paste(x.y, "scales", sep=".")]]$relation <- "free"
        .limits <- paste(x.y, "limits", sep=".")
        .num.limit <- paste(x.y, "num.limit", sep=".")
        .used.at <- paste(x.y, "used.at", sep=".")
        if (is.null(obj[[.limits]])) obj[[.limits]] <- NA
        if (is.null(obj[[.num.limit]])) obj[[.num.limit]] <- NA
        if (is.null(obj[[.used.at]])) obj[[.used.at]] <- NA
        if (!is.list(obj[[.limits]])) {
            obj[[.limits]] <- rep(list(obj[[.limits]]), length=npack)
            obj[[.num.limit]] <- rep(list(obj[[.num.limit]]), length=npack)
            obj[[.used.at]] <- rep(list(obj[[.used.at]]), length=npack)
        }
        obj
    }
    ## set relations to "free" if the first object has "free" scales
    ## or if the limits in the two objects are not identical
    xlimItems <- c("x.limits", "x.num.limit", "x.used.at")
    ylimItems <- c("y.limits", "y.num.limit", "y.used.at")
    if (is.na(x.same)) {
        x.same <- FALSE
        if (!is.list(obj1$x.limits) &&
            identical(unclass(obj1)[xlimItems],
                      unclass(obj2)[xlimItems]))
            x.same <- NA
    }
    if (is.na(y.same)) {
        y.same <- FALSE
        if (!is.list(obj1$y.limits) &&
            identical(unclass(obj1)[ylimItems],
                      unclass(obj2)[ylimItems]))
            y.same <- NA
    }
    if (identical(x.same, FALSE) ||
        is.list(obj1$x.limits))
    {
        obj1 <- makeFreeScales(obj1, npack=NPACK1, x.y="x")
        obj2 <- makeFreeScales(obj2, npack=NPACK2, x.y="x")
        obj1$x.limits <- c(obj1$x.limits, obj2$x.limits)
        obj1$x.num.limit <- c(obj1$x.num.limit, obj2$x.num.limit)
        obj1$x.used.at <- c(obj1$x.used.at, obj2$x.used.at)
    }
    if (identical(y.same, FALSE) ||
        is.list(obj1$y.limits))
    {
        obj1 <- makeFreeScales(obj1, npack=NPACK1, x.y="y")
        obj2 <- makeFreeScales(obj2, npack=NPACK2, x.y="y")
        obj1$y.limits <- c(obj1$y.limits, obj2$y.limits)
        obj1$y.num.limit <- c(obj1$y.num.limit, obj2$y.num.limit)
        obj1$y.used.at <- c(obj1$y.used.at, obj2$y.used.at)
    }

    ## merge common panel args into panel.args
    ## check for identical() args
    commonNames <- intersect(names(obj1$panel.args.common),
                             names(obj2$panel.args.common))
    identNames <- commonNames[unlist(lapply(commonNames, function(x)
                                     identical(obj1$panel.args.common[[x]],
                                               obj2$panel.args.common[[x]])))]
    obj1Common <- names(obj1$panel.args.common) %in% identNames
    obj2Common <- names(obj2$panel.args.common) %in% identNames
    obj1$panel.args <- lapply(obj1$panel.args, c,
                              obj1$panel.args.common[!obj1Common])
    obj2$panel.args <- lapply(obj2$panel.args, c,
                              obj2$panel.args.common[!obj2Common])
    obj1$panel.args.common <- obj1$panel.args.common[obj1Common]
    ## the actual data
    obj1$panel.args <- c(obj1$panel.args, obj2$panel.args)
    obj1$packet.sizes <- c(obj1$packet.sizes, obj2$packet.sizes)
    ## some prepanel functions require a 'subscripts' argument in each 'panel.args'
    fargNames <- function(f) if (is.null(f)) NULL else names(formals(f))
    if ("subscripts" %in% c(fargNames(obj1$prepanel.default), fargNames(obj1$prepanel)))
    {
        for (i in seq_along(obj1$panel.args))
        {
            if (!("subscripts" %in% names(obj1$panel.args[[i]])))
            {
                obj1$panel.args[[i]]$subscripts <- TRUE
            }
        }
    }

    ## recalculate panel limits using all data
    if ((isTRUE(x.same) || isTRUE(y.same))) {
        scalesSpec <- list()
        if (isTRUE(x.same)) scalesSpec$x$relation <- "same"
        if (isTRUE(y.same)) scalesSpec$y$relation <- "same"
        obj1 <- update(obj1, scales = scalesSpec)
    }

    if (identical(obj1$strip.left, FALSE)) {
        ## turn strips on if either object has strips, or names were given
        if (identical(obj1$strip, FALSE) &&
            !identical(obj2$strip, FALSE))
            obj1$strip <- obj2$strip
        if (identical(obj1$strip, FALSE) &&
            !is.null(names(objs)))
            obj1$strip <- "strip.default"
    }

    ## TODO: can use 'par.settings' from obj2 only for obj2 panels?
    obj1$par.settings <- modifyList(as.list(obj2$par.settings),
                                    as.list(obj1$par.settings))
    if (merge.legends)
        obj1$legend <- mergeTrellisLegends(obj1$legend, obj2$legend)
    
    obj1$layout <- layout
    obj1$call <- call("c", obj1$call, obj2$call,
                      x.same = x.same, y.same = y.same,
                      layout = layout)
    ## need this to allow further calls to update() to insert arguments:
    obj1$call <- call("update", obj1$call)
    obj1
}

##
## Copyright (c) 2008 Felix Andrews <felix@nfrac.org>
## GPL version 2 or newer

as.layer <- function(x, ...)
    UseMethod("as.layer")

as.layer.layer <- function(x, ...)
    x

layer <-
    function(..., data = NULL,
             magicdots = TRUE, exclude = NULL,
             packets = NULL, 
             rows = NULL, columns = NULL,
             groups = NULL,
             style = NULL, force = FALSE,
             theme = if (force) trellis.par.get() else NULL,
             under = FALSE, superpose = FALSE)
{
    ## set layer to quoted expressions in `...`
    foo <- eval(substitute(expression(...)))
    if (magicdots) {
        ## The dots `...` are magic:
        ## pass on only those arguments not named in each call
        foo <- as.expression(lapply(foo, magicDots, exclude = exclude))
    }
    ## FIXME: should we have 
    ##
    ## if (missing(data)) data <- parent.frame()
    ##
    ## ?  See tests/layer.R for a non-obvious failure. But not sure
    ## how best to fix.
    mostattributes(foo) <-
        list(data = data,
             under = under,
             packets = packets,
             rows = rows,
             columns = columns,
             groups = groups,
             superpose = superpose,
             style = style,
             theme = theme)
    lay <- list(foo)
    class(lay) <- c("layer", "trellis")
    lay
}

## convert a call containing `...` to only pass on arguments
## not named in the call
magicDots <- function(ocall, exclude = NULL, assume.xy = TRUE)
{
    if (!is.call(ocall)) stop("arguments to layer() should be calls")
    ## call recursively with any calls inside this one
    for (i in seq_along(ocall)[-1]) {
        thisArg <- ocall[[i]]
        if (missing(thisArg)) ## eg x[,1]
            next
        if (is.call(thisArg)) {
            ## skip function definitions
            if (identical(thisArg[[1]], as.symbol("function")))
                next
            ocall[[i]] <- Recall(thisArg, exclude = exclude, assume.xy = assume.xy)
        }
    }
    Args <- as.list(ocall)[-1]
    ## nothing to do if there are no dots in the call
    idots <- sapply(Args, identical, as.symbol("..."))
    if (!any(idots))
        return(ocall)
    Args <- Args[!idots]
    ## nothing to do if there are only dots in the call (unless exclude)
    if ((length(Args) == 0) && (length(exclude) == 0))
        return(ocall)
    ## assume first argument is 'x' if is un-named, and second 'y'
    if (assume.xy && (length(Args) > 0)) {
        if (is.null(names(Args)))
            names(Args) <- rep("", length = length(Args))
        if (identical(names(Args)[1], ""))
            names(Args)[1] <- "x"
        if (identical(names(Args)[2], ""))
            names(Args)[2] <- "y"
    }
    if (length(exclude) == 0) {
        ## simple case
        mcall <-
            substitute(do.call(FUN,
                               modifyList(list(...), Args)),
                       list(FUN = ocall[[1]], Args = Args))
    } else {
        ## exclude named arguments from dots
        mcall <-
            substitute(do.call(FUN,
                               modifyList(list(...)[!(names(list(...)) %in% exclude)],
                                          Args)),
                       list(FUN = ocall[[1]], Args = Args, exclude = exclude))
    }
    mcall
}

layer_ <- function(...)
{
    ccall <- match.call()
    ccall$under <- TRUE
    ccall[[1]] <- quote(layer)
    eval.parent(ccall)
}

glayer <- function(...)
{
    ccall <- match.call()
    ccall$superpose <- TRUE
    ccall[[1]] <- quote(layer)
    eval.parent(ccall)
}

glayer_ <- function(...)
{
    ccall <- match.call()
    ccall$superpose <- TRUE
    ccall$under <- TRUE
    ccall[[1]] <- quote(layer)
    eval.parent(ccall)
}

## to avoid print.trellis
print.layer <- function(x, ...) print.default(x, ...)

## to avoid [.trellis and to keep the class attribute
"[.layer" <- function (x, i, ...)
    structure(unclass(x)[i], class = class(x))

"+.trellis" <- function(object, lay)
{
    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(`+`)
    if (missing(object) || missing(lay)) stop("Only one argument supplied to binary operator + which requires two.")
    stopifnot(inherits(object, "trellis"))
    lay <- as.layer(lay)
    if (inherits(object, "layer")) {
        ## just concatenate lists
        return(structure(c(unclass(object), unclass(lay)),
                         class = c("layer", "trellis")))
    }
    panel <- if ("panel" %in% names(object$panel.args.common))
        object$panel.args.common$panel
    else object$panel
    panel <- if (is.function(panel)) panel
    else if (is.character(panel)) {
        ## could be just get(panel), but for flattenPanel:
        ## do not expand original panel function eg panel.xyplot(...)
        tmp <- function(...) NA
        body(tmp) <- call(panel, quote(...))
        environment(tmp) <- globalenv()
        tmp
    } else eval(panel)
    ## a flag to indicate this panel function has layers
    ## (used by flattenPanel and undoLayer)
    .is.a.layer <- TRUE
    newpanel <- function(...) {
        .UNDER <- unlist(lapply(lay, attr, "under"))
        ## underlaying items only
        drawLayer(lay[.UNDER], list(...))
        ## original panel function:
        panel(...)
        ## overlaying items only
        drawLayer(lay[.UNDER == FALSE], list(...))
    }
    if ("panel" %in% names(object$panel.args.common))
        object$panel.args.common$panel <- newpanel
    else object$panel <- newpanel
    ## need this to allow further calls to update() to insert arguments:
    object$call <- call("update", ocall)
    object
}

drawLayer <- function(lay, panelArgs = trellis.panelArgs())
{
    lay <- as.layer(lay)
    .UNDER <- unlist(lapply(lay, attr, "under"))
    ## underlayers, in reverse order
    for (.ITEM in rev(lay[.UNDER]))
        drawLayerItem(.ITEM, panelArgs)
    ## overlayers
    for (.ITEM in lay[.UNDER == FALSE])
        drawLayerItem(.ITEM, panelArgs)
    invisible()
}

drawLayerItem <- function(layer.item, panelArgs)
{
    stopifnot(is.expression(layer.item))
    ## check that any restrictions on packets/rows/columns are met
    matchesok <- function(spec, value) {
        if (is.null(spec)) return(TRUE)
        if (is.numeric(spec) && all(spec <= 0))
            ## negative indexes exclude items
            return(value %in% -spec == FALSE)
        else
            return(value %in% spec)
    }
    matchesallok <-
        with(list(a = attributes(layer.item)),
             matchesok(a$packets, packet.number()) &&
             matchesok(a$rows, current.row()) &&
             matchesok(a$columns, current.column()))
    if (!matchesallok) return()
    ## set given theme for duration of this function
    if (!is.null(attr(layer.item, "theme"))) {
        .TRELLISPAR <- trellis.par.get()
        trellis.par.set(attr(layer.item, "theme"))
        on.exit(trellis.par.set(.TRELLISPAR))
    }
    ## define a layer drawing function, which may be per group
    drawLayerItemPerGroup <- function(...)
    {
        ## Note: layer.item is found in this function's environment
        dots <- list(...)
        ## restrict to specified group numbers
        groupok <- (matchesok(attr(layer.item, "groups"), dots$group.number) ||
                    matchesok(attr(layer.item, "groups"), as.character(dots$group.value)))
        if (!groupok)
            return()
        if (!is.null(attr(layer.item, "style"))) {
            ## extract plot style attributes from given index into superpose.*
            .TRELLISPAR <- trellis.par.get()
            local({
                i <- attr(layer.item, "style")
                line <- Rows(trellis.par.get("superpose.line"), i)
                symbol <- Rows(trellis.par.get("superpose.symbol"), i)
                polygon <- Rows(trellis.par.get("superpose.polygon"), i)
                trellis.par.set(plot.line = line,
                                superpose.line = line,
                                add.line = line,
                                add.text = line,
                                plot.symbol = symbol,
                                superpose.symbol = symbol,
                                plot.polygon = polygon,
                                superpose.polygon = polygon,
                                axis.text = line,
                                axis.line = line
                                )
            })
            on.exit(trellis.par.set(.TRELLISPAR))
        }
        with(dots,
             eval(layer.item, attr(layer.item, "data"),
                  environment()))
    }
    ## call panel.superpose for group layers
    if (isTRUE(attr(layer.item, "superpose"))) {
        do.call("panel.superpose",
                modifyList(panelArgs,
                  list(panel.groups = drawLayerItemPerGroup)))
    } else {
        do.call("drawLayerItemPerGroup", panelArgs)
    }
}

flattenPanel <- function(object)
{
    flattenFun <- function(fun)
    {
        env <- environment(fun)
        ## check if this panel function is simple or has layers
        if (is.null(env) ||
            !exists(".is.a.layer", env, inherits = FALSE))
            return(as.expression(body(fun)))
        ## merge: under layers, existing panel, over layers
        .UNDER <- sapply(env$lay, attr, "under")
        c(do.call("c", rev(env$lay[.UNDER])),
          flattenFun(env$panel),
          do.call("c", env$lay[.UNDER == FALSE]))
    }
    flat <- flattenFun(object$panel)
    ## wrap in braces, as in a function body
    as.call(c(quote(`{`), flat))
}

## not exported -- I do not think this is really useful
undoLayer <- function(x)
{
    stopifnot(is.function(x$panel))
    env <- environment(x$panel)
    if (!exists(".is.a.layer", env, inherits=FALSE))
        stop("does not look like a layer")
    update(x, panel=env$panel)
}

axis.grid <-
    function(side = c("top", "bottom", "left", "right"),
             ..., ticks = c("default", "yes", "no"),
             scales, components, line.col)
{
    side <- match.arg(side)
    ticks <- match.arg(ticks)
    scales.tck <- switch(side,
                         left = , bottom = scales$tck[1], 
                         right = , top = scales$tck[2])
    ## only draw major ticks (those corresponding to labels)
    comps.major <- components
    mycomps <- components[[side]]
    if (is.list(mycomps)) {
        lab <- as.character(mycomps$labels$labels)
        if (any(lab != "")) {
            tck <- mycomps$ticks$tck
            if (any(tck * scales.tck != 0)) {
                tck <- rep(tck, length = length(lab))
                comps.major[[side]]$ticks$tck <- ifelse(lab == "", NA, tck)
            }
        }
    }
    ## We can have ticks even if labels are absent
    ## else {
    ##     ticks <- "no"
    ## }

    ## Use axis.text for ticks because axis.line$col might be transparent
    axis.text <- trellis.par.get("axis.text")
    axis.default(side, scales = scales, ticks = ticks,
                 components = comps.major, ...,
                 line.col = axis.text$col)
    ## Now draw grid lines corresponding to axis ticks.
    ## Do this only for the bottom and right sides; because top/left
    ## are called from strip viewports, not panel.
    if (side %in% c("top", "left"))
        return()
    if (scales$draw == FALSE)
        return()
    ref.line <- trellis.par.get("reference.line")
    if (side == "bottom") {
        tck <- abs(mycomps$ticks$tck)
        panel.refline(v = mycomps$ticks$at, 
                      lwd = ref.line$lwd * tck,
                      alpha = ref.line$alpha * tck / max(tck, na.rm = TRUE))
    }
    if (side == "right") {
        if (!is.list(mycomps))
            mycomps <- components[["left"]]
        tck <- abs(mycomps$ticks$tck)
        panel.refline(h = mycomps$ticks$at, 
                      lwd = ref.line$lwd * tck,
                      alpha = ref.line$alpha * tck / max(tck, na.rm = TRUE))
    }
}

panel.2dsmoother <-
    function(x, y, z, subscripts = TRUE,
             form = z ~ x * y, method = "loess", ...,
             args = list(), n = 100)
{
    if (length(subscripts) == 0)
        return()
    ## allow 'form' to be passed as the first argument
    missing.x <- missing(x)
    if (!missing.x && inherits(x, "formula")) {
        form <- x
        missing.x <- TRUE
    }
    ## use 'x', 'y', 'z' if given
    ## otherwise try to find them in the formula environment
    if (missing.x)
        x <- environment(form)$x
    if (missing(y))
        y <- environment(form)$y
    if (missing(z))
        z <- environment(form)$z
    x <- x[subscripts]
    y <- y[subscripts]
    z <- z[subscripts]
    ok <- is.finite(x) & is.finite(y) & is.finite(z)
    if (sum(ok) < 1) 
        return()
    x <- as.numeric(x)[ok]
    y <- as.numeric(y)[ok]
    z <- as.numeric(z)[ok]
    mod <- do.call(method,
                   c(alist(form, data = list(x = x, y = y, z = z)),
                     args))
    ## use the limits of the data, or panel limits, whichever is smaller
    lims <- current.panel.limits()
    xrange <- c(max(min(lims$x), min(x)), min(max(lims$x), max(x)))
    yrange <- c(max(min(lims$y), min(y)), min(max(lims$y), max(y)))
    xseq <- seq(xrange[1], xrange[2], length = n)
    yseq <- seq(yrange[1], yrange[2], length = n)
    ## zseq <- seq(min(z), max(z), length = n)
    grid <- expand.grid(x = xseq, y = yseq)
    fit <- predict(mod, grid)
    panel.levelplot(x = grid$x, y = grid$y, z = fit, subscripts = TRUE,
                    ...)
}

## Want to plot intervals from start to end, color coded by decoded, 
## and several in a panel  according to source

## since there are colors involved and levelplot already 
## has a colorkey, we'll use that


prepanel.segplot <- 
    function(x, y, z, subscripts, horizontal = TRUE, ...)
{
    ans <- 
        list(xlim = range(x[subscripts], y[subscripts], finite = TRUE), 
             ylim = if (is.factor(z)) levels(z) else range(z[subscripts], finite = TRUE),
             yat = if (is.factor(z)) sort(unique(as.numeric(z[subscripts]))) else NULL)
    if (horizontal) ans
    else with(ans, list(xlim = ylim, ylim = xlim, xat = yat))
}

## prepanel.segplot <- 
##     function(x, y, z, subscripts, horizontal = TRUE, ...)
## {
##     isnum.z <- is.numeric(z)
##     ans <- 
##         list(xlim = range(x[subscripts], y[subscripts], finite = TRUE), 
##              ylim = if (isnum.z) range(z[subscripts], finite = TRUE) else levels(z),
##              yat = if (isnum.z) NULL else sort(unique(as.numeric(z[subscripts]))))
##     if (horizontal) ans
##     else with(ans, list(xlim = ylim, ylim = xlim, xat = yat))
## }


panel.segplot <- 
    function(x, y, z, level = NULL, subscripts,
             at,
             draw.bands = is.factor(z),
             col = if (draw.bands) plot.polygon$col else plot.line$col,
             alpha = if (draw.bands) plot.polygon$alpha else plot.line$alpha,
             lty = if (draw.bands) plot.polygon$lty else plot.line$lty,
             lwd = if (draw.bands) plot.polygon$lwd else plot.line$lwd,
             border = if (draw.bands) plot.polygon$border else "transparent",
             col.symbol = col, alpha.symbol = alpha,
             col.regions = regions$col,
             band.height = 0.6,
             horizontal = TRUE,
             ...,
             segments.fun = panel.segments,
             centers = NULL,
             pch = 16)
{
    plot.line <- trellis.par.get("plot.line")
    plot.polygon <- trellis.par.get("plot.polygon") 
    regions <- trellis.par.get("regions")
    x1 <- as.numeric(x[subscripts])
    x2 <- as.numeric(y[subscripts])
    z <- z[subscripts]
    if (!is.null(level))
    {
        ## col is overridden
        level <- as.numeric(level[subscripts])
        col <- level.colors(level, at, col.regions, colors = TRUE)
    }
    if (draw.bands)
    {
        if (horizontal)
            panel.rect(x = 0.5 * (x1 + x2),
                       width = x2 - x1,
                       y = as.numeric(z), height = band.height,
                       border = border, col = col, alpha = alpha,
                       lty = lty, lwd = lwd,
                       ...)
        else 
            panel.rect(y = 0.5 * (x1 + x2),
                       height = x2 - x1,
                       x = as.numeric(z), width = band.height,
                       border = border, col = col, alpha = alpha,
                       lty = lty, lwd = lwd,
                       ...)
    }
    else
    {
        if (horizontal)
            segments.fun(x1, as.numeric(z), x2, as.numeric(z), 
                         col = col, alpha = alpha, lty = lty, lwd = lwd,
                         ...)
        else
            segments.fun(as.numeric(z), x1, as.numeric(z), x2,
                         col = col, alpha = alpha, lty = lty, lwd = lwd,
                         ...)

    }
    if (!is.null(centers))
    {
        if (horizontal)
            panel.points(x = as.numeric(centers[subscripts]),
                         y = as.numeric(z),
                         col = col.symbol, alpha = alpha.symbol,
                         pch = pch, ...)
        else
            panel.points(y = as.numeric(centers[subscripts]),
                         x = as.numeric(z),
                         col = col.symbol, alpha = alpha.symbol,
                         pch = pch, ...)

    }
}


segplot <- function(x, data, ...) UseMethod("segplot")


segplot.formula <- 
    function(x, data = parent.frame(),
             level = NULL, centers = NULL,
             prepanel = prepanel.segplot,
             panel = panel.segplot,
             xlab = NULL, ylab = NULL,
             horizontal = TRUE,
             ...,
             at, cuts = 30, colorkey = !is.null(level))
{
    level <- eval(substitute(level), data, parent.frame())
    centers <- eval(substitute(centers), data, parent.frame())
    if (!is.null(level))
    {
        rng <- lattice:::extend.limits(range(as.numeric(level), finite = TRUE))
        if (missing(at)) at <- do.breaks(rng, cuts + 1)
    }
    foo <- levelplot(x, data, level = level, centers = centers,
              ...,
              default.scales = 
              if (horizontal) list(y = list(alternating = FALSE, tck = 0))
              else list(x = list(alternating = FALSE, tck = 0)),
              xlab = xlab,
              ylab = ylab,
              at = at,
              colorkey = colorkey,
              horizontal = horizontal,
              prepanel = prepanel, 
              panel = panel)
    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(segplot)
    foo
}


## green.red <- function(n, gamma = 1, power = 1)
## {
##     m <- ceiling(n/2)
##     c(hsv(h = 0.33, s = seq(1, 0, length = m)^power, gamma = gamma),
##       hsv(h = 1, s = seq(0, 1, length = m)^power, gamma = gamma))
## }



## segplot(1:100 ~ rnorm(100) + runif(100), data = parent.frame())

## based on some of the default themes and scales used in ggplot2 by Hadley Wickham.

ggplot2like.opts <- function()
{
    list(default.args =
         list(axis = axis.grid,
              xscale.components = xscale.components.subticks,
              yscale.components = yscale.components.subticks,
              between = list(x = 0.4, y = 0.4))
         )
}

ggplot2like <-
    function(..., n = 6, h = c(0,360) + 15, l = 65, c = 100,
             h.start = 0, direction = 1,
             low = "#3B4FB8", high = "#B71B1A", space = "rgb")
{
    ## copied from ggplot2::scale_colour_hue
    rotate <- function(x) (x + h.start) %% 360 * direction
    if ((diff(h) %% 360) < 1) {
        h[2] <- h[2] - 360 / n
    }
    colseq <-
        hcl(h = rotate(seq(h[1], h[2], length = n)), c = c, l = l)
    ## copied from ggplot2::scale_colour_gradient
    ramp  <- colorRampPalette(c(low, high), space = space, interpolate = "linear")(100)
    theme <- custom.theme(symbol = colseq,
                          fill = colseq,
                          region = ramp)
    ## based on ggplot2::theme_gray
    theme <-
        modifyList(theme,
                   list(axis.line = list(col = "transparent"),
                        axis.text = list(cex = 0.8, lineheight = 0.9, col = "grey50"),
                        panel.background = list(col = "grey90"),
                        reference.line = list(col = "white"),
                        strip.background = list(col = c("grey80", "grey70", "grey60")),
                        strip.shingle = list(col = c("grey60", "grey50", "grey40")),
                        strip.border = list(col = "transparent"),
                        add.text = list(cex = 0.8))
                   )
    ## misc
    theme <-
        modifyList(theme,
                   list(plot.symbol = list(col = "black", pch = 19, cex = 0.6),
                        superpose.symbol = list(pch = 19, cex = 0.6),
                        plot.line = list(col = "black"),
                        plot.polygon = list(col = "grey20", border = "transparent"),
                        superpose.polygon = list(border = "transparent"),
                        box.dot = list(col = "grey20", pch = "|"),
                        box.rectangle = list(fill = "white", col = "grey20"),
                        box.umbrella = list(col = "grey20", lty = 1),
                        dot.line = list(col = "white"),
                        dot.symbol = list(col = "black", pch = 19))
                   )
    ## custom over-rides
    modifyList(theme, simpleTheme(...))
}

### Some obsolete experimental stuff. Remove at some point.


### Now, the question is, (1) whether to have the method for data
### frames (which is what's of immediate interest) here or in lme4.
### It doesn't really make sense to have it in lme4, since we won't be
### defining a new class.  But most of the features would be borrowed
### from nlme (2) whether to have methods for any other class (like
### 'lm' maybe)



### data frame method: gplotArgs.data.frame(x, <special args>, ...)

## 4 levels of information:
##    o attr(x, "ginfo")
##    o <special args>
##    o attr(x, "gplot.args")
##    o ...
## the first 2 determine a reasonable list, overridden by the 3rd and
## then the 4th



## should we have a non-generic "gplotArgs<-" for setting the
## "gplot.args" attribute?



## First, we need some good default panel functions.  These need to
## depend on

## (1) the type of display formula -
##      o factor ~ numeric   [ model: numeric ~ 1       | factor ]
##      o numeric ~ numeric  [ model: numeric ~ numeric | factor ]
##      o  ~ numeric         [ model:  factor ~ numeric | factor ] NEW


panel.df.fn <-               # factor ~ numeric
    function(x, y, groups = NULL, ...)
{
    panel.dotplot(x=x, y=y, groups = groups, ...)
}

panel.df.nn <-               # numeric ~ numeric
    function(x, y, lines = all(type == "p"), type = "p", ...)
{
    panel.xyplot(x, y, type = type, ...)
    if (lines)
    {
        y.avg <- tapply(y, x, mean) # lines through average y
        y.avg <- y.avg[!is.na(y.avg)]
        if (length(y.avg) > 0)
        {
            xvals <- as.numeric(names(y.avg))
            ord <- order(xvals)
            panel.xyplot(xvals[ord], y.avg[ord], type = "l", ...)
        }
    }
}



panel.df <- # combines above 2, should be called
    function(x, y = NULL, groups = NULL, grid = TRUE, ...)
{
    yNull <- is.null(y)
    groupsNull <- is.null(groups)
    xFactor <- is.factor(x)
    yFactor <- is.factor(y)

    if (yNull) ## for factor response, may not be in use yet
    {
        if (groupsNull) panel.densityplot(x = x, ...)
        else panel.superpose(x = x, groups = groups,
                             panel.groups = "panel.densityplot", ...)
    }
    else if (!xFactor && !yFactor) ## numeric ~ numeric
    {
        if (grid) panel.grid()
        if (groupsNull) panel.df.nn(x = x, y = y, ...)
        else panel.superpose(x = x, y = y, groups = groups,
                             panel.groups = panel.df.nn, ...)
    }
    else if (yFactor && !xFactor)
    {
        panel.df.fn(x = x, y = y, groups = groups, ...)
    }
    else stop("can't handle both x and y being factors yet")
    invisible()
}


## display formula

## generally, the display is determined by the arguments 'formula' and
## 'groups'.  In the nlme scheme, the formula associated with the data
## is a model formula, not a display formula.  The display is further
## determined by the 'outer' and 'inner' arguments.  

## Let's try to outline a strategy for this.

## attr(x, "ginfo") can contain things used in nlme's groupedData.

## formula can be of the form 'y ~ x | id' (only one variable in 'id',
## although it can be an interaction). inner=~a becomes default
## grouping factor. outer=~b+c used for ordering levels of id by
## values of y.  outer can be also an argument to
## gplotArgs.data.frame.  If TRUE, it's equivalent to outer =
## ginfo$outer, or it could be =~e+f.  In either case, it becomes the
## conditioning variables, id becomes groups (inner would then be
## ignored)

## In the formula itself, if x is a factor, the display formula
## becomes id ~ y, with groups = x (unless x = 1, then no groups)



## .typeInDF <- function(x, data)
## {
##     if (x == "1") "one"
##     else if (is.factor(data[[x]])) "factor"
##     else if (is.numeric(data[[x]])) "numeric"
##     else stop(paste("don't recognize", class(data[[x]])))
## }



## should work with expressions like log(height) as well
.typeInDF <- function(x, data)
{
    if (all(is.na(x))) return(NA)
    x <- eval(parse(text = x), data, parent.frame())
    if (length(x) == 1 && x == 1) "one"
    else if (is.factor(x)) "factor"
    else if (is.numeric(x)) "numeric"
    else stop(paste("don't recognize", class(x)))
}




## S3 method: underlies groupedData() in lme4

## FIXME: the following doesn't work in the docs.  What's the recommended way?
## \method{"gplotArgs<-"}{data.frame}(x, value)



"gplotArgs<-.data.frame" <- 
    function(x, value)
{
    if (!is.list(value)) stop("assigned value must be list")
    process.args <-
        function(formula, 
                 order.groups = TRUE,
                 FUN = function(x, ...) max(x, na.rm = TRUE),
                 outer = NULL, inner = NULL,
                 labels = list(), units = list(), ...)
        {
            list(ginfo =
                 list(formula = formula,
                      order.groups = order.groups,
                      FUN = FUN,
                      outer = outer,
                      inner = inner,
                      labels = labels,
                      units = units),
                 dots = list(...))
        }
    pargs <- do.call(process.args, value)
    attr(x, "ginfo") <- pargs$ginfo
    if (length(pargs$dots) > 0)
        attr(x, "gplot.args") <- pargs$dots
    x
}


## S3 method

## basic idea: get defaults from "ginfo" attribute, then overwrite by
## "gplot.args" attribute, followed by ...

gplotArgs.data.frame <-
    function(x, display.formula, outer = FALSE, inner = FALSE,
             groups = NULL,
             ...,
             subset = TRUE)
{

    ## The final result should contain only standard trellis args,
    ## with a special component plotFun, and optionally a
    ## display.formula, which overrides formula.  However, for data
    ## frames, there are some other issues.

    ## The "ginfo" attribute can only contain info traditionally in
    ## nlme groupedData objects.  This, along with explicit arguments
    ## here, will be used to create a list of trellis args.  These can
    ## be overridden by the "gplot.args" attribute, and then by
    ## ... here.


    ginfo <- attr(x, "ginfo")
    gplot.args <- attr(x, "gplot.args")
    ## equivalent to default method if ginfo is NULL
    if (is.null(ginfo) || !is.list(ginfo))
        return(updateList(gplot.args, list(...)))

    ## First (longish) task: determine display formula and groups

    ## groups is by far the most irritating thing to handle.  The only
    ## options I can think of: (1) evaluate groups here and pass it on
    ## and (2) change lattice to allow groups to be a formula.  Use
    ## (1) for now

    groups <- eval(substitute(groups), x, parent.frame()) # typically NULL

    ## subset poses a similar problem

    subset <- eval(substitute(subset), x, parent.frame())


    if (missing(display.formula))
        display.formula <- gplot.args$display.formula

    ## major step: get the display formula, but only if it's NULL
    model.formula <- ginfo$formula
    if (!is.null(display.formula))
        ## no point in jumping through hoops
    {
        ans <- list(display.formula = display.formula)
    }
    else if (!is.null(model.formula)) ## the interesting stuff
    {
        vars <-
            list(resp = .responseName(model.formula),
                 cov = .covariateName(model.formula),
                 grp = .groupsName(model.formula))




        if (ginfo$order.groups)
        {
            ## reorder grp based on values of resp

            if (is.null(ginfo$FUN)) ginfo$FUN <- function(x, ...) max(x, na.rm = TRUE)

            respVar <- vars$resp
            grpVar <- vars$grp

            scores <- tapply(x[[respVar]], x[[grpVar]], ginfo$FUN)

            if (inherits(ginfo$outer, "formula"))
            {
                outerVar <- .covariateName(ginfo$outer)
                outer.unique <- tapply(x[[outerVar]], x[[grpVar]], unique)
                ord <- order(outer.unique, scores)
            }
            else
                ord <- order(scores)

            x[[grpVar]] <- factor(x[[grpVar]], levels = names(scores)[ord])
        }






        ## display formula may be further modified by inner and outer.
        ## How does that affect rest of the calculations?

        if (is.logical(outer) && outer) outer <- ginfo$outer
        if (is.logical(inner) && inner) inner <- ginfo$inner

        ## both cannot happen. outer makes outer the conditioning
        ## variables, normal grp becomes groups. inner behaves as
        ## groups. outer takes precedence.

        if (inherits(outer, "formula"))
        {
            if (is.null(groups)) groups <- as.formula(paste("~", vars$grp))
            vars$grp <- .covariateName(outer)
        }
        else if (inherits(inner, "formula"))
        {
            ## FIXME: this may not be the right thing to do
            if (is.null(groups)) groups <- inner
        }
        


        varTypes <-
            lapply(vars, .typeInDF, data = x)

        ## Next step depends on varTypes

        ## case 1: cov = "one" - grp ~ resp
        ## case 2: cov = "factor" - grp ~ resp, groups = cov
        ## case 1: cov = "numeric" - resp ~ cov | grp

        fc <- switch(varTypes$cov,
                     one = paste(vars$grp, "~", vars$resp),
                     factor = paste(vars$grp, "~", vars$resp),
                     numeric =
                     paste(vars$resp, "~", vars$cov, "|", vars$grp))
        ans <-
            list(display.formula = as.formula(fc))

        if (varTypes$cov == "factor" && is.null(groups))
            groups <-
                eval(parse(text = vars$cov), x, parent.frame())
    }
    else 
    {
        stop("no formula!")
    }





    ## determine default plot function based on display.formula

    dvars <-
        list(resp = .responseName(ans$display.formula),
             cov = .covariateName(ans$display.formula),
             grp = .groupsName(ans$display.formula)) ## NA is none
    dvarTypes <-
        lapply(dvars, .typeInDF, data = x)
    if (dvarTypes$resp == "numeric" && dvarTypes$cov == "numeric")
        plotFun.constructed <- "xyplot"
    else if (dvarTypes$resp == "factor" && dvarTypes$cov == "numeric")
        plotFun.constructed <- "dotplot"
    else {
        ## str(dvarTypes)
        stop("unsupported combination")
    }

    ## other stuff in ginfo?

    ylab.constructed <-
        if ("labels" %in% names(ginfo) && dvars$resp %in% names(ginfo$labels))
            ginfo$labels[[dvars$resp]]
        else dvars$resp
    xlab.constructed <-
        if ("labels" %in% names(ginfo) && dvars$cov %in% names(ginfo$labels))
            ginfo$labels[[dvars$cov]]
        else dvars$cov
    if ("units" %in% names(ginfo) && dvars$resp %in% names(ginfo$units))
        ylab.constructed <- paste(ylab.constructed, ginfo$units[[dvars$resp]])
    if ("units" %in% names(ginfo) && dvars$cov %in% names(ginfo$units))
        xlab.constructed <- paste(xlab.constructed, ginfo$units[[dvars$cov]])


    ans <-
        updateList(ans,
                   list(plotFun = plotFun.constructed,
                        data = x,
                        panel = panel.df,
                        groups = groups,
                        subset = subset,
                        xlab = xlab.constructed, ylab = ylab.constructed,
                        aspect = if (plotFun.constructed == "xyplot") "xy" else "fill",
                        auto.key =
                        switch(plotFun.constructed,
                               xyplot = list(points = FALSE, lines = TRUE, space = "right"),
                               dotplot = list(points = TRUE, space = "right"))))


    
    if (!is.null(gplot.args))
        ans <- updateList(ans, gplot.args) ## leave this out?
    updateList(ans, list(...))
}

## Implementation Copyright (c) 2009 Felix Andrews
## based on plot style used in The Economist magazine.

theEconomist.theme <-
   function(win.fontfamily = NULL, #"Gill Sans MT"
            with.bg = FALSE, box = "black", ...)
{
    theme <- list(
         background = list(col = if (with.bg) "#D5E2E9" else "transparent"),
         plot.line = list(col = "#00526D", lwd = 2.5),
         superpose.line = list(col = c("#00526D", "#00A3DB", "#7A2713", "#939598", "#6CCFF6"), lwd = 2.5),
         plot.symbol = list(col = "#00526D", pch = 16),
         superpose.symbol = list(col = c("#00526D", "#00A3DB", "#7A2713", "#939598", "#6CCFF6"), pch = 16),
         plot.polygon = list(col = "#00526D"),
         superpose.polygon = list(col = c("#5F92A8", "#00526D", "#6CCFF6", "#00A3DB", "#A7A9AC")),
         regions = list(col = colorRampPalette(RColorBrewer::brewer.pal(9, "Blues"))(100)),
         reference.line = list(col = if (with.bg) "white" else "#aaaaaa", lwd = 1.75),
         dot.line = list(col = if (with.bg) "white" else "#aaaaaa", lwd = 1.75),
         add.line = list(col = "#ED1C24", lwd = 1.5),
         axis.line = list(col = box),
         box.3d = list(col = box),
         strip.border = list(col = box),
         strip.background = list(col = if (with.bg) "white" else "#CBDDE6"),
         strip.shingle = list(col = if (with.bg) "#CBDDE6" else "#00A3DB", alpha = 0.5),
         par.main.text = list(font = 1, just = "left", x = grid::unit(5, "mm")),
         par.sub.text = list(font = 1, just = "left", x = grid::unit(5, "mm")),
         axis.text = list(cex = 0.8),
         box.dot = list(col = "#00526D", pch = "|", lwd = 1.75),
         box.rectangle = list(fill = "#00526D", alpha = 0.5, col = "#00526D", lwd = 1.75),
         box.umbrella = list(col = "#00526D", lty = 1, lwd = 1.75)
         )
    if (.Platform$OS.type == "windows" && !is.null(win.fontfamily)) {
        windowsFonts(TheEconomistLike = win.fontfamily)
        theme$grid.pars$fontfamily <- "TheEconomistLike"
    } else {
        ## TODO: how do fonts work on linux etc?
    }
    modifyList(modifyList(standard.theme("pdf"), theme), simpleTheme(...))
}

theEconomist.opts <- function()
{
    list(default.args =
         list(axis = theEconomist.axis,
              xscale.components = xscale.components.subticks,
              between = list(x = 0.8, y = 0.8)),
         axis.padding = list(numeric = 0, factor = 0.6),
         skip.boundary.labels = 0,
         layout.widths =
         list(axis.left = list(x = 0, units = "char"),
              axis.right = list(x = 6, units = "char"))
         )
}

theEconomist.axis <-
    function(side = c("top", "bottom", "left", "right"),
             scales, components, ...,
             labels = c("default", "yes", "no"),
             ticks = c("default", "yes", "no"),
             line.col)
{
    if (scales$draw == FALSE)
        return()
    side <- match.arg(side)
    labels <- match.arg(labels)
    ticks <- match.arg(ticks)
    if (side %in% c("bottom", "top")) {
        if (side == "top")
            ticks <- "no"
        if (scales$relation == "same") {
            scales$alternating <- 1 ## bottom side only
        }
    }
    if (side %in% c("left", "right")) {
        ticks <- "no"
        components[["left"]]$ticks$tck <- 0
        if (scales$relation == "same") {
            scales$alternating <- 2 ## right side only
        } else {
            if (side == "right") {
                labels <- if (scales$draw) "yes" else "no"
                if (!is.list(components$right)) {
                    components$right <- components$left
                }
            }
            if (side == "left") {
                ## check for two different axes on left and right
                if (!is.list(components$right)) {
                    labels <- "no"
                }
            }
        }
    }
    ## use axis.text for ticks because axis.line$col might be transparent
    axis.text <- trellis.par.get("axis.text")
    axis.default(side, scales = scales,
                 components = components, ...,
                 labels = labels, ticks = ticks,
                 line.col = axis.text$col)
    ## now draw grid lines corresponding to horizontal axis ticks.
    ## can only do this with the bottom and right sides;
    ## otherwise the strip viewports are current, not panel.
    if (side %in% c("top", "left"))
        return()
    if (side == "right") {
        comp.list <- components[["right"]]
        if (!is.list(comp.list))
            comp.list <- components[["left"]]
        panel.refline(h = comp.list$ticks$at)
        ## draw axis line along bottom (assuming transparent axis.line)
        lims <- current.panel.limits()
        panel.abline(h = lims$y[1], col = axis.text$col)
    }
}

asTheEconomist <-
    function(x, ...,
             type = "l",
             ylab = expression(NULL),
             xlab = expression(NULL),
             par.settings =
               theEconomist.theme(with.bg = with.bg, box = "transparent"),
             with.bg = FALSE,
             par.strip.text = list(font = 2))
{
    ans <- x
    ## make nice left-aligned title
    title <- ans$main
    if (is.null(title)) title <- ans$ylab
    if (is.null(title)) title <- ans$ylab.default
    ans <- update(ans, main = title,
                  type = type, ylab = ylab, xlab = xlab,
                  par.settings = par.settings,
                  par.strip.text = par.strip.text,
                  between = list(x = 0.8, y = 0.8),
                  scales = list(y = list(axs = "i", alternating = 2)),
                  skip.boundary.labels = 0,
                  lattice.options = list(
                  layout.widths =
                  list(axis.left = list(x = 0, units = "char"),
                       axis.right = list(x = 6, units = "char"))
                  )
                  )
    ## these do not get through update()
    ans$axis <- theEconomist.axis
    ans$xscale.components <- xscale.components.subticks
    ans$call <- match.call()
    ans
}


genGillSans <- function()
{
    ## generate AFM font metrics of GillSans - for ps/pdf
    ## TODO: is there a similar looking free/open font?
    oldwd <- getwd()
    on.exit(setwd(oldwd))
    setwd(system.file("afm", package = "grDevices"))
    fnames <- c("GIL_____", "GILB____", "GILI____", "GILBI___")
    for (fn in fnames) {
        system(sprintf("ttf2afm -o %s.afm %s.TTF", fn, 
                       file.path(Sys.getenv("windir"), "Fonts", fn)))
        system(sprintf("gzip %s.afm", fn))
    }
    TheEconomistLike <- Type1Font("TheEconomistLike",
                                  paste(fnames, ".afm", sep = ""))
    postscriptFonts(TheEconomistLike = TheEconomistLike)
    pdfFonts(TheEconomistLike = TheEconomistLike)
    ps.options(fonts = c("sans", "serif", "mono", "Times", "Helvetica", "Courier", "URWHelvetica", "TheEconomistLike"))
    pdf.options(fonts = ps.options()$fonts)
    ## TODO: problem: after this, plots generated by bitmap() use wrong font
}

##
## Copyright (c) 2008 Felix Andrews <felix@nfrac.org>
## GPL version 2 or newer

doubleYScale <-
    function(obj1, obj2, use.style = TRUE,
             style1 = if (use.style) 1, style2 = if (use.style) 2,
             add.axis = TRUE, add.ylab2 = FALSE,
             text = NULL, auto.key = if (!is.null(text))
               list(text, points = points, lines = lines, ...),
             points = FALSE, lines = TRUE, ..., under = FALSE)
{
    stopifnot(inherits(obj1, "trellis"))
    stopifnot(inherits(obj2, "trellis"))

    if (any(style1 == 0)) style1 <- NULL
    if (any(style2 == 0)) style2 <- NULL
    ## force same x scales
    #xlim1 <- obj1$x.limits
    #if (is.list(xlim1))
    #    xlim1 <- rep(xlim1, length = prod(dim(obj2)))
    #obj2 <- update(obj2, xlim = xlim1, ylim = obj2$y.limits)
    ## TODO - ylim only here to workaround bug in lattice 0.17-15

    if (!is.null(auto.key)) {
        space <- "top"
        if (!is.null(auto.key$space))
            space <- auto.key$space
        auto.key$space <- NULL
        keyLeg <- list(space = list(fun = "drawSimpleKey",
                       args = auto.key))
        names(keyLeg) <- space
        obj1 <- update(obj1, legend = keyLeg)
    }

    ## merge legends
    obj1$legend <- mergeTrellisLegends(obj1$legend, obj2$legend)
    
    if (add.ylab2) {
        ## add ylab2 as a 'legend' (idea from John Maindonald)
        ## draw both ylabs in their style, if specified
        ylabStyledGrob <- function(label, style) {
            textGrob(label, y = 0.5, rot = 90,
                     gp = if (!is.null(style))
                     gpar(col = trellis.par.get("superpose.line")$col[style]))
        }
        is.characterOrExpression <- function(x)
            is.character(x) || is.expression(x)

        if (!is.null(style1)) {
            ylab1 <- obj1$ylab
            if (is.list(ylab1))
                ylab1 <- obj1$ylab.default
            if (is.characterOrExpression(ylab1)) {
                obj1$legend <-
                    mergeTrellisLegends(obj1$legend,
                             list(left = 
                                  list(fun = ylabStyledGrob,
                                      args = list(label = ylab1,
                                      style = style1))),
                             vertical = FALSE)
                obj1$ylab <- expression(NULL)
            }
        }
        ## TODO: use ylab.right from lattice 0.19-6
        ylab2 <- obj2$ylab
        if (is.list(ylab2))
            ylab2 <- obj2$ylab.default
        if (is.characterOrExpression(ylab2)) {
            obj1$legend <-
                    mergeTrellisLegends(list(right =
                             list(fun = ylabStyledGrob,
                                  args = list(label = ylab2,
                                  style = style2))),
                             obj1$legend,
                             vertical = FALSE)
        }
    }

    if (add.axis == FALSE) {
        ## if not drawing a second axis, nothing to do but...
        foo <- obj1 + as.layer(obj2, style = style2,
                               x.same = TRUE, y.same = FALSE,
                               axes = NULL, under = under)
    } else {
        ## need to specify padding to draw second y axis
        yAxPad <- list(layout.widths = list(
                       axis.left = list(x = 2.5, units = "char"),
                       axis.right = list(x = 3, units = "char")))

        dummy <- update(obj1, panel = function(...) NULL,
                        scales = list(y = list(draw = FALSE)),
                        lattice.options = yAxPad)
        
        foo <-
            dummy +
                as.layer(obj1, style = style1,
                         x.same = TRUE, y.same = FALSE,
                         axes = "y", out = TRUE, opp = FALSE) +
                as.layer(obj2, style = style2,
                         x.same = TRUE, y.same = FALSE,
                         axes = "y", out = TRUE, opp = TRUE,
                         under = under)
    }
    foo$call <- match.call()
    foo
}

as.layer.trellis <-
    function(x,
             x.same = TRUE,
             y.same = TRUE,
             axes = c(if (!x.same) "x", if (!y.same) "y"),
             opposite = TRUE,
             outside = FALSE,
             theme = x$par.settings,
             ...)
{
    if (identical(axes, TRUE)) axes <- c("x", "y")
    if (identical(axes, FALSE)) axes <- NULL
    opposite <- rep(opposite, length = 2)
    outside <- rep(outside, length = 2)
    if (x.same && y.same) {
        ## simply run the panel function in existing panel viewport
        return(layer({
                   packet.number <- min(packet.number(), prod(dim(x)))
                   do.call(x$panel, trellis.panelArgs(x, packet.number))
               }, data = list(x = x), theme = theme, ...))
    }
    ## else
    ## take one or more scales from layered object (so new viewport)
    ## draw panels and axes from this trellis object
    layer({
        packet.number <- min(packet.number(), prod(dim(x)))
        ## axis details...
        ## this all copied from lattice:::plot.trellis
        ## TODO: this is horrible; can use the axis function instead?
        x.relation.same <- x$x.scales$relation == "same"
        y.relation.same <- x$y.scales$relation == "same"
        xscale.comps <-
            if (x.relation.same)
                x$xscale.components(lim = x$x.limits,
                                    top = TRUE,
                                    ## rest passed on to
                                    ## calculateAxisComponents
                                    ## in the default
                                    ## case:
                                    at = x$x.scales$at,
                                    used.at = x$x.used.at,
                                    num.limit = x$x.num.limit,
                                    labels = x$x.scales$labels,
                                    logsc = x$x.scales$log,
                                    abbreviate = x$x.scales$abbr,
                                    minlength = x$x.scales$minl,
                                    n = x$x.scales$tick.number,
                                    format.posixt = x$x.scales$format)
            else
                x$xscale.components(lim = x$x.limits[[packet.number]],
                                    top = FALSE,
                                    ## rest passed on to
                                    ## calculateAxisComponents
                                    ## in the default
                                    ## case:
                                    at = if (is.list(x$x.scales$at))
                                    x$x.scales$at[[packet.number]]
                                    else x$x.scales$at,
                                    used.at = x$x.used.at[[packet.number]],
                                    num.limit = x$x.num.limit[[packet.number]],
                                    labels =
                                    if (is.list(x$x.scales$labels))
                                    x$x.scales$labels[[packet.number]]
                                    else x$x.scales$labels,
                                    logsc = x$x.scales$log,
                                    abbreviate = x$x.scales$abbr,
                                    minlength = x$x.scales$minl,
                                    n = x$x.scales$tick.number,
                                    format.posixt = x$x.scales$format)

        yscale.comps <-
            if (y.relation.same)
                x$yscale.components(lim = x$y.limits,
                                    right = TRUE,
                                    ## rest passed on to
                                    ## calculateAxisComponents
                                    ## in the default
                                    ## case:
                                    at = x$y.scales$at,
                                    used.at = x$y.used.at,
                                    num.limit = x$y.num.limit,
                                    labels = x$y.scales$labels,
                                    logsc = x$y.scales$log,
                                    abbreviate = x$y.scales$abbr,
                                    minlength = x$y.scales$minl,
                                    n = x$y.scales$tick.number,
                                    format.posixt = x$y.scales$format)
            else
                x$yscale.components(lim = x$y.limits[[packet.number]],
                                    right = FALSE,
                                    ## rest passed on to
                                    ## calculateAxisComponents
                                    ## in the default
                                    ## case:
                                    at = if (is.list(x$y.scales$at))
                                    x$y.scales$at[[packet.number]]
                                    else x$y.scales$at,
                                    used.at = x$y.used.at[[packet.number]],
                                    num.limit = x$y.num.limit[[packet.number]],
                                    labels =
                                    if (is.list(x$y.scales$labels))
                                    x$y.scales$labels[[packet.number]]
                                    else x$y.scales$labels,
                                    logsc = x$y.scales$log,
                                    abbreviate = x$y.scales$abbr,
                                    minlength = x$y.scales$minl,
                                    n = x$y.scales$tick.number,
                                    format.posixt = x$y.scales$format)
        xscale <- xscale.comps$num.limit
        yscale <- yscale.comps$num.limit
        ## maybe over-ride with original limits
        if (x.same)
            xscale <- current.panel.limits()$xlim
        if (y.same)
            yscale <- current.panel.limits()$ylim
        ## do panel(); need a new viewport with scales from 'x'
        pushViewport(viewport(xscale = xscale, yscale = yscale))
        do.call(x$panel, trellis.panelArgs(x, packet.number))
        ## use axis components from the standard side only
        xscale.comps$top <- TRUE
        yscale.comps$right <- TRUE
        x.comp.list <- xscale.comps$bottom
        y.comp.list <- yscale.comps$left
        ## axes viewport (clip = "off" for outside axes)
        ## note: to draw an outside axis at top when there are stips:
        ## should really do it in strip.column.row.off
        pushViewport(viewport(xscale = xscale, yscale = yscale,
                              clip = "off"))
        if (("x" %in% axes) && is.list(x.comp.list) && x$x.scales$draw) {
            comp.list <- x.comp.list
            scales.tck <- x$x.scales$tck[1]
            rot <- as.numeric(x$x.scales$rot)[1]
            if (outside[1]) {
                ## use axis.default where possible (i.e. where outside=TRUE)
                ## because it handles labels well in multi-panel layouts
                x$x.scales$alternating <- 3
                x$axis(side = if (opposite[1]) "top" else "bottom",
                       scales = x$x.scales,
                       components = xscale.comps,
                       as.table = x$as.table,
                       rot = rot)
            } else {
                panel.axis(side = if (opposite[1]) "top" else "bottom",
                       at = comp.list$ticks$at,
                       labels = comp.list$labels$labels,
                                        #tick = do.ticks,
                                        #draw.labels = do.labels,
                       check.overlap = comp.list$labels$check.overlap,
                       outside = outside[1],
                       half = FALSE,
                       tck = scales.tck * comp.list$ticks$tck,
                       rot = rot)
            }
        }
        if (("y" %in% axes) && is.list(y.comp.list) && x$y.scales$draw) {
            comp.list <- y.comp.list
            scales.tck <- x$y.scales$tck[1]
            rot <- as.numeric(x$y.scales$rot)[1]
            if (outside[2]) {
                ## use axis.default where possible (i.e. where outside=TRUE)
                ## because it handles labels well in multi-panel layouts
                x$y.scales$alternating <- 3
                x$axis(side = if (opposite[2]) "right" else "left",
                       scales = x$y.scales,
                       components = yscale.comps,
                       as.table = x$as.table,
                       rot = rot)
            } else {
                panel.axis(side = if (opposite[2]) "right" else "left",
                       at = comp.list$ticks$at,
                       labels = comp.list$labels$labels,
                                        #tick = do.ticks,
                                        #draw.labels = do.labels,
                       check.overlap = comp.list$labels$check.overlap,
                       outside = outside[2],
                       half = FALSE,
                       tck = scales.tck * comp.list$ticks$tck,
                       rot = rot)
            }
        }
        upViewport(2)
    }, data = list(x = x, x.same = x.same, y.same = y.same,
       axes = axes, opposite = opposite, outside = outside),
          theme = theme, ...)
}

## update elements of a list recursively. 

updateList <-
    function (x, val)
{
    if (is.null(x))
        x <- list()
    modifyList(x, val)
}


## common operations that only make sense in certain contexts


useOuterStrips <-
    function(x,
             strip = strip.default,
             strip.left = strip.custom(horizontal = FALSE),
             strip.lines = 1,
             strip.left.lines = strip.lines)
{
    dimx <- dim(x)
    stopifnot(inherits(x, "trellis"))
    stopifnot(length(dimx) == 2)
    as.table <- x$as.table
    
    opar <- if (is.null(x$par.settings)) list() else x$par.settings
    par.settings <-
        modifyList(opar,
                   list(layout.heights =
                        if (as.table) list(strip = c(strip.lines, rep(0, dimx[2]-1)))
                        else list(strip = c(rep(0, dimx[2]-1), strip.lines)),
                        layout.widths =
                        list(strip.left = c(strip.left.lines, rep(0, dimx[1]-1)))))
    if (is.character(strip))
        strip <- get(strip)
    if (is.logical(strip) && strip)
        strip <- strip.default
    new.strip <-
        if (is.function(strip))
        {
            function(which.given, which.panel, var.name, ...) {
                row.to.keep <- if (as.table) 1 else nrow(trellis.currentLayout())
                if (which.given == 1 && current.row() == row.to.keep)
                    strip(which.given = 1,
                          which.panel = which.panel[1],
                          var.name = var.name[1],
                          ...)
            }
        }
        else strip # This could reasonable happen only if strip == FALSE
    if (is.character(strip.left))
        strip.left <- get(strip.left)
    if (is.logical(strip.left) && strip.left)
        strip.left <- strip.custom(horizontal = FALSE)
    new.strip.left <-
        if (is.function(strip.left))
        {
            function(which.given, which.panel, var.name, ...) {
                if (which.given == 2 && current.column() == 1)
                    strip.left(which.given = 1,
                               which.panel = which.panel[2],
                               var.name = var.name[2],
                               ...)
            }
        }
        else strip.left
    update(x,
           par.settings = par.settings,
           strip = new.strip,
           strip.left = new.strip.left,
           par.strip.text = list(lines = 0.5),
           layout = dimx)
}




resizePanels <-
    function(x, h = 1, w = 1)
{
    if (!missing(x))
        return(update(x,
                      par.settings =
                      list(layout.heights = list(panel = h),
                           layout.widths = list(panel = w))))

    cl <- trellis.currentLayout()
    if (all(dim(cl) > 1))
        stop("layout must have single column or single row.")
    if (all(dim(cl) == 1))
    {
        message("Nothing to be done.")
        return()
    }
    if (any(cl == 0)) stop("missing panels not allowed")
    if (dim(cl)[2] == 1) ## single column
    {
        pos <- seq(length = dim(cl)[1])
        heights <- 
            sapply(pos,
                   function(i) {
                       trellis.focus("panel", 1, i, highlight = FALSE)
                       ylim <- current.panel.limits()$ylim
                       trellis.unfocus()
                       diff(range(ylim))
                   })
        return(trellis.last.object(par.settings =
                                   list(layout.heights = list(panel = heights))))
    }
    else if (dim(cl)[1] == 1) ## single row
    {
        pos <- seq(length = dim(cl)[2])
        widths <- 
            sapply(pos,
                   function(i) {
                       trellis.focus("panel", i, 1, highlight = FALSE)
                       xlim <- current.panel.limits()$xlim
                       trellis.unfocus()
                       diff(range(xlim))
                   })
        return(trellis.last.object(par.settings =
                                   list(layout.widths = list(panel = widths))))
    }
    print(dim(cl))
    stop("shouldn't come here")
}




## utility functions to extract components of a formula.  Don't work
## reliably with unusual symbols

.responseName <- function(formula)
{
    if (length(formula) == 3) as.character(formula[2])
    else stop("invalid formula")
}

.covariateName <- function(formula)
{
    RHS <- 
        if (length(formula) == 3) as.character(formula[3])
        else if (length(formula) == 2) as.character(formula[2])
        else stop("invalid formula")
    RHS <- strsplit(RHS, " | ", fixed = TRUE)[[1]]
    RHS[1]
}

.groupsName <- function(formula)
{
    RHS <- 
        if (length(formula) == 3) as.character(formula[3])
        else if (length(formula) == 2) as.character(formula[2])
        else stop("invalid formula")
    RHS <- strsplit(RHS, " | ", fixed = TRUE)[[1]]
    RHS[2]
}

##
## Copyright (c) 2008 Felix Andrews <felix@nfrac.org>
##

panel.xyarea <- function(x, ...)
    UseMethod("panel.xyarea")

## Plot a series as a filled polygon connected at given origin (on y axis).
## With groups, acts like panel.superpose, but with polygon style settings.
panel.xyarea.default <-
    function(x, y, groups = NULL, origin = NULL, horizontal = FALSE,
             col = if (is.null(groups)) plot.polygon$col else superpose.polygon$col,
             col.line = if (is.null(groups)) plot.polygon$col else superpose.polygon$col,
             border = if (is.null(groups)) plot.polygon$border else superpose.polygon$border,
             lty = if (is.null(groups)) plot.polygon$lty else superpose.polygon$lty,
             lwd = if (is.null(groups)) plot.polygon$lwd else superpose.polygon$lwd,
             alpha = if (is.null(groups)) plot.polygon$alpha else superpose.polygon$alpha,
             ..., fill, panel.groups = panel.xyarea)
{
    plot.polygon <- trellis.par.get("plot.polygon")
    superpose.polygon <- trellis.par.get("superpose.polygon")
    x <- as.numeric(x)
    y <- as.numeric(y)
    if (length(x) == 0) return()
    if (!is.null(groups)) {
        ## NOTE superpose does not handle 'border' argument, so pass it as col.line
        panel.superpose(x, y, ..., groups = groups, panel.groups = panel.groups,
                        col = col, col.line = col.line, lty = lty, lwd = lwd, border = border,
                        alpha = alpha, origin = origin, horizontal = horizontal)
    } else {
        if (!missing(col.line))
            col <- col.line
        if (horizontal == TRUE) {
            ## actually means origin is vertical. for consistency with panel.xyplot.
            xlim <- current.panel.limits()$xlim
            if (is.null(origin))
                origin <- xlim[1]
            infi <- is.infinite(x)
            x[infi] <- ifelse(x[infi] > 0, max(xlim), min(xlim))
        } else {
            ## default case; origin is horizontal
            ylim <- current.panel.limits()$ylim
            if (is.null(origin))
                origin <- ylim[1]
            infi <- is.infinite(y)
            y[infi] <- ifelse(y[infi] > 0, max(ylim), min(ylim))
        }
        stopifnot(is.numeric(origin))
        ## need to split up the series into chunks without any missing values
        ## (because NAs split the polygon)
        xy <- data.frame(x = x, y = y)
        ## order by ordinate values
        ord <- if (horizontal) order(xy$y) else order(xy$x)
        xy <- xy[ord,]
        ok <- complete.cases(xy)
        runs <- rle(ok)
        ## assign unique values to each chunk, and NAs between (dropped by 'split')
        runs$values[runs$values == TRUE] <- seq_len(sum(runs$values))
        runs$values[runs$values == FALSE] <- NA
        ## expand into long format
        chunks <- inverse.rle(runs)
        lapply(split(xy, chunks), function(xyi, ...) {
            x <- xyi$x
            y <- xyi$y
            ## drop ends of series to the origin; the polygon will be joined up at that level
            if (horizontal == TRUE) {
                ## non-default case
                yy <- c(head(y,1), y, tail(y,1))
                xx <- c(origin, x, origin)
            } else {
                ## default case
                xx <- c(head(x,1), x, tail(x,1))
                yy <- c(origin, y, origin)
            }
            ## we need to catch the 'fill' argument from panel.superpose, otherwise over-rides 'col'
            panel.polygon(xx, yy, alpha = alpha, col = col, border = border, lty = lty, lwd = lwd, ...)
        }, ...)
    }
}

panel.xyarea.ts <- function(x, y = x, ...)
{
    panel.xyarea(as.vector(time(x)), y, ...)
}

panel.xyarea.zoo <- function(x, y = x, ...)
{
    panel.xyarea(zoo::index(x), zoo::coredata(y), ...)
}

## A slightly modified copy of panel.qqmath
panel.qqmath.xyarea <-
    function(x, y = NULL,
             f.value = NULL,
             distribution = qnorm,
             qtype = 7,
             groups = NULL, ...,
             tails.n = 0)
{
    x <- as.numeric(x)
    distribution <-
        if (is.function(distribution)) distribution 
        else if (is.character(distribution)) get(distribution)
        else eval(distribution)
    nobs <- sum(!is.na(x))
    if (!is.null(groups))
        panel.xyarea(x, y = NULL,
                     f.value = f.value,
                     distribution = distribution,
                     qtype = qtype,
                     groups = groups,
                     panel.groups = panel.qqmath.xyarea,
                     ...,
                     tails.n = tails.n)
    else if (nobs)
    {
        if (is.null(f.value)) # exact data instead of quantiles
        {
            panel.xyarea(x = distribution(ppoints(nobs)),
                         y = sort(x),
                         ...)
        }
        else
        {
            pp <- if (is.numeric(f.value)) f.value else f.value(nobs)
            if (tails.n > 0)
            {
                ## use exact data for tails of distribution
                tails.n <- min(tails.n, nobs %/% 2)
                ppd <- ppoints(nobs)
                ## omit probabilities within the exact tails
                pp <- pp[(pp > ppd[tails.n] &
                          pp < ppd[nobs + 1 - tails.n])]
                ## add on probs corresponding to exact tails
                pp <- c(head(ppd, tails.n), pp, tail(ppd, tails.n))
                ## must use a quantile type that recovers exact values:
                qtype <- 1
            }
            xx <- distribution(pp)
            yy <- quantile(x, pp, 
                                  names = FALSE,
                                  type = qtype,
                                  na.rm = TRUE)
            panel.xyarea(x = xx, y = yy, ...)
        }
    }
}

simpleSmoothTs <-
    function(x, ...)
{
    UseMethod("simpleSmoothTs")
}

simpleSmoothTs.default <-
    function(x, ...,
             width = NROW(x) %/% 10 + 1, n = NROW(x),
             c = 1, sides = 2, circular = FALSE,
             kern = kernel("daniell", rep(floor((width/sides)/sqrt(c)), c)))
{

    if (sides == 2) {
        ii <- -kern$m:kern$m
        filter <- kern[ii]
    } else if (sides == 1) {
        ii <- 0:kern$m
        filter <- kern[ii] / sum(kern[ii]) ## normalise
    } else stop("unrecognised value of 'sides'")
    x <- as.ts(x)
    xf <- x
    xf[] <- filter(x, filter, sides = sides, circular = circular)
    if (n < NROW(x)) {
        ## reduce the number of points by aggregating chunks of 'reduce' time steps
        reduce <- round(NROW(x) / n)
        if (reduce > 1) {
            ndeltat <- deltat(xf) * reduce
            ## work-around for bug in aggregate.ts
            if ((ndeltat > 1) && (getRversion() < "2.11.1"))
                ndeltat <- ndeltat * (1 + getOption("ts.eps")/1000)
            xf <- aggregate(xf, ndeltat = ndeltat, FUN = mean)
            ## and adjust it so that each point is centered compared to the original series
            tsp(xf)[1:2] <- tsp(xf)[1:2] + (deltat(xf) %/% 2)
        }
    }
    xf
}

simpleSmoothTs.zoo <-
    function(x, ..., n = NROW(x))
{
    xts <- as.ts(x)
    xtsf <- simpleSmoothTs(xts, ..., n = n)
    ii <- TRUE
    if (n < NROW(x)) {
        ## find indices of aggregated time series in the original data
        ii <- findInterval(time(xtsf), time(xts))
        ## extract elements of time index corresponding to aggregated series
        ans <- zoo::zoo(as.matrix(xtsf), time(x)[ii])
    } else {
        ans <- x
        zoo::coredata(ans) <- zoo::coredata(xtsf)
    }
    ans
}

## a panel function for cloud that draws "3d bar charts" which
## shouldn't be used except in unusual circumstances


panel.3dbars <-
    function(x, y, z,
             rot.mat = diag(4), distance,
             xbase = 1, ybase = 1,
             xlim, xlim.scaled,
             ylim, ylim.scaled,
             zlim, zlim.scaled,
             zero.scaled,
             col = "black",
             lty = 1, lwd = 1,
             alpha = 1,
             ...,
             col.facet = "white",
             alpha.facet = 1)
{
    n <- length(z)
    col <- rep(col, length = n)
    col.facet <- rep(col.facet, length = n)
    alpha <- rep(alpha, length = n)
    alpha.facet <- rep(alpha.facet, length = n)
    lty <- rep(lty, length = n)
    lwd <- rep(lwd, length = n)
    id <-
        ((x >= xlim.scaled[1]) & (x <= xlim.scaled[2]) &
         (y >= ylim.scaled[1]) & (y <= ylim.scaled[2]) &
         !is.na(x) & !is.na(y) & !is.na(z))
    m <- ltransform3dto3d(rbind(x, y, 0), rot.mat, distance)
    ord <- sort.list(m[3,])
    ord <- ord[id[ord]]
    zero.scaled <-
        if (zero.scaled < zlim.scaled[1]) zlim.scaled[1]
        else if (zero.scaled > zlim.scaled[2]) zlim.scaled[2]
        else zero.scaled
    inRange <- function(x, lim)
    {
        rng <- range(x, finite = TRUE)
        rng[1] >= min(lim) && rng[2] <= max(lim)
    }
    ## draw bars one by one
    for (i in ord)
    {
        ## print(i)
        xbase.scaled <- diff(xlim.scaled) * xbase / diff(xlim)
        ybase.scaled <- diff(ylim.scaled) * ybase / diff(ylim)
        zz.sides <- matrix(c(zero.scaled, z[i]), 2, 1)[, rep(1, 5)]
        xx.sides <-
            c(x[i], x[i]) + xbase.scaled * 0.5 *
                rbind(c(-1, 1, 1, -1, -1), c(-1, 1, 1, -1, -1))
        yy.sides <-
            c(y[i], y[i]) + ybase.scaled * 0.5 *
                rbind(c(-1, -1, 1, 1, -1), c(-1, -1, 1, 1, -1))
        zz.top <- matrix(z[i], 2, 2)
        xx.top <- 
            c(x[i], x[i]) + xbase.scaled * 0.5 *
                rbind(c(-1, 1), c(-1, 1))
        yy.top <-
            c(y[i], y[i]) + ybase.scaled * 0.5 *
                rbind(c(-1, -1), c(1, 1))

        zz <- cbind(zz.sides, c(NA, NA), zz.top)
        xx <- cbind(xx.sides, c(NA, NA), xx.top)
        yy <- cbind(yy.sides, c(NA, NA), yy.top)
        ## str(list(xx, yy, zz))
        if (inRange(xx, xlim.scaled) &&
            inRange(yy, ylim.scaled) &&
            inRange(zz, zlim.scaled))
        {
            panel.3dwire(xx, yy, zz,
                         rot.mat = rot.mat, distance = distance,
                         xlim = xlim, xlim.scaled = xlim.scaled,
                         ylim = ylim, ylim.scaled = ylim.scaled,
                         zlim = zlim, zlim.scaled = zlim.scaled,
                         col = col[i], lty = lty[i], lwd = lwd[i],
                         alpha = alpha[i],
                         ...,
                         at = c(0, 1), # dummy
                         col.regions = col.facet[i],
                         alpha.regions = alpha.facet[i])
        }
    }
}




## panel.3dpolygon <-
##     function(x, y, z, rot.mat = diag(4), distance,
##              type = 'p',
##              xlim.scaled,
##              ylim.scaled,
##              zlim.scaled,
##              zero.scaled,
##              col = "white", 
##              border = "black", 
##              lty = 1, lwd = 1,
##              min.sides = 3,
##              ...,
##              subscripts = TRUE)
## {
##     m <- ltransform3dto3d(rbind(x, y, z), rot.mat, distance)
##     ## ord <- sort.list(m[3,])
##     n <- ncol(m)
##     w <- which(is.na(x) | is.na(y))
##     id.lengths <- diff(c(0, w, n))
##     cum.lengths <- c(0, cumsum(id.lengths))

##     idlist <-
##         lapply(seq_along(id.lengths),
##                function(i) {
##                    ind <- seq_len(id.lengths[i]) + cum.lengths[i]
##                    ind[-id.lengths[i]]
##                })

##     ord <-
##         order(sapply(idlist,
##                      function(ind) {
##                          min(m[3, ind])
##                      }))

##     for (ind in idlist[ord])
##     {
##         if (length(ind) >= min.sides)
##             panel.polygon(x = m[1, ind], y = m[2, ind],
##                           col = col, border = border)
##     }

## }




panel.3dpolygon <-
    function(x, y, z, rot.mat = diag(4), distance,
             xlim.scaled,
             ylim.scaled,
             zlim.scaled,
             zero.scaled,
             col = "white", 
             border = "black", 
             ## min.sides = 3,
             font, fontface, ## gpar() doesn't like these
             ...)
{
    if (all(is.na(x) | is.na(y) | is.na(z))) return()
    border <- 
        if (all(is.na(border)))
            "transparent"
        else if (is.logical(border))
        {
            if (border) "black"
            else "transparent"
        }
        else border
    m <- ltransform3dto3d(rbind(x, y, z), rot.mat, distance)
    ## ord <- sort.list(m[3,])
    n <- ncol(m)
    w <- which(is.na(x) | is.na(y))
    id.lengths <- diff(c(0, w, n))

    ## need to reorder multiple polygons by some measure of "average" depth

    id.long <- rep(seq_along(id.lengths), id.lengths)
    ord.depth <- order(tapply(m[3,], id.long, min, na.rm = TRUE))
    id.ordered <- ord.depth[id.long]

    ord.long <- order(id.ordered)
    
    grid.polygon(x = m[1, ord.long], y = m[2, ord.long],
                 id = id.ordered[ord.long],
                 default.units = "native",
                 gp =
                 gpar(fill = col,
                      col = border,
                      ...))

##     print(data.frame(x = m[1, ord.long],
##                      y = m[2, ord.long],
##                      id = id.ordered[ord.long]))

    return()
}




panel.3dtext <-
    function(x, y, z, labels = seq_along(x),
             rot.mat = diag(4), distance, ...)
{
    if (all(is.na(x) | is.na(y) | is.na(z))) return()
    m <- ltransform3dto3d(rbind(x, y, z), rot.mat, distance)
    ord <- sort.list(m[3,])
    panel.text(x = m[1, ord], y = m[2, ord], labels = labels, ...)
}



## d <- data.frame(x = rnorm(10),
##                 y = rnorm(10),
##                 z = rnorm(10))
## rownames(d) <- letters[1:10]

## cloud(z ~ x * y, d, panel.3d.cloud = panel.3dtext)
## cloud(z ~ x * y, d, panel.3d.cloud = panel.3dtext,
##       labels = rownames(d), col = "red")

## ## for multipanel plots

## cloud(z ~ x * y, d,
##       panel.3d.cloud = function(..., subscripts) {
##           panel.3dtext(..., labels = rownames(d)[subscripts])
##       })

##
## Copyright (c) 2010 Felix Andrews <felix@nfrac.org>
## GPL version 2 or newer

horizonplot <- function(x, data, ...)
    UseMethod("horizonplot")

horizonplot.default <-
    function(x, data = NULL, ...,
             nbands = 3L,
             horizonscale = NA,
             origin = function(y) na.omit(y)[1],
             colorkey = FALSE, legend = NULL,
             panel = panel.horizonplot,
             prepanel = prepanel.horizonplot,
             col.regions = hcl.colors(2 * nbands, palette="RdYlBu"),
             strip = FALSE, strip.left = TRUE,
             par.strip.text = list(cex = 0.6),
             colorkey.digits = 3,
             layout = c(1, NA),
             groups = NULL,
             default.scales =
               list(y = list(relation = "free", axs = "i",
                             draw = FALSE, tick.number = 2)))
{
    if (!is.null(groups))
        stop("'groups' does not work in this plot")
    ans <- xyplot(x, data = data, ...,
                  origin = origin, horizonscale = horizonscale,
                  panel = panel, prepanel = prepanel,
                  col.regions = col.regions,
                  strip = strip, strip.left = strip.left,
                  par.strip.text = par.strip.text,
                  layout = layout,
                  default.scales = default.scales,
                  nbands = nbands)
    ans$call <- match.call()
    ## add colorkey
    if (isTRUE(colorkey)) colorkey <- list()
    if (is.list(colorkey))
    {
        bands.at <- seq(-nbands, nbands)
        if (ans$y.scales$relation == "same") {
            origin <- ans$y.limits[1]
            horizonscale <- diff(ans$y.limits)
        }
        if (is.na(horizonscale)) {
            ## labels <- expression(
            ##    - 3 * Delta[i], - 2 * Delta[i], - 1 * Delta[i], 0,
            ##    + 1 * Delta[i], + 2 * Delta[i], + 3 * Delta[i], 0)
            labels <- parse(text = sprintf("%+d * Delta[i]", bands.at))
            labels[nbands + 1] <- if (is.numeric(origin)) origin else "origin"
        }
        else {
            if (is.numeric(origin)) {
                labels <- round(origin + bands.at * horizonscale, colorkey.digits)
            } else {
                labels <- sprintf("%+g", round(bands.at * horizonscale, colorkey.digits))
                labels[nbands + 1] <- "origin"
            }
        }
        ii <- round(seq(1, length(col.regions), length.out = 2 * nbands))
        colorkey <-
            modifyList(list(col = col.regions[ii], at = bands.at,
                            labels = list(labels = labels, at = bands.at)),
                       colorkey)
        space <- colorkey$space
        if (is.null(space)) space <- "right"
        if (is.null(legend)) legend <- list()
        legend[[space]] <- list(fun = "draw.colorkey",
                                args = list(colorkey)) 
        ans <- update(ans, legend = legend)
    }
    ans
}


panel.horizonplot <-
    function(x, y, ..., border = NA,
             nbands = 3L,
             col.regions = hcl.colors(2 * nbands, palette="RdYlBu"),
             origin) ## catch origin, don't pass to panel.xyarea!
{
    origin <- current.panel.limits()$ylim[1]
    scale <- diff(current.panel.limits()$ylim)
    ## ordered for drawing, from least extreme to most extreme
    #sections <- c(0, -1, 1, -2, 2, -3) ## these are the lower bounds
    sections <- as.vector(rbind(seq_len(nbands)-1, -seq_len(nbands)))
    #ii <- round(((sections + 3) / 5) * (length(col.regions)-1)) + 1
    ii <- round(((sections + nbands) / (2*nbands-1)) * (length(col.regions)-1)) + 1
    #ii <- sections + nbands + 1
    col <- col.regions[ii]
    for (i in seq_along(sections)) {
        section <- sections[i]
        yi <- y
        if (section < 0) {
            yi <- origin + origin - y
            section <- abs(section) - 1
        }
        baseline <- origin + section * scale
        if (all(yi <= baseline, na.rm = TRUE))
            next
        yi <- yi - baseline
        yi <- origin + pmax(pmin(yi, scale), 0)
        panel.xyarea(x, yi, border = border, col = col[i], col.line = col[i], ...)
    }
}

prepanel.horizonplot <-
    function(x, y, ..., horizonscale = NA,
             nbands = 3L,
             origin = function(y) na.omit(y)[1])
{
    if (is.function(origin))
        origin <- origin(y)
    ans <- prepanel.default.xyplot(x, y, ...)
    if (is.na(horizonscale))
        horizonscale <- max(abs(ans$ylim - origin)) / nbands
    ans$ylim <- origin + c(0, horizonscale)
    ans
}

## Construct a custom theme based on supplied colors.  Defaults to
## colors from RColorBrewer

custom.theme <-
    function(symbol = RColorBrewer::brewer.pal(n = 8, name = "Dark2"),
             fill = RColorBrewer::brewer.pal(n = 12, name = "Set3"),
             region = RColorBrewer::brewer.pal(n = 11, name = "Spectral"),
             reference = "#e8e8e8",
             bg = "transparent",
             fg = "black",
             ...)
{
    theme <-
        list(plot.polygon      = list(col = fill[1], border = fg[1]),
             box.rectangle     = list(col= symbol[1]),
             box.umbrella      = list(col= symbol[1]),
             dot.line          = list(col = reference),
             dot.symbol        = list(col = symbol[1]),
             plot.line         = list(col = symbol[1]),
             plot.symbol       = list(col= symbol[1]),
             regions           = list(col = colorRampPalette(region)(100)),
             reference.line    = list(col = reference),
             superpose.line    = list(col = symbol),
             superpose.symbol  = list(col = symbol),
             superpose.polygon = list(col = fill, border = fg),

             background        = list(col = bg),
             add.line          = list(col = fg),
             add.text          = list(col = fg),
             box.dot           = list(col = fg),
             axis.line         = list(col = fg),
             axis.text         = list(col = fg),
             strip.border      = list(col = fg),
             box.3d            = list(col = fg),
             par.xlab.text     = list(col = fg),
             par.ylab.text     = list(col = fg),
             par.zlab.text     = list(col = fg),
             par.main.text     = list(col = fg),
             par.sub.text      = list(col = fg))
    modifyList(modifyList(standard.theme("pdf"), theme),
               simpleTheme(...))
}

custom.theme.2 <- function(...)
{
    doit <-
        function(symbol = RColorBrewer::brewer.pal(n = 9, name = "Set1")[c(2:1, 3:5, 7:9)], ## blue first
                 fill = RColorBrewer::brewer.pal(n = 8, name = "Accent"),
                 region = RColorBrewer::brewer.pal(n = 11, name = "RdBu"),
                 ...)
        {
            custom.theme(symbol = symbol, fill = fill, region = region, ...)
        }
    doit(...)
}

panel.tskernel <-
    function(x, y, ..., 
             width = NROW(x) %/% 10 + 1, n = 300,
             c = 1, sides = 2, circular = FALSE,
             kern = kernel("daniell", rep(floor((width/sides)/sqrt(c)), c)))
{
    if (!missing(kern))
        .Deprecated("The 'kern' argument to panel.tskernel is Deprecated. Use simpleSmoothTs directly.")
    if (!missing(y)) {
        x <- as.numeric(x)
        y <- as.numeric(y)
        stopifnot(NCOL(x) == 1)
        if (diff(range(diff(x))) > getOption("ts.eps"))
            stop("'x' should be a regular series")
        x <- ts(y, start = x[1], end = tail(x,1), deltat = diff(x[1:2]))
    }
    x <- as.ts(x)
    s <- simpleSmoothTs(x, width = width, c = c, sides = sides,
                        circular = circular, kern = kern, n = n)
    panel.lines(s, ...)
}

prepanel.rootogram <-
    function(x, y = table(x),
             dfun = NULL,
             transformation = sqrt,
             hang = TRUE,
             probability = TRUE,
             ...)
{
    stopifnot(is.function(dfun))
    if (probability) y <- y / sum(y)
    yy <- transformation(y)
    xx <- sort(unique(x))
    dotArgs <- list(...)
    dfunArgs <- names(formals(dfun))
    if (!("..." %in% dfunArgs))
        dotArgs <- dotArgs[dfunArgs[-1]]
    dd <- transformation(do.call(dfun, c(list(xx), dotArgs)))
    list(xlim = range(xx),
         ylim =
         if (hang) range(dd, dd-yy, 0)
         else range(dd, yy, 0),
         dx = diff(xx),
         dy = diff(dd))
}


panel.rootogram <-
    function(x, y = table(x),
             dfun = NULL,
             col = plot.line$col,
             lty = plot.line$lty,
             lwd = plot.line$lwd,
             alpha = plot.line$alpha,
             transformation = sqrt,
             hang = TRUE,
             probability = TRUE,
             type = "l", pch = 16,
             ...)
{
    plot.line <- trellis.par.get("plot.line")
    ref.line <- trellis.par.get("reference.line")
    stopifnot(is.function(dfun))
    if (probability) y <- y / sum(y)
    yy <- transformation(y)
    xx <- sort(unique(x))
    dotArgs <- list(...)
    dfunArgs <- names(formals(dfun))
    if (!("..." %in% dfunArgs))
        dotArgs <- dotArgs[dfunArgs[-1]]
    dd <- transformation(do.call(dfun, c(list(xx), dotArgs)))
    panel.abline(h = 0,
                 col = ref.line$col,
                 lty = ref.line$lty,
                 lwd = ref.line$lwd,
                 alpha = ref.line$alpha)
    panel.segments(xx,
                   if (hang) dd else 0,
                   xx,
                   if (hang) (dd - yy) else yy,
                   col = col,
                   lty = lty,
                   lwd = lwd,
                   alpha = alpha,
                   ...)
    if ("l" %in% type) panel.lines(xx, dd)
    if ("p" %in% type) panel.points(xx, dd, pch = pch)
}


rootogram <-
    function(x, ...)
    UseMethod("rootogram")





rootogram.formula <-
    function(x, data = parent.frame(),
             ylab = expression(sqrt(P(X == x))),
             prepanel = prepanel.rootogram,
             panel = panel.rootogram,
             ...,
             probability = TRUE)
{
    if (!probability && missing(ylab)) ylab <- NULL
    if (length(x) == 2) ## formula like ~ x
        foo <-
            densityplot(x, data,
                        prepanel = prepanel,
                        panel = panel,
                        ylab = ylab,
                        ...,
                        probability = probability)
    else ## formula like y ~ x 
        foo <-
            xyplot(x, data,
                   prepanel = prepanel,
                   panel = panel,
                   ylab = ylab,
                   ...,
                   probability = probability)
    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(rootogram)
    foo
}

##
## Copyright (c) 2008 Felix Andrews <felix@nfrac.org>
##


panel.qqmath.tails <-
    function(x, f.value = NULL, distribution = qnorm,
    groups = NULL, ..., approx.n = 100, tails.n = 10)
{
    if (getRversion() >= "2.11.0") 
        .Deprecated(msg = paste("'panel.qqmath.tails' is deprecated.",
                    "Use 'panel.qqmath' from lattice 0.18-4 onwards."))
    x <- as.numeric(x)
    distribution <- if (is.function(distribution))
        distribution
    else if (is.character(distribution))
        get(distribution)
    else eval(distribution)
    nobs <- sum(!is.na(x))
    if (!is.null(groups)) {
        panel.superpose(x, y = NULL, f.value = f.value, distribution = distribution,
                        groups = groups, panel.groups = panel.qqmath.tails,
                        ..., approx.n = approx.n, tails.n = tails.n)
        return()
    }
    if (nobs == 0) return()
    pp <- ppoints(nobs)
    y <- sort(x)
    qq <- distribution(pp)
    if (nobs > approx.n + tails.n*2) {
        keep <- rep(FALSE, nobs)
        ## keep lowest and highest points
        keep[c(1:tails.n, nobs+1-(1:tails.n))] <- TRUE
        ## keep points spaced equally along distribution
        qqkeep <- seq(qq[tails.n+1], qq[nobs-tails.n], length=approx.n)
        keep[findInterval(qqkeep, qq)] <- TRUE
        qq <- qq[keep]
        y <- y[keep]
    }
    panel.xyplot(x = qq, y = y, ...)
}

## FIXME: want a convenience function that behaves like heatmap()





## Goal: create a grob that could usefully represent a dendrogram


## can a dendrogram node have more than 2 children?
## long term FIXME: use better graph layout algorithms?




## returns a modified dendrogram object, with an extra attribute
## 'position=c(x, y)' for each node

## FIXME: should have and honor 'center=FALSE' argument

addPositions <-
    function(x, order)
{
    if (!is.null(attr(x, "position"))) return(x)
    else if (is.leaf(x))
    {
        attr(x, "position") <-
            list(x = which(x == order)[1],
                 y = attr(x, "height"))
        return(x)
    }
    else
    {
        for (i in seq_along(x))
        {
            x[[i]] <- addPositions(x[[i]], order)
        }
        attr(x, "position") <-
            list(x = mean(sapply(x, function(x) attr(x, "position")$x )),
                 y = attr(x, "height"))
        return(x)
    }
}

## returns a vector data.frame(x0, y0, x1, y1, ...), to be used in a
## call to segmentsGrob after being combined.  The possibility of
## attaching parameters exists, but is not (or barely) tested

edgeLocation <-
    function(pos.node, pos.child, type, ...)
{
    switch(type,
           rectangle = {
               data.frame(x0 = c(pos.node$x, pos.child$x),
                          y0 = c(pos.node$y, pos.node$y),
                          x1 = c(pos.child$x, pos.child$x),
                          y1 = c(pos.node$y, pos.child$y),
                          ..., stringsAsFactors = FALSE) ## 'col' can be strings
           },
           triangle = {
               data.frame(x0 = pos.node$x,  y0 = pos.node$y,
                          x1 = pos.child$x, y1 = pos.child$y,
                          ..., stringsAsFactors = FALSE) ## 'col' can be strings
           })
}


dendrogramGrob <- 
    function(x, ord = order.dendrogram(x),
             side = c("right", "top"),
             add = list(),
             size = 5,
             size.add = 1,
             type = c("rectangle", "triangle"),
             ...)
{
    ## Note: We use dendrapply() a couple of times.  The return value
    ## is unused (we are only interested in side-effects), but certain
    ## types of return values of FUN can make dendrapply() go into an
    ## infinite loop.  To be safe, we return original node.

    if (size <= 0) return(textGrob(label = NULL))
    type <- match.arg(type)
    native.height <- attr(x, "height")
    native.xscale <- c(1, length(ord)) + c(-1, 1) * lattice.getOption("axis.padding")$factor
    xpos <- addPositions(x, ord) ## version of x with positions

    ## how many non-leaf nodes are there?  For a binary tree, n-1,
    ## where n is the number of leaves (join any 2 ==> nodes++,
    ## leaves--), but we're more tolerant

    nnodes <- 0
    dendrapply(xpos,
               function(x) {
                   if (!is.leaf(x)) nnodes <<- nnodes + 1
                   x
               })
    xseg <- vector(mode = "list", length = nnodes)

    ## FIXME: add something similar to have nodes drawn as points
    i <- 0
    getSegments <- function(x, ...)
    {
        if (!is.leaf(x))
        {
            i <<- i + 1
            pos.node <- attr(x, "position")
            xseg[[i]] <<-
                do.call(rbind,
                        lapply(x,
                               function(child) {
                                   pos.child <- attr(child, "position")
                                   edgeLocation(pos.node, pos.child,
                                                type = type,
                                                ...)
                               }))
        }
        x
    }
    dendrapply(xpos, getSegments)
    all.segs <- do.call(rbind, xseg)
    ## number of additional indicators
    nadd <- length(add)
    ## nleaf <- length(ord)
    native.unit <- 1 / diff(native.xscale) # side of one square
    
    switch(side,
           right = {
               key.layout <-
                   grid.layout(nrow = 1, ncol = 1 + nadd,
                               heights = unit(1, "null"),
                               widths =
                               unit(c(rep(size.add, length = nadd), size),
                                    c(rep("lines", nadd), "lines")),
                               respect = FALSE)
               key.gf <- frameGrob(layout = key.layout)
               ## key.gf <- placeGrob(key.gf, rectGrob(gp = gpar(fill = "pink")))
               for (i in seq_len(nadd))
               {
                   addi <- add[[i]]
                   typei <- names(add)[i]
                   switch(typei,
                          rect = {
                              key.gf <-
                                  placeGrob(key.gf,
                                            rectGrob(y = (order(ord) - native.xscale[1]) * native.unit,
                                                     height = native.unit,
                                                     gp = do.call(gpar, addi)),
                                            row = 1, col = i)
                          })
               }
               key.gf <-
                   placeGrob(key.gf, 
                             with(all.segs,
                                  segmentsGrob((y0 / native.height),
                                               (x0 - native.xscale[1]) * native.unit,
                                               (y1 / native.height),
                                               (x1 - native.xscale[1]) * native.unit)),
                             row = 1, col = 1 + nadd)
               key.gf
           },
           top = {
               key.layout <-
                   grid.layout(nrow = 1 + nadd, ncol = 1,
                               widths = unit(1, "null"),
                               heights =
                               unit(c(size, rep(size.add, length = nadd)),
                                    c("lines", rep("lines", nadd))),
                               respect = FALSE)

               key.gf <- frameGrob(layout = key.layout)
               ## key.gf <- placeGrob(key.gf, rectGrob(gp = gpar(fill = "pink")))

               for (i in seq_len(nadd))
               {
                   addi <- add[[i]]
                   typei <- names(add)[i]
                   switch(typei,
                          rect = {
                              key.gf <-
                                  placeGrob(key.gf,
                                            rectGrob(x = (order(ord) - native.xscale[1]) * native.unit,
                                                     width = native.unit,
                                                     gp = do.call(gpar, addi)),
                                            row = 1 + i, col = 1)
                          })
               }
               key.gf <-
                   placeGrob(key.gf, 
                             with(all.segs,
                                  segmentsGrob((x0 - native.xscale[1]) * native.unit,
                                               (y0 / native.height),
                                               (x1 - native.xscale[1]) * native.unit,
                                               (y1 / native.height))),
                             row = 1, col = 1)
               key.gf
           })
}

prepanel.mapplot <- function(x, y, map, ...) {
    list(xlim = range(map$x, finite = TRUE),
         ylim = range(map$y, finite = TRUE))
}

panel.mapplot <-
    function(x, y, map, breaks, colramp,
             exact = FALSE, lwd = 0.5, ...)
{
    names(x) <- tolower(as.character(y))
    mapnames <- tolower(map$names)
    mapval <- x[mapnames]
    xmatched <- names(x) %in% mapnames
    if (any(!xmatched) && !exact) {
        ## lump sub-regions together (strip name after ':')
        mapnames <- gsub(":.*$", "", mapnames)
        ## only replace values which did not match exactly
        mapval <- ifelse(is.na(mapval), x[mapnames], mapval)
        xmatched <- xmatched | (names(x) %in% mapnames)
    }
    if (any(!xmatched))
        warning(sum(!xmatched), " unmatched regions: ",
                toString(y[!xmatched], width = 60))
    interval <-
        cut(mapval, breaks = breaks,
            labels = FALSE, include.lowest = TRUE)
    col.regions <- colramp(length(breaks) - 1)
    col <- col.regions[interval]
    panel.polygon(map, col = col, lwd = lwd, ...)
}


mapplot <- function(x, data, ...) UseMethod("mapplot")


mapplot.formula <-
    function(x, data, map, outer = TRUE,
             prepanel = prepanel.mapplot,
             panel = panel.mapplot,
             aspect = "iso",
             legend = NULL,
             breaks, cuts = 30,
             colramp = colorRampPalette(hcl.colors(n = 11, palette = "Spectral")),
             colorkey = TRUE,
             ## col.regions,
             ## alpha.regions,
             ...)
{
    colrampNULL <- is.null(colramp)
    if (is.null(colramp))
        colramp <- function(n)
            colorRampPalette(trellis.par.get("regions")$col)(n)
    ccall <- match.call()
    ccall$data <- data
    ccall$map <- map
    ccall$outer <- outer
    ccall$prepanel <- prepanel
    ccall$panel <- panel
    ccall$aspect <- aspect
    ccall$legend <- legend
    ccall$colramp <- colramp
    ccall$default.scales <- list(x = list(tck = 1), y = list(tck = 1))
    ccall[[1]] <- quote(lattice::dotplot)
    ans <- eval(ccall, parent.frame())
    if (missing(breaks))
    {
        x <- unlist(lapply(ans$panel.args, "[[", "x"))
        breaks <-
            if (is.factor(x)) seq_len(1 + nlevels(x)) - 0.5
            else do.breaks(range(x, finite = TRUE), cuts)
    }
##     regions <- trellis.par.get("col.regions")
##     if (missing(col.regions)) col.regions <- regions$col
##     if (missing(alpha.regions)) alpha.regions <- regions$alpha
    if (colorkey) {
        keydef <- list(at = breaks, draw = FALSE)
        if (!colrampNULL) keydef$col <- colramp(length(breaks))
        ans <-
            update(ans,
                   breaks = breaks,
                   legend = lattice:::updateList(ans$legend,
                                                 list(right = 
                                                      list(fun = draw.colorkey,
                                                           args = list(key = keydef)))))
    }
    ans$call <- sys.call(sys.parent())
    ans$call[[1]] <- quote(mapplot)
    ans
}

panel.scaleArrow <-
    function(x = unit(0:1, "npc"), y = unit(0:1, "npc"),
             default.units = "npc",
             digits = 0, append = "", label = NULL,
             angle = 30, length = 0.5, unit = "char",
             type = "open", ends = "both",
             ...,
             col = add.line$col, fill = col, alpha = add.line$alpha,
             lty = add.line$lty, lwd = add.line$lwd,
             col.text = add.text$col, alpha.text = add.text$alpha)
{
    add.line <- trellis.par.get("add.line")
    add.text <- trellis.par.get("add.text")
    if (!is.unit(x)) x <- unit(x, default.units)
    if (!is.unit(y)) y <- unit(y, default.units)
    x <- rep(x, length.out = 2)
    y <- rep(y, length.out = 2)
    xnat <- convertX(x, "native", valueOnly = TRUE)
    ynat <- convertY(y, "native", valueOnly = TRUE)
    panel.arrows(xnat[1], ynat[1], xnat[2], ynat[2], ends = ends,
                 angle = angle, length = length, unit = unit, type = type,
                 col = col, fill = fill, alpha = alpha, lty = lty, lwd = lwd)
    #grid.lines(x = x, y = y, default.units = default.units,
    #           arrow = arrow, gp = gp)
    #xnat <- convertX(x, "native", valueOnly = TRUE)
    #ynat <- convertX(y, "native", valueOnly = TRUE)
    d <- sqrt(diff(xnat)^2 + diff(ynat)^2)
    d <- round(d, digits = digits)
    if (is.null(label)) label <- paste(d, append, sep = "")
    panel.text(x = 0.5 * (xnat[1] + xnat[2]), y = 0.5 * (ynat[1] + ynat[2]),
              labels = label, col = col.text, alpha = alpha.text, ...)
}

## based on the stat_smooth() function in ggplot2 package.

panel.smoother <-
    function(x, y, form = y ~ x, method = "loess", ...,
             se = TRUE, level = 0.95, n = 100,
             col = plot.line$col, col.se = col,
             lty = plot.line$lty, lwd = plot.line$lwd,
             alpha = plot.line$alpha,
             alpha.se = 0.25, border = NA,
             ## ignored (do not pass to method()):
             subscripts, group.number, group.value,
             type, col.line, col.symbol, fill,
             pch, cex, font, fontface, fontfamily)
{
    plot.line <- trellis.par.get("plot.line")
    if (!missing(col.line))
        col <- col.line
    ## allow 'form' to be passed as the first argument
    missing.x <- missing(x)
    if (!missing.x && inherits(x, "formula")) {
        form <- x
        missing.x <- TRUE
    }
    ## use 'x' and 'y' if given
    ## otherwise try to find them in the formula environment
    if (missing.x)
        x <- environment(form)$x
    if (missing(y))
        y <- environment(form)$y
    ok <- is.finite(x) & is.finite(y)
    if (sum(ok) < 1) 
        return()
    x <- as.numeric(x)[ok]
    y <- as.numeric(y)[ok]
    mod <- do.call(method,
                   c(alist(form, data = list(x = x, y = y)),
                     list(...)))
    ## use the limits of the data, or panel limits, whichever is smaller
    lims <- current.panel.limits()
    xrange <- c(max(min(lims$x), min(x)), min(max(lims$x), max(x)))
    xseq <- seq(xrange[1], xrange[2], length = n)
    pred <- predict(mod, data.frame(x = xseq), se = se)
    if (se) {
        std <- qnorm(level/2 + 0.5)
        panel.polygon(x = c(xseq, rev(xseq)),
                      y = c(pred$fit - std * pred$se,
                      rev(pred$fit + std * pred$se)),
                      col = col.se, alpha = alpha.se, border = border)
        pred <- pred$fit
    }
    panel.lines(xseq, pred, col = col, alpha = alpha,
                lty = lty, lwd = lwd)
}

## adapted from the Lattice book by Deepayan Sarkar

xscale.components.logpower <- function(lim, ...) {
    ans <- xscale.components.default(lim, ...)
    ans$bottom$labels$labels <- parse(text = ans$bottom$labels$labels)
    ans
}

yscale.components.logpower <- function(lim, ...) {
    ans <- yscale.components.default(lim, ...)
    ans$left$labels$labels <- parse(text = ans$left$labels$labels)
    ans
}

xscale.components.fractions <- function(lim, logsc = FALSE, ...) {
    ans <- xscale.components.default(lim, logsc = logsc, ...)
    ## get 'at' in data coordinates
    if (identical(logsc, TRUE)) logsc <- 10
    if (identical(logsc, "e")) logsc <- exp(1)
    at <- ans$bottom$labels$at
    if (!identical(logsc, FALSE))
        at <- logsc ^ at
    ans$bottom$labels$labels <- MASS::fractions(at)
    ans
}

yscale.components.fractions <- function(lim, logsc = FALSE, ...) {
    ans <- yscale.components.default(lim, logsc = logsc, ...)
    ## get 'at' in data coordinates
    if (identical(logsc, TRUE)) logsc <- 10
    if (identical(logsc, "e")) logsc <- exp(1)
    at <- ans$left$labels$at
    if (!identical(logsc, FALSE))
        at <- logsc ^ at
    ans$left$labels$labels <- MASS::fractions(at)
    ans
}

## compute nice log-ticks.  This is a version from the Lattice book
## that is not very sophisticated.

logTicksOld <- function (lim, loc = c(1, 5)) {
    ii <- floor(log10(range(lim))) + c(-1, 2)
    main <- 10^(ii[1]:ii[2])
    r <- as.numeric(outer(loc, main, "*"))
    r[lim[1] <= r & r <= lim[2]]
}

## A more sophisticated version that uses the same algorithm used in
## traditional graphics, via axisTicks() - new in R 2.14.0

logTicks <- function (lim, loc = NULL) {
    if (is.null(loc)) axisTicks(log10(lim), log=TRUE)
    else logTicksOld(lim, loc)
}

xscale.components.log <- function(lim, logsc = FALSE, at = NULL, loc = NULL, ...) {
    ans <- xscale.components.default(lim = lim, logsc = logsc, at = at, ...)
    if (is.null(at)) return(ans)
    if (identical(logsc, FALSE)) return(ans)
    logbase <- logsc
    if (identical(logbase, TRUE)) logbase <- 10
    if (identical(logbase, "e")) logbase <- exp(1)
    tick.at <- logTicks(logbase^lim, loc = loc)
    ans$bottom$ticks$at <- log(tick.at, logbase)
    ans$bottom$labels$at <- log(tick.at, logbase)
    ans$bottom$labels$labels <- as.character(tick.at)
    ans
}

yscale.components.log <- function(lim, logsc = FALSE, at = NULL, loc = NULL, ...) {
    ans <- yscale.components.default(lim = lim, logsc = logsc, at = at, ...)
    if (is.null(at)) return(ans)
    if (identical(logsc, FALSE)) return(ans)
    logbase <- logsc
    if (identical(logbase, TRUE)) logbase <- 10
    if (identical(logbase, "e")) logbase <- exp(1)
    tick.at <- logTicks(logbase^lim, loc = loc)
    ans$left$ticks$at <- log(tick.at, logbase)
    ans$left$labels$at <- log(tick.at, logbase)
    ans$left$labels$labels <- as.character(tick.at)
    ans
}

xscale.components.log10.3 <- function(lim, logsc = FALSE, at = NULL, ...) {
    xscale.components.log(lim, logsc = logsc, at = at, loc = c(1, 3)) 
}

yscale.components.log10.3 <- function(lim, logsc = FALSE, at = NULL, ...) {
    yscale.components.log(lim, logsc = logsc, at = at, loc = c(1, 3))
}


# major + minor ticks for powers of 10

xscale.components.log10ticks <- function(lim, logsc = FALSE, at = NULL, ...) {
    ans <- xscale.components.default(lim = lim, logsc = logsc, at = at, ...)
    if (is.null(at)) return(ans)
    if (identical(logsc, FALSE)) return(ans)
    logbase <- logsc
    if (identical(logbase, TRUE)) logbase <- 10
    if (identical(logbase, "e")) logbase <- exp(1)
    tick.at <- logTicks(logbase^lim, loc = 1:9)
    tick.at.major <- logTicks(logbase^lim, loc = 1)
    major <- tick.at %in% tick.at.major
    ans$bottom$ticks$at <- log(tick.at, logbase)
    ans$bottom$ticks$tck <- ifelse(major, 1, 0.5)
    ans$bottom$labels$at <- log(tick.at, logbase)
    ans$bottom$labels$labels <- as.character(tick.at)
    ans$bottom$labels$labels[!major] <- ""
    ans$bottom$labels$check.overlap <- FALSE
    ans
}

yscale.components.log10ticks <- function(lim, logsc = FALSE, at = NULL, ...) {
    ans <- yscale.components.default(lim = lim, logsc = logsc, at = at, ...)
    if (is.null(at)) return(ans)
    if (identical(logsc, FALSE)) return(ans)
    logbase <- logsc
    if (identical(logbase, TRUE)) logbase <- 10
    if (identical(logbase, "e")) logbase <- exp(1)
    tick.at <- logTicks(logbase^lim, loc = 1:9)
    tick.at.major <- logTicks(logbase^lim, loc = 1)
    major <- tick.at %in% tick.at.major
    ans$left$ticks$at <- log(tick.at, logbase)
    ans$left$ticks$tck <- ifelse(major, 1, 0.5)
    ans$left$labels$at <- log(tick.at, logbase)
    ans$left$labels$labels <- as.character(tick.at)
    ans$left$labels$labels[!major] <- ""
    ans$left$labels$check.overlap <- FALSE
    ans
}


## major + minor ticks (e.g. for date/time axes):

xscale.components.subticks <-
    function(lim, ..., n = 5, n2 = n * 5, min.n2 = n + 5)
{
    ans <- xscale.components.default(lim = lim, ..., n = n)
    ans2 <- xscale.components.default(lim = lim, ..., n = n2, min.n = min.n2)
    ticks <- ans$bottom$ticks$at
    ticks2 <- ans2$bottom$ticks$at
    ticks2 <- ticks2[!(ticks2 %in% ticks)]
    ans$bottom$ticks$at <- c(ticks, ticks2)
    ans$bottom$ticks$tck <- c(rep(1, length(ticks)),
                              rep(0.5, length(ticks2)))
    ans$bottom$labels$at <- ans$bottom$ticks$at
    ans$bottom$labels$labels <- c(ans$bottom$labels$labels,
                                  rep(" ", length(ticks2)))
    ans$bottom$labels$check.overlap <- FALSE
    ans
}

yscale.components.subticks <-
    function(lim, ..., n = 5, n2 = n * 5, min.n2 = n + 5)
{
    ans <- yscale.components.default(lim = lim, ..., n = n)
    ans2 <- yscale.components.default(lim = lim, ..., n = n2, min.n = min.n2)
    ticks <- ans$left$ticks$at
    ticks2 <- ans2$left$ticks$at
    ticks2 <- ticks2[!(ticks2 %in% ticks)]
    ans$left$ticks$at <- c(ticks, ticks2)
    ans$left$ticks$tck <- c(rep(1, length(ticks)),
                            rep(0.5, length(ticks2)))
    ans$left$labels$at <- ans$left$ticks$at
    ans$left$labels$labels <- c(ans$left$labels$labels,
                                rep(" ", length(ticks2)))
    ans$left$labels$check.overlap <- FALSE
    ans
}

xyplot.stl <-
    function(x, data = NULL,
             outer = TRUE,
             layout = c(1, 4),
             strip = FALSE,
             strip.left = TRUE,
             as.table = TRUE,
             ylab = "",
             between = list(y = 0.5),
             panel =
             function(..., type) {
                 if (packet.number() == 4) type <- "h"
                 panel.xyplot(..., type = type)
             },
             ...)
{
    stopifnot(is.null(data))
    mstrip <- missing(strip.left)
    sers <- x$time.series
    ## ncomp <- ncol(sers)
    data <- rowSums(sers)
    X <- cbind(data, sers)
    colnames(X) <- c("data", colnames(sers))
    ans <-
        xyplot(X,
               outer = outer,
               layout = layout,
               strip = strip,
               strip.left = strip.left,
               as.table = as.table,
               ylab = ylab,
               between = between,
               panel = panel,
               ...,
               default.scales =
               list(x = list(axs = "i"),
                    y =
                    list(relation = "free",
                         tick.number = 3,
                         rot = 0)))
    if (mstrip)
    {
        mx <- min(rx <- abs(sapply(ans$y.limits, diff)))
        int <- cbind(-mx / rx, mx / rx)
        ans <-
            update(ans,
                   strip.left =
                   strip.custom(horizontal = FALSE,
                                strip.names = FALSE,
                                strip.levels = TRUE,
                                shingle.intervals = int))
    }
    ans$call <- sys.call(sys.parent()); ans$call[[1]] <- quote(xyplot)
    ans
}

# data ellipses for xyplot
# M. Friendly, 7/7/2010 9:24AM
# - replaced panel.polygon with panel.lines

# calculate one data ellipse; modified from car to do just the calculation

conf.ellipse <- function(x, y, level = 0.68, segments = 50L, robust = FALSE)
{
    if (length(x) != length(y))
        stop("x and y must be vectors of the same length")
    xy <- na.omit(cbind(x, y))
    if (robust)
    {
        v <- MASS::cov.trob(xy)
        shape <- v$cov
        center <- v$center
    }
    else
    {
        shape <- var(xy)
        center <- colMeans(xy)
    }
    radius <- sqrt(2 * qf(level, df1 = 2, df2 = length(x) - 1))
    angles <- seq(0, 2 * pi, length.out = segments + 1)
    unit.circle <- cbind(cos(angles), sin(angles))
    ell <- t(center + radius * t(unit.circle %*% chol(shape)))
    list(ellipse = ell, center = center)
}

panel.ellipse <-
    function(x, y, groups = NULL,
             level = 0.68, segments = 50, robust = FALSE,
             center.pch = 3, center.cex = 2, 
             ..., type, pch, cex)
{
    x <- as.numeric(x)
    y <- as.numeric(y)
    if (!is.null(groups))
        panel.superpose(x = x, y = y, groups = groups,
                        level = level, segments = segments, robust = robust,
                        center.pch = center.pch, center.cex = center.cex,
                        panel.groups = panel.ellipse, ...)
    else
    {
        ell <- conf.ellipse(x = x, y = y, level = level, segments = segments, robust = robust)
        panel.xyplot(ell$ellipse[,1], ell$ellipse[,2], ..., type = "l")
        if (!is.null(center.pch))
            panel.xyplot(ell$center[1], ell$center[2], ...,
                         pch = center.pch, cex = center.cex)
    }
}


if (FALSE)
{

    old.panel.ellipse <-
        function(x, y, groups=NULL, level=0.68, segments=51, robust=FALSE,
                 col=NA, col.line = if (is.null(groups)) plot.line$col else superpose.line$col,
                 lwd = if (is.null(groups)) plot.line$lwd else superpose.line$lwd, 
                 lty = if (is.null(groups)) plot.line$lty else superpose.line$lty,
                 center.pch=3, center.cex=2, 
                 ...)
        {
            x <- as.numeric(x)
            y <- as.numeric(y)
            plot.line <- trellis.par.get("plot.line")
            superpose.line <- trellis.par.get("superpose.line")

            if (!is.na(col))
            {
                if (missing(col.line)) col.line <- col
            }
            
            groups <- as.factor(if(is.null(groups)) rep(1, length(x)) else as.character(groups))
            n.groups <- length(levels(groups))
            col <- rep(col.line, length.out=n.groups)
            lty <- rep(lty, length.out=n.groups)
            lwd <- rep(lwd, length.out=n.groups)
            for (i in 1:n.groups)
            {
                subs <- groups == levels(groups)[i]
                XY <- na.omit(data.frame(x=x[subs], y=y[subs]))
                ell <- .ellipse(XY, level = level,
                                segments = segments,
                                robust = robust)
                ## panel.polygon(ell[,1], ell[,2], border=col[i], col="transparent", lty=lty[i], lwd=lwd[i], ...)
                panel.lines(ell[,1], ell[,2], col = col[i], lty = lty[i], lwd = lwd[i], ...)
                if (!is.null(center.pch)) {
                    center = colMeans(XY)
                    panel.points(center[1], center[2], col=col[i],
                                 pch = center.pch, cex = center.cex)
                }
            }
        }

}

## extracted and simplified from lattice:::plot.trellis

panel.key <-
    function(text, ..., corner = c(0, 1), x = corner[1], y = corner[2])
{
    key <- simpleKey(text, ...)
    key.gf <- draw.key(key, draw = FALSE)
    vp <- viewport(x = unit(x, "npc") + unit(0.5 - corner[1], "grobwidth", list(key.gf)),
                   y = unit(y, "npc") + unit(0.5 - corner[2], "grobheight", list(key.gf)))
    pushViewport(vp)
    grid.draw(key.gf)
    upViewport()
}

\name{biocAccess}
\alias{biocAccess}
\docType{data}
\title{ Hourly access attempts to Bioconductor website }
\description{
  This data set records the hourly number of access attempts to
  the Bioconductor website (\url{https://www.bioconductor.org}) during
  January through May of 2007.  The counts are essentially an
  aggregation of the number of entries in the access log.
}
\usage{data(biocAccess)}
\format{
  A data frame with 3623 observations on the following 7 variables.
  \describe{
    \item{\code{counts}}{ the number of access attempts}
    \item{\code{day}}{ the day of the month }
    \item{\code{month}}{a factor with levels \code{Jan}, \code{Feb},
      \dots, \code{Dec}}
    \item{\code{year}}{ the year (all 2007) }
    \item{\code{hour}}{ hour of the day, a numeric vector}
    \item{\code{weekday}}{ a factor with levels \code{Monday},
      \code{Tuesday}, \dots, \code{Sunday}}
    \item{\code{time}}{a POSIXt representation of the start of the hour}
  }
}
\examples{
data(biocAccess)
xyplot(stl(ts(biocAccess$counts[1:(24 * 30)], frequency = 24), "periodic"))
}
\keyword{datasets}

\name{panel.xyarea}
\alias{panel.xyarea}
\alias{panel.xyarea.default}
\alias{panel.xyarea.ts}
\alias{panel.xyarea.zoo}
\alias{panel.qqmath.xyarea}
\title{
  Plot series as filled polygons.
}
\description{
  Plot series as filled polygons connected at given origin level (on y axis).
}
\usage{
panel.xyarea(x, ...)

\method{panel.xyarea}{default}(x, y, groups = NULL, origin = NULL, horizontal = FALSE,
             col, col.line, border, lty, lwd, alpha, ...,
             fill, panel.groups = panel.xyarea)

\method{panel.xyarea}{ts}(x, y = x, ...)
\method{panel.xyarea}{zoo}(x, y = x, ...)

panel.qqmath.xyarea(x, y = NULL, f.value = NULL, distribution = qnorm,
                    qtype = 7, groups = NULL, ..., tails.n = 0)
}
\arguments{
  \item{x, y}{
    data vectors.
  }
  \item{groups}{
    a factor defining groups.
  }
  \item{origin}{
    level on y axis to connect the start and end of the series.
    If \code{NULL}, the polygon is filled to the bottom of the panel.
    It is flipped if \code{horizontal = TRUE}.
  }
  \item{horizontal}{
    if this is set to \code{TRUE}, then the origin is a level on the x
    axis, rather than the default which is on the y axis. This is the
    opposite of what you might expect, but is for consistency with
    \code{panel.xyplot}. 
  }
  \item{col, col.line, border, lty, lwd, alpha}{
    graphical parameters taken from
    \code{trellis.par.get("plot.polygon")} or
    \code{trellis.par.get("superpose.polygon")} (when groups defined).
    \code{col.line} overrides \code{col}.
  }
  \item{\dots}{
    further arguments passed on to \code{\link[lattice]{panel.polygon}}.
    For \code{panel.qqmath.xyarea}, passed to \code{panel.xyarea}.
  }
  \item{fill}{
    ignored; use \code{col} instead.
  }
  \item{panel.groups}{
    used in \code{\link[lattice]{panel.superpose}}.
  }
  \item{f.value, distribution, qtype, tails.n}{
    as in \code{\link[lattice]{panel.qqmath}}.
  }
}
\details{
  none yet.
}
\author{
  Felix Andrews \email{felix@nfrac.org}
}
\seealso{
  \code{\link[lattice]{panel.xyplot}},
  \code{\link[lattice]{panel.polygon}}
}
\examples{
xyplot(sunspot.year, panel = panel.xyarea, origin = 0,
  aspect = "xy", cut = list(n = 3, overlap = 0))

## two series superposed: one filled, one as a line.
xyplot(ts.union(data = sunspot.year, lag10 = lag(sunspot.year, 10)),
  aspect = "xy", cut = list(n = 3, overlap = 0),
  superpose = TRUE,
  panel = panel.superpose,
  panel.groups = function(..., group.number) {
    if (group.number == 1)
      panel.xyarea(...) else panel.xyplot(...)
  }, border = NA, 
  par.settings = simpleTheme(col = c("grey", "black"), lwd = c(5,2)))

## missing values are handled by splitting the series
tmp <- window(sunspot.year, start = 1900)
tmp[c(1:2, 50:60)] <- NA
xyplot(tmp, panel = panel.xyarea, origin = 0)

set.seed(0)
qqmath(~ data, make.groups(baseline = rnorm(100),
                           other = rnorm(100) * 2 - 0.5),
  groups = which, distribution = qunif,
  panel = panel.qqmath.xyarea, xlim = 0:1,
  auto.key = list(points = FALSE, rectangles = TRUE),
  par.settings = simpleTheme(col = c("blue", "green"),
                             alpha = 0.5))
}
\keyword{ dplot }

\name{ggplot2like.theme}
\alias{ggplot2like.theme}
\alias{ggplot2like.opts}
\alias{ggplot2like}
\alias{axis.grid}
\title{A ggplot2-like theme for Lattice}
\description{
  A theme for Lattice based on some of the default styles used in the
  \pkg{ggplot2} package by Hadley Wickham. Specifically, parts of the
  functions \code{scale_colour_hue}, \code{scale_colour_gradient} and
  \code{theme_gray} were copied. Although superficially similar, the
  implementation here lacks much of the flexibility of the \pkg{ggplot2}
  functions: see \url{http://had.co.nz/ggplot2/}.
}

\usage{
ggplot2like(..., n = 6, h = c(0,360) + 15, l = 65, c = 100,
            h.start = 0, direction = 1,
            low = "#3B4FB8", high = "#B71B1A", space = "rgb")

ggplot2like.opts()

axis.grid(side = c("top", "bottom", "left", "right"),
          ..., ticks = c("default", "yes", "no"),
          scales, components, line.col)
}
\arguments{
  \item{\dots}{
    further arguments passed on to \code{\link[lattice]{simpleTheme}} to
    over-ride defaults.
  }
  \item{n}{
    number of \code{superpose} styles to generate, with equally spaced hues.
  }
  \item{h, c, l, h.start, direction}{
    range of hues, starting hue and direction to generate a discrete
    colour sequence with \code{\link{hcl}}.
  }
  \item{low, high, space}{
    extreme colors to interpolate with \code{\link{colorRampPalette}}
    for a continuous color scale.
  }
  \item{side, ticks, scales, components, line.col}{
    see \code{\link[lattice]{axis.default}}. Typically \code{axis.grid} is not
    called directly so these should not be needed.
  }
}

\value{
  \code{ggplot2like()} produces a list of settings which can be
  passed as the \code{par.settings} argument to a high-level Lattice
  plot, or to \code{\link[lattice]{trellis.par.set}}.
  \code{ggplot2like.opts()} produces a list which can be passed as the
  \code{lattice.options} argument to a high-level Lattice plot, or to
  \code{\link[lattice]{lattice.options}}. 

}
\author{
  Felix Andrews \email{felix@nfrac.org};
  copied and adapted from \pkg{ggplot2} by Hadley Wickham.
}
\seealso{
  the \pkg{ggplot2} package: \url{http://had.co.nz/ggplot2/}.
  
  \code{\link{custom.theme}}, \code{\link{scale.components}}
}

\examples{
set.seed(0)

## basic theme does not include white grid lines
xyplot(exp(1:10) ~ 1:10, type = "b", 
    par.settings = ggplot2like())

## add lines for axis ticks with custom axis function
xyplot(exp(1:10) ~ 1:10, type = "b", 
    par.settings = ggplot2like(), axis = axis.grid)
    
## this can be used together with scale.components
## (minor lines only visible on devices supporting translucency)
xyplot(exp(rnorm(500)) ~ rnorm(500),
    scales = list(y = list(log = TRUE)),
    yscale.components = yscale.components.log10ticks,
    par.settings = ggplot2like(), axis = axis.grid)

## ggplotlike.opts() specifies axis = axis.grid as well as
## xscale.components.subticks / yscale.components.subticks

xyg <- make.groups(
    "group one" = rnorm(80, 1),
    "group two" = rnorm(80, 5),
    "group three" = rnorm(80, 2))
xyg$x <- rev(xyg$data)

## group styles: specify number of equi-spaced hues
xyplot(data ~ x, xyg, groups = which, auto.key = TRUE, 
    par.settings = ggplot2like(n = 3),
    lattice.options = ggplot2like.opts()) +
  glayer(panel.smoother(...))

## or set it as the default:
opar <- trellis.par.get()
trellis.par.set(ggplot2like(n = 4, h.start = 180))
oopt <- lattice.options(ggplot2like.opts())

bwplot(voice.part ~ height, data = singer)

histogram(rnorm(100))

barchart(Titanic[,,,"No"], main = "Titanic deaths",
    layout = c(1, 2), auto.key = list(columns = 2))

## reset
trellis.par.set(opar)
lattice.options(oopt)

## axis.grid and scale.components.subticks can be used alone:
## (again, lines for minor ticks need transculency-support to show up)
xyplot(exp(1:10) ~ 1:10, type = "b", 
    lattice.options = ggplot2like.opts(),
    par.settings = list(axis.line = list(col = NA),
        reference.line = list(col = "grey")),
    scales = list(tck = c(0,0)))
}

\keyword{dplot}

\name{panel.quantile}
\alias{panel.quantile}
\title{
  Plot a quantile regression line with standard error bounds.
}
\description{
  Plot a quantile regression line with standard error bounds, using the
  \pkg{quantreg} package. This is based on the
  \code{\link[ggplot2]{stat_quantile}} function from \pkg{ggplot2}.
}
\usage{
panel.quantile(x, y, form = y ~ x, method = "rq", ...,
    tau = 0.5, ci = FALSE, ci.type = "default", level = 0.95,
    n = 100, col = plot.line$col, col.se = col,
    lty = plot.line$lty, lwd = plot.line$lwd,
    alpha = plot.line$alpha, alpha.se = 0.25, border = NA,
    superpose = FALSE,
    ## ignored: ##
    subscripts, group.number, group.value,
    type, col.line, col.symbol, fill,
    pch, cex, font, fontface, fontfamily)
}
\arguments{
  \item{x, y}{
    data points. If these are missing, they will be looked for in the
    environment of \code{form}. So in many cases you can skip these if
    passing \code{form}. In fact, for convenience, the formula
    can be passed as the first argument (i.e. \code{x}).
  }
  \item{form, method}{
    Formula and the name of a modeling function (as a character
    string). The model is constructed (approximately) as
    \code{method(form, tau = tau, data = list(x=x, y=y), ...)}.
    Currently, \code{method} is expected to be either \code{"rq"} or
    \code{"rqss"}, and for these the corresponding functions in the
    \pkg{quantreg} package is used. In principle, any other function
    that supports the same interface can also be specified by name.
  }
  \item{\dots}{
    further arguments passed on to the model function (\code{method}),
    typically \code{\link[quantreg]{rq}}.
  }
  \item{tau}{
    \var{p} values for the quantiles to estimate.

    Note: only one value for \code{tau} can be specified if
    estimating confidence intervals with \code{ci}.
  }
  \item{ci, ci.type, level}{
    estimate a confidence interval at level \code{level} using the
    method \code{ci.type}; see \code{\link[quantreg]{predict.rq}}.
  }
  \item{n}{
    number of equi-spaced points on which to evaluate the function.
  }
  \item{col, col.se, lty, lwd, alpha, alpha.se, border}{
    graphical parameters. \code{col} and \code{alpha} apply to the
    line(s), while \code{col.se} and \code{alpha.se}
    apply to the shaded \code{ci} region.
  }
  \item{superpose}{
    if \code{TRUE}, plot each quantile line (\code{tau}) in a
    different style (using \code{trellis.par.get("superpose.line")}).
  }
  \item{subscripts, group.number, group.value,
    type, col.line, col.symbol, fill,
    pch, cex, font, fontface, fontfamily}{
    ignored.
  }
}
\details{
  It is recommended to look at \code{vignette("rq", package="quantreg")}.
}
\author{
  Felix Andrews \email{felix@nfrac.org}

  Based on \code{\link[ggplot2]{stat_quantile}} by Hadley Wickham.
}
\seealso{
  \code{\link[quantreg]{rq}},
  \code{\link{panel.smoother}},
  \code{\link[ggplot2]{stat_quantile}}
}
\examples{
## library("quantreg")

set.seed(1)
xy <- data.frame(x = runif(100), y = rt(100, df = 5))
xyplot(y ~ x, xy) +
    layer(panel.quantile(x, y, tau = c(.95, .5, .05)))

if (require("splines")) {
    xyplot(y ~ x, xy) +
        layer(panel.quantile(y ~ ns(x, 3), tau = 0.9))

    xyplot(y ~ x, xy) +
        layer(panel.quantile(y ~ ns(x, 3), tau = 0.9, ci = TRUE))
}

xyplot(y ~ x, xy) +
    layer(panel.quantile(x, y, tau = c(.5, .9, .1), superpose = TRUE))

update(trellis.last.object(),
       auto.key = list(text = paste(c(50,90,10), "\% quantile"),
                  points = FALSE, lines = TRUE))

xyplot(y ~ x, xy) +
    layer(panel.quantile(y ~ qss(x, lambda=1), method = "rqss"))

}
\keyword{ dplot }

\name{rootogram}
\alias{rootogram}
\alias{rootogram.formula}
\alias{panel.rootogram}
\alias{prepanel.rootogram}
\title{Trellis Displays of Tukey's Hanging Rootograms}
\description{
  Displays hanging rootograms. 
}
\usage{
rootogram(x, \dots)

\method{rootogram}{formula}(x, data = parent.frame(),
          ylab = expression(sqrt(P(X == x))),
          prepanel = prepanel.rootogram,
          panel = panel.rootogram,
          ...,
          probability = TRUE)

prepanel.rootogram(x, y = table(x),
                   dfun = NULL,
                   transformation = sqrt,
                   hang = TRUE,
                   probability = TRUE,
                   \dots)

panel.rootogram(x, y = table(x),
                dfun = NULL,
                col = plot.line$col,
                lty = plot.line$lty,
                lwd = plot.line$lwd,
                alpha = plot.line$alpha,
                transformation = sqrt,
                hang = TRUE,
                probability = TRUE,
                type = "l", pch = 16,
                \dots)

}
\arguments{
  \item{x, y}{ For \code{rootogram}, \code{x} is the object on which
    method dispatch is carried out.  For the \code{"formula"} method,
    \code{x} is a formula describing the form of conditioning plot.  The
    formula can be either of the form \code{~x} or of the form
    \code{y~x}.  In the first case, \code{x} is assumed to be a vector
    of raw observations, and an observed frequency distribution is
    computed from it.  In the second case, \code{x} is assumed to be
    unique values and \code{y} the corresponding frequencies.  In either
    case, further conditioning variables are allowed.

    A similar interpretation holds for \code{x} and \code{y} in
    \code{prepanel.rootogram} and \code{panel.rootogram}.

    Note that the data are assumed to arise from a discrete distribution
    with some probability mass function.  See details below.
  }

  \item{data}{ For the \code{"formula"} method, a data frame containing
    values for any variables in the formula, as well as those in
    \code{groups} and \code{subset} if applicable (\code{groups} is
    currently ignored by the default panel function).  By default the
    environment where the function was called from is used.  }

  \item{dfun}{ a probability mass function, to be evaluated at unique x
    values }

  \item{prepanel, panel}{ panel and prepanel function used to create the
    display.  }

  \item{ylab}{ the y-axis label; typically a character string or an
    expression. }

  \item{col, lty, lwd, alpha}{ graphical parameters }
  
  \item{transformation}{ a vectorized function.  Relative frequencies
    (observed) and theoretical probabilities (\code{dfun}) are
    transformed by this function before being plotted. }

  \item{hang}{logical, whether lines representing observed relative
    freuqncies should \dQuote{hang} from the curve representing the
    theoretical probabilities. }

  \item{probability}{ A logical flag, controlling whether the y-values
    are to be standardized to be probabilities by dividing by their sum.
  }

  \item{type}{ A character vector consisting of one or both of
    \code{"p"} and \code{"l"}. If \code{"p"} is included, the evaluated
    values of \code{dfun} will be denoted by points, and if \code{"l"}
    is included, they will be joined by lines. }

  \item{pch}{ The plotting character to be used for the \code{"p"}
    type. }
  
  \item{\dots}{ extra arguments, passed on as appropriate.  Standard
    lattice arguments as well as arguments to \code{panel.rootogram}
    can be supplied directly in the high level \code{rootogram} call.
  }
}

\details{

  This function implements Tukey's hanging rootograms.  As implemented,
  \code{rootogram} assumes that the data arise from a discrete
  distribution (either supplied in raw form, when \code{y} is
  unspecified, or in terms of the frequency distribution) with some
  unknown probability mass function (p.m.f.).  The purpose of the plot
  is to check whether the supplied theoretical p.m.f. \code{dfun} is a
  reasonable fit for the data.

  It is reasonable to consider rootograms for continuous data by
  discretizing it (similar to a histogram), but this must be done by the
  user before calling \code{rootogram}.  An example is given below.

  Also consider the \code{rootogram} function in the \code{vcd} package,
  especially if the number of unique values is small.

}

\value{

  \code{rootogram} produces an object of class \code{"trellis"}.  The
  \code{update} method can be used to update components of the object and
  the \code{print} method (usually called by default) will plot it on an
  appropriate plotting device.

}

\references{

  John W. Tukey (1972) Some graphic and semi-graphic displays. In
  T. A. Bancroft (Ed) \emph{Statistical Papers in Honor of George
  W. Snedecor}, pp. 293--316.  Available online at
  \url{https://www.edwardtufte.com/notebook/john-tukey-classic-paper-on-statistical-graphics/}

}

\author{ Deepayan Sarkar \email{deepayan.sarkar@gmail.com}}
\seealso{
  \code{\link[lattice]{xyplot}}
}

\examples{

library(lattice)

x <- rpois(1000, lambda = 50)

p <- rootogram(~x, dfun = function(x) dpois(x, lambda = 50))
p

lambdav <- c(30, 40, 50, 60, 70)

update(p[rep(1, length(lambdav))],
       aspect = "xy",
       panel = function(x, ...) {
           panel.rootogram(x,
                           dfun = function(x)
                           dpois(x, lambda = lambdav[panel.number()]))
       })


lambdav <- c(46, 48, 50, 52, 54)

update(p[rep(1, length(lambdav))],
       aspect = "xy",
       prepanel = function(x, ...) {
           tmp <-
               lapply(lambdav,
                      function(lambda) {
                          prepanel.rootogram(x,
                                             dfun = function(x)
                                             dpois(x, lambda = lambda))
                      })
           list(xlim = range(sapply(tmp, "[[", "xlim")),
                ylim = range(sapply(tmp, "[[", "ylim")),
                dx = do.call("c", lapply(tmp, "[[", "dx")),
                dy = do.call("c", lapply(tmp, "[[", "dy")))
       },
       panel = function(x, ...) {
           panel.rootogram(x,
                           dfun = function(x)
                           dpois(x, lambda = lambdav[panel.number()]))
           grid::grid.text(bquote(Poisson(lambda == .(foo)),
                                  where = list(foo = lambdav[panel.number()])),
                           y = 0.15,
                           gp = grid::gpar(cex = 1.5))
       },
       xlab = "",
       sub = "Random sample from Poisson(50)")


## Example using continuous data

xnorm <- rnorm(1000)

## 'discretize' by binning and replacing data by bin midpoints

h <- hist(xnorm, plot = FALSE)

## Option 1: Assume bin probabilities proportional to dnorm()

norm.factor <- sum(dnorm(h$mids, mean(xnorm), sd(xnorm)))

rootogram(counts ~ mids, data = h,
          dfun = function(x) {
              dnorm(x, mean(xnorm), sd(xnorm)) / norm.factor
          })

## Option 2: Compute probabilities explicitly using pnorm()

pdisc <- diff(pnorm(h$breaks, mean = mean(xnorm), sd = sd(xnorm)))
pdisc <- pdisc / sum(pdisc)

rootogram(counts ~ mids, data = h,
          dfun = function(x) {
              f <- factor(x, levels = h$mids)
              pdisc[f]
          })

}

\keyword{dplot}

\name{resizePanels}
\alias{resizePanels}
\title{ Resize panels to match data scales }
\description{

  Modify a \code{"trellis"} object so that when
  plotted, the panels have the specified relative width and height.

}
\usage{

resizePanels(x, h = 1, w = 1)

}

\arguments{
  \item{x}{ An object of class \code{"trellis"}. }
  \item{h}{ numeric vector specifying panel heights}
  \item{w}{ numeric vector specifying of panel widths}
}

\details{

  \code{resizePanels} modifies a \code{"trellis"} object so that when
  plotted, the panels have the specified relative width and height; this
  is only interesting when \code{h} or \code{w} are vectors with unequal
  entries.  \code{resizePanels} can be called with no arguments, in
  which case the currently plotted \code{"trellis"} object (if any) is
  used for \code{x}, and a suitable \code{h} or \code{w} (based on the
  current panel layout) is chosen so that sizes are relative to the
  current panel ranges in the native coordinate system.  This is only
  interesting when \code{scales="free"}; the resulting object, when
  plotted again, will have varying panel sizes but the same number of
  data units per inch in all panels.

}

\value{

  An object of class \code{"trellis"}; essentially the same as \code{x},
  but with certain properties modified.

}
\author{ Deepayan Sarkar }

\seealso{ \code{\link[lattice]{Lattice}}, \code{\link[lattice]{xyplot}} }

\examples{

state <- data.frame(state.x77, state.region, state.name)
state$state.name <-
    with(state, reorder(reorder(state.name, Frost),
                        as.numeric(state.region)))
dpfrost <-
    dotplot(state.name ~ Frost | reorder(state.region, Frost),
            data = state, layout = c(1, 4),
            scales = list(y = list(relation = "free")))

## approximate
resizePanels(dpfrost,
             h = with(state, table(reorder(state.region, Frost))))

## exact (including boundary padding)
resizePanels()

}

\keyword{dplot}

\name{panel.key}
\alias{panel.key}
\title{
  Draw a simple key inside a panel of a lattice plot.
}
\description{
  Draw a \code{\link[lattice]{simpleKey}} inside a panel of a lattice plot.
}
\usage{
panel.key(text, ..., corner = c(0, 1), x = corner[1], y = corner[2])
}
\arguments{
  \item{text, \dots}{
    entries in the \code{\link[lattice]{simpleKey}}.
  }
  \item{corner, x, y}{
    defines the position of the key within the panel viewport. These are
    given in normalised coordinates between 0 and 1. The two elements
    of \code{corner} specify the x and y positions respectively.
  }
}
\seealso{
  \code{\link[lattice]{simpleKey}},
  \code{\link[lattice]{draw.key}},
  \code{\link[lattice]{xyplot}}
}
\examples{
xyplot(ozone ~ wind | equal.count(temperature, 2),
       data = environmental) +
   layer(panel.loess(x, y, span = 0.5), style = 1) +
   layer(panel.loess(x, y, span = 1.0), style = 2) +
   layer(panel.key(c("span = 0.5", "span = 1.0"), corner = c(1,.98),
                   lines = TRUE, points = FALSE), packets = 1)
}
\keyword{ aplot }

\name{panel.tskernel}
\alias{panel.tskernel}
\alias{simpleSmoothTs}
\alias{simpleSmoothTs.default}
\alias{simpleSmoothTs.zoo}
\title{
  Calculate and plot smoothed time series.
}
\description{
  Plot time series smoothed by discrete symmetric smoothing kernels.
  These kernels can be used to smooth time series objects.
  Options include moving averages, triangular filters, or
  approximately Gaussian filters.
}
\usage{
panel.tskernel(x, y, ...,
    width = NROW(x) \%/\% 10 + 1, n = 300,
    c = 1, sides = 2, circular = FALSE,
    kern = kernel("daniell",
           rep(floor((width/sides) / sqrt(c)), c)))

simpleSmoothTs(x, ...)

\method{simpleSmoothTs}{default}(x, ...,
    width = NROW(x) \%/\% 10 + 1, n = NROW(x),
    c = 1, sides = 2, circular = FALSE,
    kern = kernel("daniell",
           rep(floor((width/sides)/sqrt(c)), c)))

\method{simpleSmoothTs}{zoo}(x, ..., n = NROW(x))
}
\arguments{
  \item{x, y}{
    data points. Should define a regular, ordered series.
    A time series object can be passed as the first argument, in which
    case \code{y} can be omitted. The \code{x} argument given to
    \code{simpleSmoothTs} is allowed to be a multivariate time series,
    i.e. to have multiple columns.
  }
  \item{\dots}{
    further arguments passed on to \code{\link[lattice]{panel.lines}}.
  }
  \item{width}{
    nominal width of the smoothing kernel in time steps. In the default
    case, which is a simple moving average, this is the actual width.
    When \code{c > 1} the number of time steps used in the kernel
    increases but the equivalent bandwidth stays the same.
    If only past values are used (with \code{sides = 1}) then
    \code{width} refers to one side of the symmetric kernel.
  }
  \item{n}{
    approximate number of time steps desired for the result. If this is
    less than the length of \code{x}, the smoothed time series will be
    aggregated by averaging blocks of (an integer number of) time
    steps, and this aggregated series will be centered with respect to
    the original series.
  }
  \item{c}{
    smoothness of the kernel: \code{c = 1} is a moving average,
    \code{c = 2} is a triangular kernel, \code{c = 3} and higher
    approximate smooth Gaussian kernels.
    \code{c} is actually the number of times to
    recursively convolve a simple moving average kernel with itself.
    The kernel size is adjusted to maintain a constant equivalent
    bandwidth as \code{c} increases.
  }
  \item{sides}{
    if \code{sides=1} the smoothed series is calculed from past values
    only (using one half of the symmetric kernel); if \code{sides=2} it
    is centred around lag 0.
  }
  \item{circular}{
    to treat the data as circular (periodic).
  }
  \item{kern}{
    a \code{tskernel} object; if given, this over-rides \code{width} and
    \code{c}.
  }
}
\note{
  The author is not an expert on time series theory.
}
\author{
  Felix Andrews \email{felix@nfrac.org}
}
\seealso{
  \code{\link{kernel}},
  \code{\link{filter}},
  \code{\link[lattice]{xyplot.ts}}
}
\examples{
## a Gaussian-like filter (contrast with c = 1 or c = 2)
xyplot(sunspot.year) +
layer(panel.tskernel(x, y, width = 20, c = 3, col = 1, lwd = 2))

## example from ?kernel:
## long and short moving averages, backwards in time
xyplot(EuStockMarkets[,1]) +
  layer(panel.tskernel(x, y, width = 100, col = 1, sides = 1)) +
  layer(panel.tskernel(x, y, width = 20, col = 2, sides = 1))

## per group, with a triangular filter
xyplot(EuStockMarkets, superpose = TRUE) +
  glayer(panel.tskernel(..., width = 100, c = 2),
         theme = simpleTheme(lwd = 2))

## plot the actual kernels used; note adjustment of width
width = 100
kdat <- lapply(1:4, function(c) {
    k <- kernel("daniell", rep(floor(0.5*width / sqrt(c)), c))
    ## demonstrate that the effective bandwidth stays the same:
    message("c = ", c, ": effective bandwidth = ", bandwidth.kernel(k))
    ## represent the kernel as a time series, for plotting
    ts(k[-k$m:k$m], start = -k$m)
})
names(kdat) <- paste("c =", 1:4)
xyplot(do.call(ts.union, kdat), type = "h",
    scales = list(y = list(relation = "same")))
}
\keyword{ dplot }

\name{theEconomist.theme}
\alias{theEconomist.theme}
\alias{theEconomist.opts}
\alias{theEconomist.axis}
\alias{asTheEconomist}
\title{
  Generate plots with style like The Economist.
}
\description{
  Uses colors, drawing styles, axis settings, etc, to produce plots
  inspired by those in The Economist magazine.
  (\url{https://www.economist.com/}).
}
\usage{
theEconomist.theme(win.fontfamily = NULL,
    with.bg = FALSE, box = "black", \dots)

theEconomist.opts()

asTheEconomist(x, ...,
    type = "l",
    ylab = expression(NULL),
    xlab = expression(NULL),
    par.settings =
      theEconomist.theme(with.bg = with.bg, box = "transparent"),
    with.bg = FALSE,
    par.strip.text = list(font = 2))
}
\arguments{
  \item{win.fontfamily}{
    on Windows systems, sets the font by name.
  }
  \item{with.bg}{
    if \code{TRUE}, uses a light blue background and a few other
    corresponding changes; otherwise white.
  }
  \item{box}{
    color for panel boxes, strip outlines, and axis ticks.
  }
  \item{\dots}{
    further arguments passed to \code{\link[lattice]{simpleTheme}}
    and used to modify the theme.
  }
  \item{x}{
    a \code{trellis} object, i.e. the result of a high-level plot
    function in the Lattice framework.
  }
  \item{type}{
    plot type, relevant for \code{xyplot}s, see \code{\link[lattice]{panel.xyplot}}.
  }
  \item{ylab, xlab}{
    axis labels, blank by default.
  }
  \item{par.settings}{
    style settings, defaulting to \code{theEconomist.theme}.
  }
  \item{par.strip.text}{
    see \code{\link[lattice]{xyplot}}.
  }
}
\details{
  You can just use \code{par.settings = theEconomist.theme()}, which
  gives you some colors and styles, but it does not do the
  grid lines or axis settings.
}
\references{
  \url{https://www.economist.com/displayStory.cfm?story_id=15065782}

  \url{https://www.economist.com/displayStory.cfm?story_id=14941181}
}
\author{
  Felix Andrews \email{felix@nfrac.org}
}
\seealso{
  \code{\link{custom.theme}}
}
\examples{
xyplot(window(sunspot.year, start = 1900),
  main = "Sunspot cycles", sub = "Number per year",
  par.settings = theEconomist.theme(box = "transparent"),
  lattice.options = theEconomist.opts())

asTheEconomist(xyplot(window(sunspot.year, start = 1900),
  main = "Sunspot cycles", sub = "Number per year"))

trellis.last.object() +
  layer_(panel.xblocks(x, x >= 1980, col = "#6CCFF6", alpha = .5)) +
  layer(panel.text(1988, 180, "Forecast", font = 3, pos = 2))

## set as defaults -- remember to set back when finished.
opar <- trellis.par.get()
trellis.par.set(theEconomist.theme(box = "transparent"))
oopt <- lattice.options(theEconomist.opts())

barchart(Titanic[,,,"No"], main = "Titanic deaths", layout = 1:2,
    sub = "by sex and class", auto.key = list(columns = 2),
    scales = list(y = list(alternating = 2)))

asTheEconomist(
  dotplot(VADeaths, main = "Death Rates in Virginia (1940)",
    auto.key = list(corner = c(.9,0.1))),
  type = "b", with.bg = TRUE)

dotplot(VADeaths, auto.key = TRUE, type = "b",
  par.settings = theEconomist.theme(with.bg = TRUE))

asTheEconomist(
  densityplot(~ height, groups = voice.part, data = singer,
    subset = grep("1", voice.part), plot.points = FALSE)) +
glayer(d <- density(x), i <- which.max(d$y),
  ltext(d$x[i], d$y[i], paste("Group", group.number), pos = 3))

## reset
trellis.par.set(opar)
lattice.options(oopt)
}
\keyword{ dplot }

\name{EastAuClimate}
\alias{EastAuClimate}
\docType{data}
\title{ Climate of the East Coast of Australia }
\description{
  A set of climate statistics for 16 coastal locations along Eastern
  Australia. These sites were chosen to be approximately equally
  spaced to cover the whole eastern coast of Australia.
  For each site, climate statistics were calculated for the standard
  30-year period 1971-2000. Only sites with nearly-complete data were
  chosen.
}
\usage{data(EastAuClimate)}
\format{
  A data frame with the following 10 variables and 5 items of metadata
  for each of 16 sites.
  \describe{
    \item{\code{SummerMaxTemp}}{ average daily maximum air
      temperature (degrees C) in February. }
    \item{\code{SummerMinTemp}}{ average daily minimum air
      temperature (degrees C) in February. }
    \item{\code{WinterMaxTemp}}{ average daily maximum air
      temperature (degrees C) in July. }
    \item{\code{WinterMinTemp}}{ average daily minimum air
      temperature (degrees C) in July. }
    \item{\code{SummerRain}}{ median total precipitation in February
      (mm/month). }
    \item{\code{WinterRain}}{ median total precipitation in July
      (mm/month). }
    \item{\code{MeanAnnRain}}{ average total amount of precipitation
      recorded in a year (mm/year). }
    \item{\code{RainDays}}{ average number of days in a year with at
      least 1 mm of precipitation. }
    \item{\code{ClearDays}}{ average number of clear days in a
      year. This statistic is derived from cloud cover observations,
      which are measured in oktas (eighths). A clear day is recorded
      when the mean of the 9 am and 3 pm cloud observations is less than
      or equal to 2 oktas. }
    \item{\code{CloudyDays}}{ average number of clear days in a
      year. A cloudy day is recorded when the mean of the 9 am and 3 pm
      cloud observations is greater than or equal to 6 oktas. }
    \item{\code{ID}}{ BOM Site number. }
    \item{\code{Latitude}}{ Site latitude (degrees North). }
    \item{\code{Longitude}}{ Site longitude (degrees East). }
    \item{\code{Elevation}}{ Site elevation (m). }
    \item{\code{State}}{ Australian state:
      TAS = Tasmania, VIC = Victoria, NSW = New
      South Wales, QLD = Queensland. }
  }

  The row names of the data frame give the location names. Note: these
  are not the official names of the climate stations.
}
\source{
  Sites were chosen by hand from maps on the Bureau of Meteorology
  website. The data were extracted manually from web pages under
  \url{http://www.bom.gov.au/climate/}
  and processed to extract a subset of statistics.
  - by Felix Andrews \email{felix@nfrac.org}

  Bureau of Meteorology, Commonwealth of Australia.
  Product IDCJCM0026 Prepared at Wed 31 Dec 2008.

  Definitions of statistics adapted from
  \url{http://www.bom.gov.au/climate/cdo/about/about-stats.shtml}
}
%map(regions = c("Australia", "Australia:Tasmania"),
%          exact = TRUE, xlim = c(130, 170))
%with(EastAuClimate, {
%  points(Longitude, Latitude, pch = 16)
%  text(Longitude, Latitude, row.names(EastAuClimate),
%       pos = c(4, 4, 2, 2, rep(4, 12)))
%})
\examples{
data(EastAuClimate)

## Compare the climates of state capital cities
EastAuClimate[c("Hobart", "Melbourne", "Sydney", "Brisbane"),]

## A function to plot maps (a Lattice version of maps::map)
lmap <-
   function(database = "world", regions = ".", exact = FALSE,
            boundary = TRUE, interior = TRUE, projection = "",
            parameters = NULL, orientation = NULL,
            aspect = "iso", type = "l",
            par.settings = list(axis.line = list(col = "transparent")),
            xlab = NULL, ylab = NULL, ...)
{
   theMap <- map(database, regions, exact = exact,
                 boundary = boundary, interior = interior,
                 projection = projection, parameters = parameters,
                 orientation = orientation, plot = FALSE)
   xyplot(y ~ x, theMap, type = type, aspect = aspect,
          par.settings = par.settings, xlab = xlab, ylab = ylab,
          default.scales = list(draw = FALSE), ...)
}

## Plot the sites on a map of Australia
if (require("maps")) {
  lmap(regions = c("Australia", "Australia:Tasmania"),
       exact = TRUE, projection = "rectangular",
       parameters = 150, xlim = c(130, 170),
       panel = function(...) {
          panel.xyplot(...)
          with(EastAuClimate, {
            panel.points(Longitude, Latitude, pch = 16)
            txt <- row.names(EastAuClimate)
            i <- c(3, 4)
            panel.text(Longitude[ i], Latitude[ i], txt[ i], pos = 2)
            panel.text(Longitude[-i], Latitude[-i], txt[-i], pos = 4)
          })
       })
}

## Average daily maximum temperature in July (Winter).
xyplot(WinterMaxTemp ~ Latitude, EastAuClimate, aspect = "xy",
       type = c("p", "a"), ylab = "Temperature (degrees C)")

## (Make a factor with levels in order - by coastal location)
siteNames <- factor(row.names(EastAuClimate),
           levels = row.names(EastAuClimate))
## Plot temperature ranges (as bars), color-coded by RainDays
segplot(siteNames ~ WinterMinTemp + SummerMaxTemp, EastAuClimate,
        level = RainDays, sub = "Color scale: number of rainy days per year",
        xlab = "Temperature (degrees C)",
        main = paste("Typical temperature range and wetness",
           "of coastal Australian cities", sep = "\n"))

## Show Winter and Summer temperature ranges separately
segplot(Latitude ~ WinterMinTemp + SummerMaxTemp, EastAuClimate,
   main = "Average daily temperature ranges \n of coastal Australian sites",
   ylab = "Latitude", xlab = "Temperature (degrees C)",
   par.settings = simpleTheme(lwd = 3, alpha = 0.5),
   key = list(text = list(c("July (Winter)", "February (Summer)")),
              lines = list(col = c("blue", "red"))),
   panel = function(x, y, z, ..., col) {
      with(EastAuClimate, {
         panel.segplot(WinterMinTemp, WinterMaxTemp, z, ..., col = "blue")
         panel.segplot(SummerMinTemp, SummerMaxTemp, z, ..., col = "red")
      })
   })

## Northern sites have Summer-dominated rainfall;
## Southern sites have Winter-dominated rainfall.
xyplot(SummerRain + WinterRain ~ Latitude, EastAuClimate,
       type = c("p", "a"), auto.key = list(lines = TRUE),
       ylab = "Rainfall (mm / month)")

## Clear days are most frequent in the mid latitudes.
xyplot(RainDays + CloudyDays + ClearDays ~ Latitude, EastAuClimate,
       type = c("p", "a"), auto.key = list(lines = TRUE),
       ylab = "Days per year")
}
\keyword{datasets}

\name{c.trellis}
\alias{c.trellis}
\alias{xyplot.list}
\alias{mergedTrellisLegendGrob}
\title{ Merge trellis objects, using same or different scales }
\description{
  Combine the panels of multiple trellis objects into one.
}
\usage{
\method{c}{trellis}(..., x.same = NA, y.same = NA,
    layout = NULL, merge.legends = FALSE, recursive = FALSE)

xyplot.list(x, data = NULL, ..., FUN = xyplot,
    y.same = TRUE, x.same = NA, layout = NULL,
    merge.legends = FALSE)
}
\arguments{
  \item{\dots}{ two or more trellis objects. If these are
    named arguments, the names will be used in the corresponding
    panel strips.
  }
  \item{x.same}{ if \code{TRUE}, set the x scale relation to
    \code{"same"} and recalculate panel limits using data
    from all panels. Otherwise, the x scales in each panel
    will be as they were in the original objects
    (so in general not the same), the default behaviour.
  }
  \item{y.same}{ as above, for y scales.
    Note that \code{xyplot.list} defaults to same y scales. Set to
    \code{NA} to leave them alone. }
  \item{layout}{ value for \code{layout} of the new plot; see
    \code{\link[lattice]{xyplot}}. }
  \item{merge.legends}{ to keep keys or legends from all plots, not just
    the first. If multiple legends share the same \code{"space"}, they are
    packed together horizontally or vertically. }
  \item{recursive}{ for consistency with the generic method, ignored. }
  \item{x}{ a \code{list} of objects to plot individually, and then
    be combined into one final plot. }
  \item{FUN, data}{ a lattice plot function, to be called on each element of
    the list \code{x}, along with \code{data} and \code{...} }
}
\details{
  This mechanism attempts to merge the panels from multiple trellis
  objects into one. The same effect could generally be achieved by
  either a custom panel function (where the display depends on
  \code{packet.number()}), or using \code{\link[lattice]{print.trellis}}
  to display multiple trellis objects. However, in some cases it is more
  convenient to use \code{c()}. Furthermore, it can be useful to
  maintain the display as a standard lattice display, rather than a
  composite using \code{\link[lattice]{print.trellis}}, to simplify
  further interaction.

  Many properties of the display, such as titles, axis settings
  and aspect ratio will be taken from the first object only.

  Note that combining panels from different types of plots does not
  really fit the trellis model. Some features of the plot may not work
  as expected. In particular, some work may be needed to
  show or hide scales on selected panels. An example is given below.

  Any trellis object with more than one conditioning variable will be
  "flattened" to one dimension, eliminating the multi-variate
  conditioning structure.
}
\value{
  a new trellis object.
}
\author{ Felix Andrews \email{felix@nfrac.org} }
\seealso{
  \code{\link{marginal.plot}} was the original motivating application,
  \code{\link[lattice]{print.trellis}},
  \code{\link[lattice]{update.trellis}},
  \code{\link[lattice]{trellis.object}}
}
\examples{

## Combine different types of plots.
c(wireframe(volcano), contourplot(volcano))

## Merging levelplot with xyplot
levObj <- levelplot(prop.table(WorldPhones, 1) * 100)
xyObj <- xyplot(Phones ~ Year, data.frame(Phones = rowSums(WorldPhones),
      Year = row.names(WorldPhones)), type="b", ylim = c(0, 150000))
## NOTE: prepanel.levelplot (from first object) is used for entire plot.
cObj <- c(levObj, xyObj, layout = 1:2)
update(cObj, scales = list(y = list(rot = 0)),
       ylab = c("proportional distribution", "number of phones"))

## Combine two xyplots.
sepals <- xyplot(Sepal.Length ~ Sepal.Width, iris, groups = Species,
    xlab = "Width", ylab = "Height")
petals <- xyplot(Petal.Length ~ Petal.Width, iris, groups = Species)
c(Sepals = sepals, Petals = petals)

## Force same scales (re-calculate panel limits from merged data):
c(Sepals = sepals, Petals = petals, x.same = TRUE, y.same = TRUE)

## Or - create xyplots from a list of formulas
xyplot.list(list(Sepals = Sepal.Length ~ Sepal.Width,
                 Petals = Petal.Length ~ Petal.Width),
             data = iris, groups = Species, x.same = TRUE,
             xlab = "Width", ylab = "Height")

## Create histograms from a list of objects, and merge them.
xyplot.list(iris, FUN = histogram)

## Create cumulative distribution plots from a list of objects
xyplot.list(iris[1:4], FUN = qqmath, groups = iris$Species,
            auto.key = TRUE)

## Display a table as both frequencies and proportions:
data(postdoc)
## remove last row (containing totals)
postdoc <- postdoc[1:(nrow(postdoc)-1),]
pdprops <- barchart(prop.table(postdoc, margin = 1),
                    auto.key = list(adj = 1))
pdmargin <- barchart(margin.table(postdoc, 1))
pdboth <- c(pdprops, pdmargin)
update(pdboth, xlab = c("Proportion", "Freq"))

## Conditioned 'quakes' plot combined with histogram.
qua <- xyplot(lat ~ long | equal.count(depth, 3), quakes,
    aspect = "iso", pch = ".", cex = 2, xlab = NULL, ylab = NULL)
qua <- c(qua, depth = histogram(quakes$depth), layout = c(4, 1))
## suppress scales on the first 3 panels
update(qua, scales = list(at = list(NULL, NULL, NULL, NA),
                          y = list(draw = FALSE)))

## Demonstrate merging of legends and par.settings.
## Note that par.settings can conflict, thus need col.line=...
mypoints <- 
    xyplot(1:10 ~ 1:10, groups = factor(rep(1:2, each = 5)),
        par.settings = simpleTheme(pch = 16), auto.key = TRUE)
mylines <-
    xyplot(1:10 ~ 1:10, groups = factor(rep(1:5, each = 2)),
        type = "l", par.settings = simpleTheme(col.line = 1:5),
        auto.key = list(lines = TRUE, points = FALSE, columns = 5))
c(mypoints, mylines)

## Visualise statistical and spatial distributions
## (advanced!). Requires the 'maps' package to be installed

if (require(maps)) {
    vars <- as.data.frame(state.x77)
    StateName <- tolower(state.name)
    form <- StateName ~ Population + Income + Illiteracy +
        `Life Exp` + Murder + `HS Grad` + Frost + sqrt(Area)
    ## construct independent maps of each variable
    statemap <- map("state", plot = FALSE, fill = TRUE)
    colkey <- draw.colorkey(list(col = heat.colors(100), at = 0:100,
                                 labels = list(labels = c("min","max"), at = c(0,100))))
    panel.mapplot.each <- function(x, breaks, ...)
        panel.mapplot(x = x, breaks = quantile(x), ...)
    vmaps <- mapplot(form, vars, map = statemap, colramp = heat.colors,
                     panel = panel.mapplot.each, colorkey = FALSE,
                     legend = list(right = list(fun = colkey)), xlab = NULL)
    ## construct independent densityplots of each variable
    vdens <- densityplot(form[-2], vars, outer = TRUE, cut = 0, 
                         scales = list(relation = "free"), ylim = c(0, NA),
                         cex = 0.5, ref = TRUE) +
        layer(panel.axis("top", half = FALSE, text.cex = 0.7))
    ## combine panels from both plots
    combo <- c(vmaps, vdens)
    ## rearrange in pairs
    n <- length(vars)
    npairs <- rep(1:n, each = 2) + c(0, n)
    update(combo[npairs], scales = list(draw = FALSE),
           layout = c(4, 4), between = list(x = c(0, 0.5), y = 0.5))
}
}
\keyword{ aplot }

\name{as.layer}
\alias{as.layer}
\alias{as.layer.layer}
\alias{as.layer.trellis}
\title{ Overlay panels of Trellis plots on same or different scales }
\description{
  Allows overlaying of Trellis plots, drawn on the same scales or on
  different scales in each of the x and y dimensions.
  There are options for custom axes and graphical styles.
}
\usage{
as.layer(x, ...)

\S3method{as.layer}{trellis}(x, x.same = TRUE, y.same = TRUE,
         axes = c(if (!x.same) "x", if (!y.same) "y"), opposite = TRUE,
         outside = FALSE, theme = x$par.settings, ...)
}
\arguments{
  \item{x}{ a trellis object. }
  \item{x.same}{ retains the existing panel x scale for the new layer,
    rather than using the layer's native x scale. }
  \item{y.same}{ retains the existing panel y scale. }
  \item{axes}{ which of the axes to draw (\code{NULL} for neither).
    Axes might not be drawn anyway, such as if
    \code{scales$draw == FALSE}. }
  \item{opposite}{ whether to draw axes on the opposite side to normal:
         that is, the top and/or right sides rather than bottom and/or
         left. May be a vector of length 2 to specify for x and y axes
         separately. }
  \item{outside}{ whether to draw the axes outside the plot region. Note
         that space for outside axes will not be allocated
         automatically. May be a vector of length 2 to specify for x and y axes
         separately. }
  \item{theme}{ passed to \code{\link{layer}}. }
  \item{\dots}{ passed to \code{\link{layer}}: typically
    the \code{style} argument would be specified. }
}
\details{
  Panels from the trellis object \code{x} will be drawn in the
  corresponding panel of another trellis object, so packet numbers match
  (see examples).

  Axis setting are taken from the trellis object \code{x}, so most
  \code{scales} arguments such as \code{draw}, \code{at}, \code{labels} etc will
  carry over to the overlaid axes. Only the main axis settings are used
  (i.e. left or bottom), even when \code{opposite = TRUE}.

  Currently, outside top axes will be drawn in the strip if there are strips.
}
\value{
  an updated trellis object.
}
\author{ Felix Andrews \email{felix@nfrac.org} }
\seealso{
  \code{\link{doubleYScale}},
  \code{\link{layer}},
  \code{\link[lattice]{panel.axis}}
}
\examples{
b1 <- barley
b2 <- barley
b2$yield <- b2$yield + 10

## panels are matched up by packet number
dotplot(variety ~ yield | site * year, b1) +
  as.layer(dotplot(variety ~ yield | site * year, b2, col = "red"))

## which gives the same result as:
dotplot(variety ~ yield | site * year, data = b1, subscripts = TRUE) +
  layer(panel.dotplot(yield[subscripts], variety[subscripts], col = "red"),
        data = b2)


## example with all same scales (the default):
xyplot(fdeaths ~ mdeaths) +
  as.layer(xyplot(fdeaths ~ mdeaths, col = 2, subset = ldeaths > 2000))

## same x scales, different y scales:
xyplot(fdeaths ~ mdeaths) +
  as.layer(bwplot(~ mdeaths, box.ratio = 0.2), y.same = FALSE)

## same y scales, different x scales:
xyplot(fdeaths ~ mdeaths) +
  as.layer(bwplot(fdeaths ~ factor(mdeaths*0), box.ratio = 0.2), x.same = FALSE)

## as.layer() is called automatically if two plots are added:
histogram(~ ldeaths, type = "density") + densityplot(~ ldeaths, lwd = 3)

## applying one panel layer to several panels of another object
xyplot(Sepal.Length + Sepal.Width ~ Petal.Length + Petal.Width | Species,
       data = iris, scales = "free") +
   as.layer(levelplot(volcano), x.same = FALSE, y.same = FALSE, under = TRUE)
}
\keyword{ aplot }

\name{layer}
\alias{layer}
\alias{layer_}
\alias{glayer}
\alias{glayer_}
\alias{+.trellis}
\alias{drawLayer}
\alias{flattenPanel}
\alias{[.layer}
\alias{print.layer}
\title{ Add layers to a lattice plot, optionally using a new data source }
\description{
  A mechanism to add new layers to a trellis object, optionally using a
  new data source. This is an alternative to modifying the
  panel function. Note the non-standard evaluation in \code{layer()}.
}
\usage{
layer(..., data, magicdots, exclude,
      packets, rows, columns, groups,
      style, force, theme, under, superpose)

layer_(...)
glayer(...)
glayer_(...)

\S3method{+}{trellis}(object, lay)

drawLayer(lay, panelArgs = trellis.panelArgs())

flattenPanel(object)
}
\arguments{
  \item{\dots}{
    expressions as they would appear in a panel function. These can
    refer to the panel function arguments (such as \code{x}, \code{y}
    and \code{subscripts}), and also to any named objects passed in
    through the \code{data} argument. The calls can also include the
    special argument \dQuote{\code{\dots}}; in the default case of
    \code{magicdots = TRUE}, only those arguments which are not already
    named in a call are passed on through \dQuote{\code{\dots}}.
    Otherwise, \dQuote{\code{\dots}} simply represents all panel
    function arguments. See Details, below.
  }
  \item{data}{
    optional. A named \code{list} containing objects needed when
    evaluating (drawing) the layer.
  }
  \item{magicdots, exclude}{
    if \code{magicdots = TRUE}, the default, any reference to
    \dQuote{\code{\dots}} 
    in the layer expressions will only pass on those arguments from the
    panel function which are not named in the call (thus avoiding 
    duplicate argument errors). If the first argument in a call is not
    named, it is assumed to be named \code{"x"}, and if the second
    argument is not named it is assumed to be named
    \code{"y"}. Furthermore, any argument names given in \code{exclude}
    will not be passed on through \dQuote{\code{\dots}}.
  }
  \item{packets, rows, columns, groups}{
    restricts the layer to draw only in specified packets (which refer
    to individual panels, but are independent of their layout), or rows
    or columns of the trellis layout (\code{\link[lattice]{trellis.currentLayout}}).
    For group layers (using \code{glayer} or \code{superpose = TRUE}),
    the groups can be restricted also, by specifying group numbers (or
    group values, as character strings). Negative values exclude the
    given items. 
  }
  \item{style}{
    style index of the layer, used only to set lattice graphical
    parameters (same effect as in grouped displays). Note that this will
    use the theme settings in effect in the existing plot, which may or
    may not be what is desired. It may be necessary to use \code{force =
      TRUE} to escape from the plot's settings and use the current
    theme. 
  }
  \item{force}{
    \code{force = TRUE} is just a shorthand for
    \code{theme = trellis.par.get()}, which is useful for over-riding
    the theme settings in effect in an existing plot. For instance, if
    the original plot specified \code{par.settings = simpleTheme(col =
      "red")} then the theme settings in effect will be entirely
    red. Use \code{force = TRUE} to reset the current theme for this
    layer, or use \code{theme} directly.
  }
  \item{theme}{
    a style specification to be passed to \code{\link[lattice]{trellis.par.set}}
    which has effect only while drawing the layer. One can pass a whole
    theme specification list, such as \code{theme = custom.theme()}, or
    a more specific list, such as \code{theme = simpleTheme(col = "red")}.
  }
  \item{under}{
    whether the layer should be drawn before the existing panel
    function. This defaults to \code{TRUE} in the convenience functions
    \code{layer_()} and \code{glayer_()}.
  }
  \item{superpose}{
    if \code{TRUE}, the layer will be drawn once for
    each level of any \code{groups} in the plot, using
    \code{\link[lattice]{panel.superpose}}. This defaults to \code{TRUE} in the
    convenience functions \code{glayer()} and \code{glayer_()}.
  }
  \item{object}{ a trellis object. }
  \item{lay}{ a layer object. }
  \item{panelArgs}{ list of arguments to the panel function. }
}
\details{
  The \code{layer} mechanism is a method for augmenting a panel
  function. It allows expressions to be added to the panel function
  without knowing what the original panel function was. In this way it
  can be useful for convenient augmentation of trellis plots.

  Note that the evaluation used in \code{layer} is non-standard, and can
  be confusing at first: you typically refer to variables as if inside
  the panel function (\code{x}, \code{y}, etc); you can usually refer to
  objects which exist in the global environment (workspace), but it is
  safer to pass them in by name in the \code{data} argument to
  \code{layer}. (And this should not to be confused with the \code{data}
  argument to the original \code{xyplot}.)

  A simple example is adding a reference line to each panel:
  \code{layer(panel.refline(h = 0))}. Note that the expressions are
  quoted, so if you have local variables they will need to be either
  accessible globally, or passed in via the \code{data} argument. For
  example:

  \code{layer(panel.refline(h = myVal)) ## if myVal is global}

  \code{layer(panel.refline(h = h), data = list(h = myVal))}

  Another non-standard aspect is that the special argument
  \dQuote{\code{\dots}} will, by default, only pass through those
  argument not already named. For example, this will over-ride the
  \code{x} argument and pass on the remaining arguments:

  \code{layer(panel.xyplot(x = jitter(x), ...))}

  The first un-named argument is assumed to be "x", so that is the same as

  \code{layer(panel.xyplot(jitter(x), ...))}

  The layer mechanism should probably still be considered experimental.

  \code{drawLayer()} actually draws the given layer object, applying the
  panel specification, style settings and so on. It should only be
  called while a panel is in focus.

  The \code{flattenPanel} function will construct a human-readable
  function incorporating code from all layers (and the original panel
  function). Note that this does not return a usable function, as it
  lacks the correct argument list and ignores any extra data sources
  that layers might use. It is intended be edited manually.
}
\value{
  a \code{layer} object is defined as a list of expression objects,
  each of which may have a set of attributes. The result of "adding"
  a layer to a trellis object (\code{+.trellis}) is the updated trellis
  object.
}
\author{ Felix Andrews \email{felix@nfrac.org} }
\seealso{
  \code{\link[lattice]{update.trellis}},
  \code{\link{as.layer}} for overlaying entire plots
}
\examples{
foo <- xyplot(ozone ~ wind, environmental)
foo

## overlay reference lines
foo <- foo + layer(panel.abline(h = 0)) +
             layer(panel.lmline(x, y, lty = 2))

## underlay a flat color
foo <- foo + layer(panel.fill(grey(.95)), under = TRUE)
foo

## layers can access the panel function arguments
foo <- foo + layer({ ok <- (y>100);
            panel.text(x[ok], y[ok], y[ok], pos = 1) })
foo

## over-ride arguments by name
foo <- foo +
  layer(panel.xyplot(y = ave(y, x, FUN = max), type = "a", ...))
foo

## see a sketch of the complete panel function
flattenPanel(foo)


## group layers, drawn for each group in each panel
dotplot(VADeaths, type = "o") +
  glayer(ltext(x[5], y[5], group.value, srt = 40))

## a quick way to print out the panel.groups arguments:
dotplot(VADeaths, type = "o") + glayer(str(list(...)))


## layers with superposed styles
xyplot(ozone ~ wind | equal.count(temperature, 2),
       data = environmental) +
   layer(panel.loess(x, y, span = 0.5), style = 1) +
   layer(panel.loess(x, y, span = 1.0), style = 2) +
   layer(panel.key(c("span = 0.5", "span = 1.0"), corner = c(1,.98),
                   lines = TRUE, points = FALSE), packets = 1)

## note that styles come from the settings in effect in the plot,
## which is not always what you want:
xyplot(1:10 ~ 1:10, type = "b", par.settings = simpleTheme(col = "red")) +
  layer(panel.lines(x = jitter(x, 2), ...)) +  ## drawn in red
  layer(panel.lines(x = jitter(x, 2), ...), force = TRUE)  ## reset theme


## using other variables from the original `data` object
## NOTE: need subscripts = TRUE in original call!
zoip <- xyplot(wind ~ temperature | equal.count(radiation, 2),
   data = environmental, subscripts = TRUE)
zoip + layer(panel.points(..., pch = 19,
             col = grey(1 - ozone[subscripts] / max(ozone))),
       data = environmental)


## restrict drawing to specified panels
barchart(yield ~ variety | site, data = barley,
         groups = year, layout = c(1,6), as.table = TRUE,
         scales = list(x = list(rot = 45))) +
layer(ltext(tapply(y, x, max), lab = abbreviate(levels(x)),
      pos = 3), rows = 1)


## example of a new data source
qua <- xyplot(lat ~ long | cut(depth, 2), quakes,
    aspect = "iso", pch = ".", cex = 2)
qua
## add layer showing distance from Auckland
newdat <- with(quakes, expand.grid(
            gridlat = seq(min(lat), max(lat), length = 60),
            gridlon = seq(min(long), max(long), length = 60)))
newdat$dist <- with(newdat, sqrt((gridlat - -36.87)^2 +
                                 (gridlon - 174.75)^2))
qua + layer_(panel.contourplot(x = gridlon, y = gridlat, z = dist,
   contour = TRUE, subscripts = TRUE), data = newdat)
}
\keyword{ aplot }

\name{panel.ellipse}
\alias{panel.ellipse}
\title{
  Lattice panel function to fit and draw a confidence ellipsoid from
  bivariate data.  
}
\description{
  A lattice panel function that computes and draws a confidence
  ellipsoid from bivariate data, possibly grouped by a third variable.
}
\usage{
panel.ellipse(x, y, groups = NULL,
              level = 0.68, segments = 50, robust = FALSE,
              center.pch = 3, center.cex = 2, \dots,
              type, pch, cex)
}
\arguments{
  \item{x, y}{
    Numeric vectors of same length giving the bivariate data.
    Non-numeric variables will be coerced to be numeric.
  }
  \item{groups}{
    Optional grouping variable.
  }
  \item{level}{
    Confidence level for the ellipse.
  }
  \item{segments}{
    Number of segments used to approximate the ellipse. 
  }
  \item{robust}{
    Logical indicating whether a robust method should be used.  If
    \code{TRUE}, the confidence ellipse is based on a bivariate
    t-distribution using the \code{\link[MASS]{cov.trob}} function in
    the \pkg{MASS} package.
  }
  \item{center.pch}{
    Plotting character for the center (fitted mean).  If \code{NULL},
    the center will not be shown on the plot.
  }
  \item{center.cex}{
    Character expansion (size) multipler for the symbol indicating the
    center.
  }
  \item{\dots}{
    Further arguments, typically graphical parameters.  Passed on to
    \code{\link[lattice]{panel.xyplot}}.  
  }
  \item{type, pch, cex}{
    Parameters that are ignored; these are present only to make sure
    they are not inadvertently passed on to \code{panel.xyplot}.
  }
}
\author{
  Deepayan Sarkar, extending code contributed by Michael Friendly.
}

% \seealso{
%   \code{\link{}}
% }

\examples{

xyplot(Sepal.Length ~ Petal.Length, groups=Species,
       data = iris, scales = "free",
       par.settings = list(superpose.symbol = list(pch=c(15:17)),
                           superpose.line = list(lwd=2, lty=1:3)),
       panel = function(x, y, ...) {
           panel.xyplot(x, y, ...)
           panel.ellipse(x, y, ...)
       },
       auto.key = list(x = .1, y = .8, corner = c(0, 0)))

## Without groups
xyplot(Sepal.Length ~ Petal.Length, 
       data = iris, scales = "free",
       par.settings = list(plot.symbol = list(cex = 1.1, pch=16)),
       panel = function(x, y, ...) {
           panel.xyplot(x, y, ...)
           panel.ellipse(x, y, lwd = 2, ...)
       },
       auto.key = list(x = .1, y = .8, corner = c(0, 0)))


## With conditioning
xyplot(Sepal.Length ~ Petal.Length | Species, 
       data = iris, scales = "free",
       par.settings = list(plot.symbol = list(cex = 1.1, pch=16)),
       layout=c(2,2),
       panel = function(x, y, ...) {
           panel.xyplot(x, y, ...)
           panel.ellipse(x, y, lwd = 2, ...)
       },
       auto.key = list(x = .6, y = .8, corner = c(0, 0)))


## Compare classical with robust 
xyplot(Sepal.Length ~ Petal.Length | Species, 
       data = iris, scales = "free",
       par.settings = list(plot.symbol = list(cex = 1.1, pch=16)),
       layout=c(2,2),
       panel = function(x, y, ...) {
           panel.xyplot(x, y, ...)
           panel.ellipse(x, y, lwd = 2, ...)
           panel.ellipse(x, y, lwd = 2, col="red", robust=TRUE,  ...)
       })

}

% __ONLY ONE__ keyword per line
\keyword{ dplot }

\name{gvhd10}
\alias{gvhd10}
\docType{data}
\title{ Flow cytometry data from five samples from a patient  }
\description{
  Flow cytometry data from blood samples taken from a Leukemia patient
  before and after allogenic bone marrow transplant.  The data spans
  five visits.
}
\usage{data(gvhd10)}
\format{
  A data frame with 113896 observations on the following 8 variables.
  \describe{
    \item{\code{FSC.H}}{ forward scatter height values}
    \item{\code{SSC.H}}{ side scatter height values }
    \item{\code{FL1.H}}{ intensity (height) in the FL1 channel }
    \item{\code{FL2.H}}{ intensity (height) in the FL2 channel }
    \item{\code{FL3.H}}{ intensity (height) in the FL3 channel }
    \item{\code{FL2.A}}{ intensity (area) in the FL2 channel }
    \item{\code{FL4.H}}{ intensity (height) in the FL4 channel }
    \item{\code{Days}}{a factor with levels \code{-6} \code{0} \code{6} \code{13} \code{20} \code{27} \code{34}}
  }
}
\source{
  \url{https://web.archive.org/web/20070427103026/http://www.ficcs.org/software.html#Data_Files}
}
\references{
  Brinkman, R.R., et al. (2007).  High-Content Flow Cytometry and
  Temporal Data Analysis for Defining a
  Cellular Signature of Graft-Versus-Host Disease.
  \emph{Biology of Blood and Marrow Transplantation} \bold{13--6}
}
\examples{
## Figure 3.4 from Sarkar (2008)
data(gvhd10)
histogram(~log2(FSC.H) | Days, gvhd10, xlab = "log Forward Scatter",
          type = "density", nint = 50, layout = c(2, 4))
}
\keyword{datasets}

\name{panel.smoother}
\alias{panel.smoother}
\title{
  Plot a smoothing line with standard error bounds.
}
\description{
  Plot a smoothing line with standard error bounds.
  This is based on the \code{\link[ggplot2]{stat_smooth}}
  function from \pkg{ggplot2}.
}
\usage{
panel.smoother(x, y, form = y ~ x, method = "loess", ...,
    se = TRUE, level = 0.95, n = 100,
    col = plot.line$col, col.se = col,
    lty = plot.line$lty, lwd = plot.line$lwd,
    alpha = plot.line$alpha, alpha.se = 0.25, border = NA,
    ## ignored: ##
    subscripts, group.number, group.value,
    type, col.line, col.symbol, fill,
    pch, cex, font, fontface, fontfamily)
}
\arguments{
  \item{x, y}{
    data points. If these are missing, they will be looked for in the
    environment of \code{form}. So in many cases you can skip these if
    passing \code{form}. In fact, for convenience, the formula
    can be passed as the first argument (i.e. \code{x}).
  }
  \item{form, method}{
    the smoothing model is constructed (approximately) as
    \code{method(form, data = list(x=x, y=y), ...)}.
    See the Examples section for common choices.
  }
  \item{\dots}{
    further arguments passed on to the model function (\code{method}).
  }
  \item{se, level}{
    estimate standard errors on the smoother, at the given \code{level},
    and plot these as a band.
  }
  \item{n}{
    number of equi-spaced points on which to evaluate the smooth function.
  }
  \item{col, col.se, lty, lwd, alpha, alpha.se, border}{
    graphical parameters. \code{col} and \code{alpha} apply to the
    smoothing line, while \code{col.se} and \code{alpha.se}
    apply to the shaded \code{se} region.
  }
  \item{subscripts, group.number, group.value,
    type, col.line, col.symbol, fill,
    pch, cex, font, fontface, fontfamily}{
    ignored.
  }
}
\details{
  This should work with any model function that takes a formula
  argument, and has a \code{predict} method with a \code{se} argument.
}
\author{
  Felix Andrews \email{felix@nfrac.org}

  Based on \code{\link[ggplot2]{stat_smooth}} by Hadley Wickham.
}
\seealso{
  \code{\link[lattice]{panel.loess}},
  \code{\link{panel.quantile}},
  \code{\link[ggplot2]{stat_smooth}}
}
\examples{
set.seed(1)
xy <- data.frame(x = runif(100),
                 y = rt(100, df = 5),
                 y2 = rt(100, df = 5) + 1)

xyplot(y ~ x, xy, panel = function(...) {
       panel.xyplot(...)
       panel.smoother(..., span = 0.9)
})

## per-group layers with glayer (pass `...` to get styles)
xyplot(y + y2 ~ x, xy) +
  glayer(panel.smoother(...))

## natural spline with 5 degrees of freedom
if (require("splines"))
  xyplot(y ~ x, xy) +
    layer(panel.smoother(y ~ ns(x,5), method = "lm"))

## thin plate regression spline with smoothness
## chosen by cross validation (see ?mgcv::gam)
if (require("mgcv"))
  xyplot(y ~ x, xy) +
    layer(panel.smoother(y ~ s(x), method = "gam"))

## simple linear regression with standard errors:
xyplot(y ~ x, xy) +
  layer(panel.smoother(x, y, method = "lm"), style = 2)
}
\keyword{ dplot }

\name{panel.lmlineq}
\alias{panel.lmlineq}
\alias{panel.ablineq}
\title{
  Draw a line with a label, by default its equation
}
\description{
  This is an extension of the panel functions \code{\link[lattice]{panel.abline}} and
  \code{\link[lattice]{panel.lmline}} to also draw a label on the line. The
  default label is the line equation, and optionally the R squared value
  of its fit to the data points.
}
\usage{
panel.ablineq(a = NULL, b = 0,
              h = NULL, v = NULL,
              reg = NULL, coef = NULL,
              pos = if (rotate) 1 else NULL,
              offset = 0.5, adj = NULL,
              at = 0.5, x, y,
              rotate = FALSE, srt = 0,
              label = NULL,
              varNames = alist(y = y, x = x),
              varStyle = "italic",
              fontfamily = "serif",
              digits = 3,
              r.squared = FALSE, sep = ", ", sep.end = "",
              col, col.text, col.line,
              ..., reference = FALSE)

panel.lmlineq(x, y, ...)
}
\arguments{
  \item{a, b, h, v, reg, coef}{
    specification of the line.
    The simplest usage is to give \code{a} and \code{b} to describe the
    line \emph{y = a + b x}.
    Horizontal or vertical lines can be specified as
    arguments \code{h} or \code{v}, respectively.
    The first argument (\code{a}) can also be a model object produced by
    \code{\link{lm}}.
    See \code{\link[lattice]{panel.abline}} for more details.
  }
  \item{pos, offset}{
    passed on to \code{\link[lattice]{panel.text}}.
    For \code{pos}: 1 = below, 2 = left, 3 = above, 4 = right,
    and the \code{offset} (in character widths) is applied.
  }
  \item{adj}{
    passed on to \code{\link[lattice]{panel.text}}.
    c(0,0) = above right, c(1,0) = above left,
    c(0,1) = below right, c(1,1) = below left;
    offset does not apply when using \code{adj}.
  }
  \item{fontfamily}{
    passed on to \code{\link[lattice]{panel.text}}.
  }
  \item{at}{
    position of the equation as a fractional distance along the line.
    This should be in the range 0 to 1.
    When a vertical line is drawn, this gives the vertical position of
    the equation.
  }
  \item{x, y}{
    position of the equation in native units. If given, this over-rides
    \code{at}.
    For \code{panel.lmlineq} this is the data, passed on as \code{lm(y ~ x)}.
  }
  \item{rotate, srt}{
    set \code{rotate = TRUE} to align the equation with the line.
    This will over-ride \code{srt}, which otherwise gives the rotation
    angle.
    Note that the calculated angle depends on the current device size;
    this will be wrong if you change the device aspect ratio after plotting.
  }
  \item{label}{
    the text to draw along with the line. If specified, this will be
    used instead of an equation.
  }
  \item{varNames}{
    names to display for \code{x} and/or \code{y}.
    This should be a list
    like \code{list(y = "Q", x = "X")}
    or, for mathematical symbols,
    \code{alist(y = (alpha + beta), x = sqrt(x[t]))}.
  }
  \item{varStyle}{
    the name of a \code{\link{plotmath}} function to wrap around the
    equation expression, or \code{NULL}. E.g. \code{"bolditalic"},
    \code{"displaystyle"}.
  }
  \item{digits}{
    number of decimal places to show for coefficients in equation.
  }
  \item{r.squared}{
    the \eqn{R^2} statistic to display along with the equation of a line.
    This can be given directly as a number, or \code{TRUE}, in which
    case the function expects a model object (typically
    \code{\link{lm}}) and extracts the \eqn{R^2} statistic from it.
  }
  \item{sep, sep.end}{
    The \eqn{R^2} (\code{r.squared}) value is separated from the
    equation by the string \code{sep}, and also \code{sep.end} is added
    to the end. For example: 
    \code{panel.ablineq(lm(y ~ x), r.squared = TRUE,
      sep = " (", sep.end = ")")}.
  }
  \item{\dots, col, col.text, col.line}{
    passed on to \code{\link[lattice]{panel.abline}} and
    \code{\link[lattice]{panel.text}}.
    Note that \code{col} applies to both text and line; \code{col.text}
    applies to the equation only, and \code{col.line} applies to line only.
  }
  \item{reference}{
    whether to draw the line in a "reference line" style, like that used
    for grid lines.
  }
}
\details{
  The equation is constructed as an expression using \code{\link{plotmath}}.
}
\author{
  Felix Andrews \email{felix@nfrac.org}
}
\seealso{
  \code{\link[lattice]{panel.abline}},
  \code{\link[lattice]{panel.text}},
  \code{\link{lm}},
  \code{\link{plotmath}}
}
\examples{
set.seed(0)
xsim <- rnorm(50, mean = 3)
ysim <- (0 + 2 * xsim) * (1 + rnorm(50, sd = 0.3))

## basic use as a panel function
xyplot(ysim ~ xsim, panel = function(x, y, ...) {
  panel.xyplot(x, y, ...)
  panel.ablineq(a = 0, b = 2, adj = c(0,1))
  panel.lmlineq(x, y, adj = c(1,0), lty = 2,
                col.line = "grey", digits = 1)
})

## using layers:
xyplot(ysim^2 ~ xsim) +
  layer(panel.ablineq(lm(y ~ x, subset = x <= 3),
    varNames = alist(y = y^2, x = x[x <= 3]), pos = 4))

## rotated equation (depends on device aspect at plotting time)
xyplot(ysim ~ xsim) +
  layer(panel.ablineq(lm(y ~ x), rotate = TRUE, at = 0.8))

## horizontal and vertical lines
xyplot(ysim ~ xsim) +
  layer(panel.ablineq(v = 3, pos = 4, at = 0.1, lty = 2,
                      label = "3.0 (critical value)")) +
  layer(panel.ablineq(h = mean(ysim), pos = 3, at = 0.15, lty = 2,
                      varNames = alist(y = plain(mean)(y))))

## using layer styles, r.squared
xyplot(ysim ~ xsim) +
  layer(panel.ablineq(lm(y ~ x), r.sq = TRUE,
                      at = 0.4, adj=0:1), style = 1) +
  layer(panel.ablineq(lm(y ~ x + 0), r.sq = TRUE,
                      at = 0.6, adj=0:1), style = 2)

## alternative placement of equations
xyplot(ysim ~ xsim) +
  layer(panel.ablineq(lm(y ~ x), r.sq = TRUE, rot = TRUE,
                      at = 0.8, pos = 3), style = 1) +
  layer(panel.ablineq(lm(y ~ x + 0), r.sq = TRUE, rot = TRUE,
                      at = 0.8, pos = 1), style = 2)

update(trellis.last.object(),
  auto.key = list(text = c("intercept", "no intercept"),
                  points = FALSE, lines = TRUE))
}
\keyword{ aplot }

\name{combineLimits}
\alias{combineLimits}
\title{Combine axis limits across margins}
\description{

  Modifies a \code{"trellis"} object with \code{"free"} scales so that
  panel limits are extended to be the same across selected conditioning
  variables (typically rows and columns).

}
\usage{
combineLimits(x, margin.x = 2L, margin.y = 1L,
              extend = TRUE, adjust.labels = TRUE)
}

\arguments{
  \item{x}{ An object of class \code{"trellis"}. }
  \item{margin.x}{
    Integer vector specifying which conditioning variables to combine
    the x-axis limits over.  Defaults to the second conditioning
    variable (rows in the default layout); that is, the limit of each
    packet is extended to include the limits of all other packets
    obtained by varying the level of the second conditioning variable
    (row).  More than one variable can be specified; for example,
    \code{margin.x = c(1, 2)} would ensure that limits are extended to
    include all levels in both the first and second conditioning
    variables.  In case there is a third conditioning variable, this
    would have the effect of per-page x-axis limits with the default
    layout.
  }
  \item{margin.y}{
    Integer vector specifying which conditioning variables to combine
    the x-axis limits over.  Similar to \code{margin.x}, but defaults to
    the first conditioning variable (columns in the default layout).
  }
  \item{extend}{
    Logical flag indicating whether the limits should be extended after
    being combined.  Usually a good idea.
  }
  \item{adjust.labels}{
    Logical flag indicating whether labels should be removed from all
    but the boundaries.  This may give misleading plots with non-default
    layouts.
  }
}

\details{

  \code{combineLimits} is useful mainly for plots with two conditioning
  variables with the default layout (columns and rows correspond to the
  first and second conditioning variables), when per-row and per-column
  limits are desired.

  The lattice approach does not tie levels of the conditioning variables
  to the plot layout, so it is possible that all panels in a row (or
  column) do not represent the same level.  It should be noted that
  \code{combineLimits} actually combines limits across levels, and not
  across rows and columns.  Results are likely to be misleading unless
  the default layout is used.

}

\note{

  Does not work for all \code{"trellis"} objects.  In particular,
  log-scales do not yet work.  Fancy layouts with \code{skip}-ped panels
  and unusual packet-to-panel mappings will probably also not work.

}

\value{

  An object of class \code{"trellis"}; essentially the same as \code{x},
  but with certain properties modified.

}
\author{ Deepayan Sarkar }

\seealso{ \code{\link[lattice]{Lattice}}, \code{\link[lattice]{xyplot}} }

\examples{

data(Cars93, package = "MASS")

## FIXME: log scales don't yet work

pcars <- 
    xyplot(Price ~ EngineSize | reorder(AirBags, Price) + Cylinders,
           data = Cars93, 
           subset = Cylinders != "rotary" & Cylinders != "5", 
           scales = list(relation = "free",
                         y = list(log = FALSE, tick.number = 3, rot = 0)),
           xlab = "Engine Size (litres)", 
           ylab = "Average Price (1000 USD)",
           as.table = TRUE) 

combineLimits(pcars)

useOuterStrips(combineLimits(update(pcars, grid = TRUE),
                             margin.x = c(1, 2), adjust.labels = FALSE))

useOuterStrips(combineLimits(update(pcars, grid = TRUE)))

}

\keyword{dplot}

\name{doubleYScale}
\alias{doubleYScale}
\title{ Draw two plot series with different y scales }
\description{
  Overplot two trellis objects with different y scales,
  optionally in different styles, adding a second y axis, and/or a
  second y axis label.

  \emph{Note:} drawing plots with multiple scales is often a bad idea
  as it can be misleading.
}
\usage{
doubleYScale(obj1, obj2, use.style = TRUE,
             style1 = if (use.style) 1, style2 = if (use.style) 2,
             add.axis = TRUE, add.ylab2 = FALSE,
             text = NULL, auto.key = if (!is.null(text))
               list(text, points = points, lines = lines, ...),
             points = FALSE, lines = TRUE, ..., under = FALSE)
}
\arguments{
  \item{obj1, obj2}{
    trellis objects. Note that most settings, like main/sub/legend/etc
    are taken only from \code{obj1}; only the panel, axis and ylab are
    taken from obj2.
  }
  \item{use.style, style1, style2}{
    \code{style1} and \code{style2} give the `group number' for obj1
    and obj2 respectively. The style is taken from these indices into
    the values of \code{trellis.par.get("superpose.line")}.
    Therefore these should be integers between 1 and 6;
    a value of 0 or NULL can be given to leave the default settings.
    These will also be applied to the y-axes and ylab, if relevant.
    \code{use.style} simply changes the defaults of the style arguments.
  }
  \item{add.axis}{
    if TRUE, draw a second y axis (for the \code{obj2} series) on the
    right side of the plot.
  }
  \item{add.ylab2}{
    if TRUE, draw a second y axis label (from \code{obj2$ylab}) on the
    right side of the plot. Note, this will replace any existing key or
    legend on the right side, i.e. with \code{space = "right"}.
  }
  \item{text, auto.key, points, lines, \dots}{
    if non-NULL, add a \code{key} to the display, using entries named by
    \code{text}. Further arguments are passed on to
    \code{\link[lattice]{simpleKey}} at plot time.
  }
  \item{under}{
    if TRUE, draw \code{obj2} under \code{obj1}.
  }
}
\details{
  Panels from the trellis object \code{obj2} will be drawn in the
  corresponding panel of \code{obj1}.

  Axis settings are taken from the trellis objects, so most
  \code{scales} arguments such as \code{draw}, \code{at}, \code{labels}
  etc from \code{obj2} will carry over to the second y axis.
}
\value{
  a merged trellis object.
}
\author{ Felix Andrews \email{felix@nfrac.org} }
\seealso{
  \code{\link{as.layer}}
}
\examples{
set.seed(1)
foo <- list(x = 1:100, y = cumsum(rnorm(100)))
## show original data
xyplot(y + y^2 ~ x, foo, type = "l")
## construct separate plots for each series
obj1 <- xyplot(y ~ x, foo, type = "l")
obj2 <- xyplot(y^2 ~ x, foo, type = "l")
## simple case: no axis for the overlaid plot
doubleYScale(obj1, obj2, add.axis = FALSE)
## draw second y axis
doubleYScale(obj1, obj2)
## ...with second ylab
doubleYScale(obj1, obj2, add.ylab2 = TRUE)
## ...or with a key
doubleYScale(obj1, obj2, text = c("obj1", "obj2"))
## ...with custom styles
update(doubleYScale(obj1, obj2, text = c("obj1", "obj2")),
  par.settings = simpleTheme(col = c('red','black'), lty = 1:2))

## different plot types
x <- rnorm(60)
doubleYScale(histogram(x), densityplot(x), use.style = FALSE)
## (but see ?as.layer for a better way to do this)

## multi-panel example
## a variant of Figure 5.13 from Sarkar (2008)
## http://lmdvr.r-forge.r-project.org/figures/figures.html?chapter=05;figure=05_13
data(SeatacWeather)
temp <- xyplot(min.temp + max.temp ~ day | month,
               data = SeatacWeather, type = "l", layout = c(3, 1))
rain <- xyplot(precip ~ day | month, data = SeatacWeather, type = "h")

doubleYScale(temp, rain, style1 = 0, style2 = 3, add.ylab2 = TRUE,
   text = c("min. T", "max. T", "rain"), columns = 3)

## re-plot with different styles
update(trellis.last.object(),
   par.settings = simpleTheme(col = c("black", "red", "blue")))
}
\keyword{ aplot }

\name{ancestry}
\alias{ancestry}
\docType{data}
\title{ Modal ancestry by County according to US 2000 Census }
\description{
  This data set records the population and the three most frequently
  reported ancestries by US county, according to the 2000 census.
}
\usage{data(ancestry)}
\format{
  A data frame with 3219 observations on the following 5 variables.
  \describe{
    \item{\code{county}}{ A factor. An attempt has been made to make the
      levels look similar to the county names used in the \code{maps}
      package.  }
    \item{\code{population}}{a numeric vector}
    \item{\code{top}}{a character vector}
    \item{\code{second}}{a character vector}
    \item{\code{third}}{a character vector}
  }
}
\source{

  U.S. Census Bureau.  The ancestry data were extracted from Summary
  File 3, available from
  
  \url{https://web.archive.org/web/20100703053258/http://www.census.gov/census2000/sumfile3.html}
  
  which is based on the \sQuote{long form} questionnaire (asked to 1 in
  6 households surveyed).
}
\seealso{
  \code{\link{mapplot}}, for examples.
}
\references{
  \url{https://web.archive.org/web/20100707180939/http://www.census.gov/prod/cen2000/doc/sf3.pdf}
}
\keyword{datasets}

\name{USCancerRates}
\alias{USCancerRates}
\docType{data}
\title{ Rate of Death Due to Cancer in US Counties }
\description{

  This data set records the annual rates of death (1999-2003) due to
  cancer by sex in US counties.

}
\usage{data(USCancerRates)}
\format{
  A data frame with 3041 observations on the following 8 variables.
  \describe{
    \item{\code{rate.male}}{a numeric vector, giving rate of death per
      100,000 due to cancer among males}
    \item{\code{LCL95.male}}{a 95\% lower confidence limit for \code{rate.male} }
    \item{\code{UCL95.male}}{a 95\% upper confidence limit for \code{rate.male} }
    \item{\code{rate.female}}{a numeric vector, giving rate of death per
      100,000 due to cancer among females}
    \item{\code{LCL95.female}}{a 95\% lower confidence limit for \code{rate.female}}
    \item{\code{UCL95.female}}{a 95\% upper confidence limit for \code{rate.female}}
    \item{\code{state}}{ a factor with levels giving name of US state }
    \item{\code{county}}{ a character vector giving county names, in a
      format similar to that used for county map boundaries in the
      \code{maps} package. }
  }
}
\details{
  See the scripts directory for details of data preprocessing steps.

  From the website: Death data provided by the National Vital Statistics
  System public use data file.  Death rates calculated by the National
  Cancer Institute using SEER*Stat. Death rates are age-adjusted to the
  2000 US standard population
  [\url{https://seer.cancer.gov/stdpopulations/stdpop.19ages.html}].
  Population counts for denominators are based on Census populations as
  modified by NCI.
  
}
\source{
  \url{https://statecancerprofiles.cancer.gov/}
}
\examples{
data(USCancerRates)
}
\keyword{datasets}

\name{panel.xblocks}
\alias{panel.xblocks}
\alias{panel.xblocks.default}
\alias{panel.xblocks.ts}
\alias{panel.xblocks.zoo}
\title{
  Plot contiguous blocks along x axis.
}
\description{
  Plot contiguous blocks along x axis. A typical use would be to
  highlight events or periods of missing data.
}
\usage{
panel.xblocks(x, ...)

\method{panel.xblocks}{default}(x, y, ..., col = NULL, border = NA, 
              height = unit(1, "npc"),
              block.y = unit(0, "npc"), vjust = 0,
              name = "xblocks", gaps = FALSE,
              last.step = median(diff(tail(x))))

\method{panel.xblocks}{ts}(x, y = x, ...)
\method{panel.xblocks}{zoo}(x, y = x, ...)
}
\arguments{
  \item{x, y}{
    In the default method, \code{x} gives the ordinates along the x axis and
    must be in increasing order. \code{y} gives the color values to plot as
    contiguous blocks. If \code{y} is numeric, data coverage is plotted,
    by converting it into a logical (\code{!is.na(y)}). Finally, if
    \code{y} is a function, it is applied to \code{x} (\code{time(x)} in
    the time series methods). 
    
    If \code{y} has character (or factor) values, these are interpreted
    as colors -- and should therefore be color names or hex
    codes. Missing values in \code{y} are not plotted.  The default
    color is taken from the current theme:
    \code{trellis.par.get("plot.line")$col}. If \code{col} is given,
    this over-rides the block colors.

    The \code{ts} and \code{zoo} methods plot the \code{y} values
    against the time index \code{time(x)}. 
  }
  \item{\dots}{
    In the default method, further arguments are graphical parameters
    passed on to \code{\link{gpar}}.
  }
  \item{col}{
    if \code{col} is specified, it determines the colors of the blocks
    defined by \code{y}. If multiple colors are specified they will be
    repeated to cover the total number of blocks.
  }
  \item{border}{
    border color.
  }
  \item{height}{
    height of blocks, defaulting to the full panel height. Numeric
    values are interpreted as native units.
  }
  \item{block.y}{
    y axis position of the blocks. Numeric values are interpreted as
    native units.
  }
  \item{vjust}{
    vertical justification of the blocks relative to \code{block.y}. See
    \code{\link[grid]{rectGrob}}.
  }
  \item{name}{
    a name for the grob (\pkg{grid} object).
  }
  \item{gaps}{
    Deprecated.
    Use \code{panel.xblocks(time(z), is.na(z))} instead.
  }
  \item{last.step}{
    width (in native units) of the final block. Defaults to the median of
    the last 5 time steps (assuming steps are regular).
  }
}
\details{
  Blocks are drawn forward in "time" from the specified x locations,
  up until the following value. Contiguous blocks are calculated using
  \code{\link{rle}}.
}
\author{
  Felix Andrews \email{felix@nfrac.org}
}
\seealso{
  \code{\link[lattice]{xyplot.ts}},
  \code{\link[lattice]{panel.rect}},
  \code{\link{grid.rect}}
}
\examples{
## Example of highlighting peaks in a time series.
set.seed(0)
flow <- ts(filter(rlnorm(200, mean = 1), 0.8, method = "r"))

## using an explicit panel function
xyplot(flow, panel = function(x, y, ...) {
  panel.xblocks(x, y > mean(y), col = "lightgray")
  panel.xyplot(x, y, ...)
})
## using layers; this is the `ts` method because `>` keeps it as ts.
xyplot(flow) +
  layer_(panel.xblocks(flow > mean(flow), col = "lightgray"))

## Example of alternating colors, here showing calendar months
flowdates <- as.Date("2000-01-01") + as.numeric(time(flow))
xyplot(flow ~ flowdates, type = "l") +
  layer_(panel.xblocks(x, months,
         col = c("lightgray", "#e6e6e6"), border = "darkgray"))

## highlight values above and below thresholds.
## blue, gray, red colors:
bgr <- hcl(c(0, 0, 260), c = c(100, 0, 100), l = c(90, 90, 90))
dflow <- cut(flow, c(0,15,30,Inf), labels = bgr)
xyplot(flow) + layer_(panel.xblocks(time(flow), dflow))

## Example of highlighting gaps (NAs) in time series.
## set up example data
z <- ts(cbind(A = 0:5, B = c(6:7, NA, NA, 10:11), C = c(NA, 13:17)))

## show data coverage only (highlighting gaps)
xyplot(z, panel = panel.xblocks,
       scales = list(y = list(draw = FALSE)))

## draw gaps in darkgray
xyplot(z, type = c("p","s")) +
  layer_(panel.xblocks(x, is.na(y), col = "darkgray"))

## Example of overlaying blocks from a different series.
## Are US presidential approval ratings linked to sunspot activity?
## Set block height, default justification is along the bottom.
xyplot(presidents) + layer(panel.xblocks(sunspot.year > 50, height = 2))
}
\keyword{ dplot }

\name{postdoc}
\alias{postdoc}
\docType{data}
\title{ Reasons for Taking First Postdoctoral Appointment }
\description{
  Reasons for Taking First Postdoctoral Appointment, by Field of
  Doctrate, 1997
}
\usage{data(postdoc)}
\format{
  The data set is avaliable as a two-way table of counts.  
}
\source{
  Survey of Doctorate Recipients, 1997
}
\references{
  Enhancing the Postdoctoral Experience for Scientists and Engineers:
  A Guide for Postdoctoral Scholars, Advisers, Institutions, Funding
  Organizations, and Disciplinary Societies

  \url{https://nap.nationalacademies.org/catalog/9831/enhancing-the-postdoctoral-experience-for-scientists-and-engineers-a-guide}
}
\examples{
data(postdoc)
library(lattice)
barchart(prop.table(postdoc, margin = 1),
         auto.key = TRUE, xlab = "Proportion")
}
\keyword{datasets}

\name{dendrogramGrob}
\alias{dendrogramGrob}
\title{ Create a Grob Representing a Dendrogram }
\description{

  This function creates a grob (a grid graphics object) that can be
  manipulated as such.  In particular, it can be used as a legend in a
  lattice display like \code{levelplot} to form heatmaps.

}


\usage{
dendrogramGrob(x, ord = order.dendrogram(x),
               side = c("right", "top"),
               add = list(), size = 5, size.add = 1,
               type = c("rectangle", "triangle"),
               \dots)
}

\arguments{
  \item{x}{ An object of class \code{"dendrogram"}.  See
    \code{\link{dendrogram}} for details }

  \item{ord}{ A vector of integer indices giving the order in which the
    terminal leaves are to be plotted.  If this is not the same as
    \code{order.dendrogram(x)}, then the leaves may not cluster together
    and branches of the dendrogram may intersect.  }

  \item{side}{ Intended position of the dendrogram when added in a
    heatmap.  Currently allowed positions are \code{"right"} and
    \code{"top"}. }

  \item{add}{ Additional annotation.  Currently, it is only possible to
    add one or more rows of rectangles at the base of the dendrogram.
    See details below.  }
  \item{size}{ Total height of the dendrogram in \code{"lines"} (see
    \code{\link[grid]{unit}})
  }
  \item{size.add}{ Size of each additional row, also in \code{"lines"} }
  \item{type}{ Whether a child node is joined to its parent  directly
    with a straight line (\code{"triangle"}) or as a \dQuote{stair} with
    two lines (\code{"rectangle"})
  }
  \item{\dots}{ Extra arguments.  Currently ignored. }
}
\details{
  The \code{add} argument can be used for additional annotation at the
  base of the dendrogram.  It should be a list with one component for
  each row, with names specifying the type of annotation and components
  specifying the contents.  Currently, the only supported name is
  \code{"rect"} (which can be repeated), producing rectangles.  The
  components in such a case is a list of graphical parameters, possibly
  vectorized, that are passed on to \code{\link[grid]{gpar}}.
}
\value{
  An object of class \code{"grob"}
}
\author{ Deepayan Sarkar \email{deepayan.sarkar@r-project.org} }

\seealso{
  \code{\link{heatmap}}, \code{\link[lattice]{levelplot}}
}

\examples{

data(mtcars)
x  <- t(as.matrix(scale(mtcars)))
dd.row <- as.dendrogram(hclust(dist(x)))
row.ord <- order.dendrogram(dd.row)

dd.col <- as.dendrogram(hclust(dist(t(x))))
col.ord <- order.dendrogram(dd.col)

library(lattice)

levelplot(x[row.ord, col.ord],
          aspect = "fill",
          scales = list(x = list(rot = 90)),
          colorkey = list(space = "left"),
          legend =
          list(right =
               list(fun = dendrogramGrob,
                    args =
                    list(x = dd.col, ord = col.ord,
                         side = "right",
                         size = 10)),
               top =
               list(fun = dendrogramGrob,
                    args =
                    list(x = dd.row, 
                         side = "top",
                         type = "triangle"))))


## Version with unsorted rows.

## For illustration, MPG ploted again at foot of dendrogram

mpg <- x["mpg", col.ord]
mpg.col <- level.colors(mpg, at = do.breaks(range(mpg), 10),
                        col.regions = terrain.colors)

levelplot(x[, col.ord],
          aspect = "iso",
          scales = list(x = list(rot = 90)),
          colorkey = FALSE,
          legend =
          list(right =
               list(fun = dendrogramGrob,
                    args =
                    list(x = dd.col, ord = col.ord,
                         side = "right",
                         add = list(rect = list(fill = mpg.col)),
                         size = 10)),
               top =
               list(fun = dendrogramGrob,
                    args =
                    list(x = dd.row, ord = sort(row.ord),
                         side = "top", size = 10,
                         type = "triangle"))))

}
\keyword{ dplot }

\name{xyplot.stl}
\alias{xyplot.stl}
\title{ Display stl fits with Lattice }

\description{
  Display \code{\link{stl}} decomposition (seasonal, trend and irregular
  components using loess) with Lattice, like the base graphics function
  \code{\link{plot.stl}}. 
}

\usage{
\method{xyplot}{stl}(x, data = NULL,
       outer = TRUE,
       layout = c(1, 4),
       strip = FALSE,
       strip.left = TRUE,
       as.table = TRUE,
       ylab = "",
       between = list(y = 0.5),
       panel =
       function(..., type) {
           if (packet.number() == 4) type <- "h"
           panel.xyplot(..., type = type)
       },
       ...)
}

\arguments{
  \item{x}{ an \code{\link{stl}} object. }
  \item{data}{ ignored. }
  \item{outer, layout, strip, strip.left}{
    passed to \code{\link[lattice]{xyplot.ts}}.
  }
  \item{as.table, ylab, between, panel, ...}{
    passed to \code{\link[lattice]{xyplot.ts}}.
  }
}

\details{
  Unless \code{strip.left} is passed in explicitly, a custom strip will
  be drawn, where shaded bars are comparable across panels (representing
  the same data range).
}

\value{
  An object of class \code{"trellis"}. The
  \code{\link[lattice:update.trellis]{update}} method can be used to
  update components of the object and the
  \code{\link[lattice:print.trellis]{print}} method (usually called by
  default) will plot it on an appropriate plotting device.
}

\author{ Deepayan Sarkar \email{deepayan.sarkar@r-project.org} }

\seealso{
  \code{\link{stl}},
  \code{\link[lattice]{xyplot.ts}},
  \code{\link[lattice]{xyplot}}
}


\examples{
## example from ?stl
xyplot(stl(log(co2), s.window=21),
       main = "STL decomposition of CO2 data")

## alternative display
data(biocAccess)
xyplot(stl(ts(biocAccess$counts[1:(24 * 30)], frequency = 24), "periodic"),
       strip.left = "strip.default")
resizePanels()

## two different spans
xyplot(stl(nottem, s.window = 4)) +
as.layer(xyplot(stl(nottem, s.window = "periodic")), style = 2)

## components superposed
xyplot(stl(nottem, s.window = 4), superpose=TRUE,
  screens = list(data = "trend", trend = "trend", "residuals"),
  strip.left = TRUE, layout = c(1,2))
}
\keyword{hplot}
\keyword{ts}

\name{horizonplot}
\alias{horizonplot}
\alias{horizonplot.default}
\alias{panel.horizonplot}
\alias{prepanel.horizonplot}
\title{Plot many time series in parallel}

\description{
  Plot many time series in parallel by cutting the y range into segments
  and overplotting them with color representing the magnitude and
  direction of deviation.
}

\usage{
horizonplot(x, data, \dots)

\S3method{horizonplot}{default}(x, data = NULL, \dots,
            nbands = 3L,
            horizonscale = NA,
            origin = function(y) na.omit(y)[1],
            colorkey = FALSE, legend = NULL,
            panel = panel.horizonplot,
            prepanel = prepanel.horizonplot,
            col.regions = hcl.colors(2 * nbands, palette="RdYlBu"),
            strip = FALSE, strip.left = TRUE,
            par.strip.text = list(cex = 0.6),
            colorkey.digits = 3,
            layout = c(1, NA),
            groups = NULL,
            default.scales =
              list(y = list(relation = "free", axs = "i", 
                            draw = FALSE, tick.number = 2)))

panel.horizonplot(x, y, ..., border = NA,
                  nbands = 3L,
                  col.regions = hcl.colors(2 * nbands, palette="RdYlBu"),
                  origin)

prepanel.horizonplot(x, y, ..., horizonscale = NA,
                     nbands = 3L,
                     origin = function(y) na.omit(y)[1])
}

\arguments{
  \item{x, y}{ Argument on which argument dispatch is carried
    out. Typically this will be a multivariate time series.
    In the panel and prepanel functions, these are the data
    coordinates. 
  }
  \item{data}{
    Not used (at least, not used by \code{xyplot.ts}).
  }
  \item{\dots}{
    further arguments.  Arguments to \code{xyplot} as
    well as to the default panel function \code{\link{panel.horizonplot}}
    can be supplied directly to \code{horizonplot}.
    In typical usage, the method of \code{xyplot} called will be
    \code{\link[lattice]{xyplot.ts}}.
  }
  \item{nbands}{ Integer giving the number of discrete color bands used
    (for both negative and positive deviations from the origin). }
  \item{horizonscale}{
    the scale of each color segment. There are 3 positive
    segments and 3 negative segments. If this is a given as a
    number then all panels will have comparable distances, though not
    necessarily the same actual values (similar in concept to
    \code{scales$relation = "sliced"}). If \code{NA}, as it is by
    default, then the scale is chosen in each panel to cover the range
    of the data (unless overridden by \code{ylim}); see Details.
  }
  \item{origin}{
    the baseline y value for the first (positive) segment
    (i.e. the value at which red changes to blue). This can be a
    number, which is then fixed across all panels, or it can be a
    function, which is evaluated with the \code{y} values in each
    panel. The default is the first non-missing y value in each
    panel. See the Details section.
  }
  \item{colorkey, legend}{
    if \code{colorkey = TRUE} a suitable color scale bar is constructed
    using the values of \code{origin} and \code{horizonscale}. Further
    options can be passed to \code{colorkey} in list form, as with
    \code{\link[lattice]{levelplot}}. 
  }
  \item{panel}{ function to render the graphic given the data.  This is
    the function that actually implements the display.  }
  \item{prepanel}{ function determining range of the data rectangle from
    data to be used in a panel. }
  \item{col.regions}{
    color scale, with at least 6 colors. This should be a divergent
    color scale (typically with white as the central color).
  }
  \item{strip, strip.left}{
    by default strips are only drawn on the left, to save space.
  }
  \item{par.strip.text}{
    graphical parameters for the strip text; see
    \code{\link[lattice]{xyplot}}. One notable argument here is
    \code{lines}, allowing multi-line text.
  }
  \item{colorkey.digits}{
    digits for rounding values in colorkey labels.
  }
  \item{layout}{
    Numeric vector of length 2 (or 3) specifying number of columns and
    rows (and pages) in the plot. The default is to have one column and
    as many rows as there are panels.
  }
  \item{default.scales}{
    sets default values of \code{scales}; leave this alone, pass
    \code{scales} instead.
  }
  \item{groups}{
    not applicable to this type of plot.
  }
  \item{border}{
    border color for the filled polygons, defaults to no border.
  }
}

\details{
  This function draws time series as filled areas, with modifications
  to effectively visualise many time series in parallel. Data that would
  be drawn off the top of each panel is redrawn from the bottom of the
  panel in a darker color. Values below the origin are inverted and
  drawn in the opposite color. There are up to three shades (typically
  in blue) for data above the baseline and up to three shades (typically
  in red) for data below the baseline. See the article referenced below
  for an introduction to Horizon plots.

  There are three different cases of using this function:
  \enumerate{
    \item \code{horizonscale} unspecified (default case): then each
     panel will have different scales, and the colors represent
     deviations from the origin up to the maximum deviation from the
     origin in that panel. If \code{origin} is specified then that will
     be constant across panels; otherwise it defaults to the initial
     value.
   \item \code{horizonscale} specified but \code{origin} unspecified:
     the origin defaults to the initial value in each panel, and colors
     represent deviations from it in steps of \code{horizonscale} (up to
     3 steps each way).
   \item both \code{horizonscale} and \code{origin} specified: each
     panel will have the same scales, and colors represent fixed ranges
     of values. 
  }

  In each of these cases the \code{colorkey} is labelled slightly
  differently (see examples).
}

\section{Warning}{
  Note that the y scale in each panel defines the actual origin and
  scale used. The \code{origin} and \code{horizonscale} arguments are
  only used in the \code{prepanel} function to choose an appropriate y
  scale. The \code{ylim} argument therefore over-rides 
  \code{origin} and \code{horizonscale}. This also implies that choices
  of \code{scales$y$relation} other than \code{"free"} may have
  unexpected effects, particularly \code{"sliced"}, as these change the
  y limits from those requested by the prepanel function.
}


\value{
  An object of class \code{"trellis"}. The
  \code{\link[lattice:update.trellis]{update}} method can be used to
  update components of the object and the
  \code{\link[lattice:print.trellis]{print}} method (usually called by
  default) will plot it on an appropriate plotting device.
}

\references{
  Stephen Few (2008). Time on the Horizon.
  \emph{Visual Business Intelligence Newsletter}, June/July 2008
  \url{https://www.perceptualedge.com/articles/visual_business_intelligence/time_on_the_horizon.pdf}
}

\author{
  Felix Andrews \email{felix@nfrac.org}
}

\seealso{
  \code{\link[lattice]{Lattice}},
  \code{\link[lattice]{xyplot.ts}},
  \code{\link{panel.xyarea}}
}

\examples{
## generate a random time series object with 12 columns
set.seed(1)
dat <- ts(matrix(cumsum(rnorm(200 * 12)), ncol = 12))
colnames(dat) <- paste("series", LETTERS[1:12])

## show simple line plot first, for reference.
xyplot(dat, scales = list(y = "same"))

## these layers show scale and origin in each panel...
infolayers <-
  layer(panel.scaleArrow(x = 0.99, digits = 1, col = "grey",
                         srt = 90, cex = 0.7)) +
  layer(lim <- current.panel.limits(),
    panel.text(lim$x[1], lim$y[1], round(lim$y[1],1), font = 2,
        cex = 0.7, adj = c(-0.5,-0.5), col = "#9FC8DC"))

## Case 1: each panel has a different origin and scale:
## ('origin' default is the first data value in each series).
horizonplot(dat, layout = c(1,12), colorkey = TRUE) +
  infolayers
 
## Case 2: fixed scale but different origin (baseline):
## (similar in concept to scales = "sliced")
horizonplot(dat, layout = c(1,12), horizonscale = 10, colorkey = TRUE) +
  infolayers

## Case 3: fixed scale and constant origin (all same scales):
horizonplot(dat, layout = c(1,12), origin = 0, horizonscale = 10, colorkey = TRUE) +
  infolayers

## same effect using ylim (but colorkey does not know limits):
horizonplot(dat, layout = c(1,12), ylim = c(0, 10), colorkey = TRUE) +
  infolayers

## same scales with full coverage of color scale:
horizonplot(dat, layout = c(1,12), origin = 0,
            scales = list(y = list(relation = "same")),
            colorkey = TRUE, colorkey.digits = 1) +
  infolayers


## use ylab rather than strip.left, for readability.
## also shade any times with missing data values.
horizonplot(dat, horizonscale = 10, colorkey = TRUE,
            layout = c(1,12), strip.left = FALSE,
            ylab = list(rev(colnames(dat)), rot = 0, cex = 0.7)) +
  layer_(panel.fill(col = "gray90"), panel.xblocks(..., col = "white"))


## illustration of the cut points used in the following plot
xyplot(EuStockMarkets, scales = list(y = "same"),
  panel = function(x, y, ...) {
    col <-
    c("#B41414","#E03231","#F7A99C","#9FC8DC","#468CC8","#0165B3")
    for (i in c(-3:-1, 2:0)) {
      if (i >= 0)
        yi <- pmax(4000, pmin(y, 4000 + 1000 * (i+1)))
      if (i < 0)
        yi <- pmin(4000, pmax(y, 4000 + 1000 * i))
      panel.xyarea(x, yi, origin = 4000,
        col = col[i+4], border = NA)
    }
    panel.lines(x, y)
    panel.abline(h = 4000, lty = 2)
  })

## compare with previous plot
horizonplot(EuStockMarkets, colorkey = TRUE,
            origin = 4000, horizonscale = 1000) +
  infolayers

## a cut-and-stack plot; use constant y scales!
horizonplot(sunspots, cut = list(n = 23, overlap = 0),
  scales = list(draw = FALSE, y = list(relation = "same")),
  origin = 100, colorkey = TRUE,
  strip.left = FALSE, layout = c(1,23)) +
layer(grid::grid.text(round(x[1]), x = 0, just = "left"))
}
\keyword{hplot}
\keyword{ts}

\name{mapplot}
\alias{mapplot}
\alias{mapplot.formula}
\alias{panel.mapplot}
\alias{prepanel.mapplot}
\title{ Trellis displays on Maps a.k.a. Choropleth maps }
\description{
  Produces Trellis displays of numeric (and eventually categorical) data
  on a map.  This is largely meant as a demonstration, and users looking
  for serious map drawing capabilities should look elsewhere (see below).
}
\usage{

mapplot(x, data, \dots)

\S3method{mapplot}{formula}(x, data, map, outer = TRUE,
        prepanel = prepanel.mapplot,
        panel = panel.mapplot,
        aspect = "iso",
        legend = NULL,
        breaks, cuts = 30,
        colramp = colorRampPalette(hcl.colors(n = 11, palette = "Spectral")),
        colorkey = TRUE,
        \dots)

prepanel.mapplot(x, y, map, \dots)
panel.mapplot(x, y, map, breaks, colramp, exact = FALSE, lwd = 0.5, \dots)

}

\arguments{
  \item{x, y}{ For \code{mapplot}, an object on which method dispatch is
    carried out.  For the formula method, a formula of the form \code{y
      ~ x}, with additional conditioning variables as desired.  The
    extended form of conditioning using \code{y ~ x1 + x2} etc. is also
    allowed.  The formula might be interpreted as in a dot plot, except
    that \code{y} is taken to be the names of geographical units in
    \code{map}.

    Suitable subsets (packets) of \code{x} and \code{y} are passed to
    the prepanel and panel functions.
  }
  \item{data}{ A data source where names in the formula are evaluated }
  \item{map}{ An object of class \code{"map"} (package \code{maps}),
    containing boundary information.  The names of the geographical
    units must match the \code{y} variable in the formula.  }

  The remaining arguments are standard lattice arguments, relevant here
  mostly because they have different defaults than usual:

  \item{outer}{ Logical flag specifying how variables separated by
    \code{+} in the formula are interpreted.  It is not advisable to
    change the default.  }
  \item{prepanel}{ The prepanel function. }
  \item{panel}{ The panel function. }
  \item{aspect}{ The aspect ratio. }
  \item{breaks, cuts, colramp}{ controls conversion of numeric \code{x}
    values to a false color.  \code{colramp} should be
    a function that produces colors (such as \code{\link{cm.colors}}).
    If it is \code{NULL}, colors are taken from
    \code{trellis.par.get("regions")}. }
  \item{exact}{ the default \code{exact = FALSE} allows the given
    \code{y} values to match sub-regions of \code{map},
    i.e. region names with a qualifier following \code{":"}, like
    \code{"michigan:north", "michigan:south"}. These will both match a
    \code{y} value of \code{"Michigan"}. }
  \item{legend, colorkey}{ controls legends; usually just a color key
    giving the association between numeric values of \code{x} and
    color.  }
  \item{lwd}{line width} 
  \item{\dots}{ Further arguments passed on to the underlying engine.
    See \code{\link[lattice]{xyplot}} for details.  }
}
\value{
  An object of class \code{"trellis"}.
}
\references{ \url{https://en.wikipedia.org/wiki/Choropleth_map} }
\author{ Deepayan Sarkar }
\note{

  This function is meant to demonstrate how maps can be incorporated in
  a Trellis display.  Users seriously interested in geographical data
  should consider using software written by people who know what they
  are doing.

}
\seealso{ \code{\link[lattice]{Lattice}}  }
\examples{

## Requires the 'maps' and 'mapproj' packages:

if (require(maps) && require(mapproj)) {

    ## Note: Alaska, Hawaii and others are not included in county map;
    ## this generates warnings with both USCancerRates and ancestry.

    data(USCancerRates)

    suppressWarnings(print(
        mapplot(rownames(USCancerRates) ~ log(rate.male) + log(rate.female),
                data = USCancerRates,
                map = map("county", plot = FALSE, fill = TRUE,
                          projection = "mercator"))
    ))

    suppressWarnings(print(
        mapplot(rownames(USCancerRates) ~ log(rate.male) + log(rate.female),
                data = USCancerRates,
                map = map("county", plot = FALSE, fill = TRUE,
                          projection = "tetra"),
                scales = list(draw = FALSE))
    ))

    data(ancestry)

    county.map <- 
        map('county', plot = FALSE, fill = TRUE, 
            projection = "azequalarea")

    ## set a sequential color palette as current theme, and use it
    opar <- trellis.par.get()
    trellis.par.set(custom.theme(region = rev(hcl.colors(9, "Purp")),
                                 alpha.line = 0.5))
    suppressWarnings(print(
        mapplot(county ~ log10(population), ancestry, map = county.map,
                colramp = NULL, border = "transparent")
    ))
    trellis.par.set(opar)
}

\dontrun{

## this may take a while (should get better area records)

if (require(maps) && require(mapproj)) {

    county.areas <- 
        area.map(county.map, regions = county.map$names, sqmi = FALSE)
    
    ancestry$density <- 
        with(ancestry, population / county.areas[as.character(county)])

    mapplot(county ~ log(density), ancestry,
            map = county.map, border = NA,
            colramp = colorRampPalette(c("white", "black")))

}
}
}
\keyword{ hplot }

\name{panel.3dmisc}
\alias{panel.3dmisc}
\alias{panel.3dtext}
\alias{panel.3dbars}
\alias{panel.3dpolygon}
\title{Miscellanous panel utilities for three dimensional Trellis Displays}
\description{
  Miscellanous panel functions for use with  three dimensional Lattice
  functions such as cloud and wireframe
}
\usage{

panel.3dbars(x, y, z,
             rot.mat = diag(4), distance,
             xbase = 1, ybase = 1,
             xlim, xlim.scaled,
             ylim, ylim.scaled,
             zlim, zlim.scaled,
             zero.scaled,
             col = "black",
             lty = 1, lwd = 1,
             alpha,
             ...,
             col.facet = "white",
             alpha.facet = 1)


panel.3dpolygon(x, y, z, rot.mat = diag(4), distance,
                xlim.scaled,
                ylim.scaled,
                zlim.scaled,
                zero.scaled,
                col = "white",
                border = "black",
                font, fontface,
                ...)

panel.3dtext(x, y, z, labels = seq_along(x),
             rot.mat = diag(4), distance, ...)


}
\arguments{
  \item{x, y, z}{
    data to be plotted
  }
  \item{rot.mat, distance}{
    arguments controlling projection
  }
  \item{labels}{ character or expression vectors to be uses as labels }
  \item{xlim, ylim, zlim}{
    limits in the original scale
  }
  \item{xlim.scaled, ylim.scaled, zlim.scaled}{
    limits after scaling
  }
  \item{zero.scaled}{
    the value of z = 0 after scaling
  }
  \item{xbase, ybase}{
    length of the sides of the bars (which are always centered on the
    \code{x} and \code{y} values).  Can not be vectorized.
  }
  \item{col, lty, lwd, alpha, border}{
    Graphical parameters for the border lines.  These can be vectors, in
    which case each component will be associated with one bar in
    \code{panel.3dbars}. 
  }
  \item{font, fontface}{ unused graphical parameters, present in the
    argument list only so that they can be captured and ignored  }
  \item{col.facet, alpha.facet}{
    Graphical parameters for surfaces of the bars . These can be
    vectors, in which case each component will be associated with one
    bar.
  }
  \item{\dots}{
    extra arguments, passed on as appropriate.
  }
}

\details{

  \code{panel.3dbars} and \code{panel.3dpolygon} are both suitable for
  use as (components of) the \code{panel.3d.cloud} argument of
  \code{panel.cloud}.  The first one produces three dimensional bars,
  and the second one draws three dimensional polygons.
}

\author{ Deepayan Sarkar \email{deepayan.sarkar@gmail.com}}
\seealso{
  \code{\link[lattice]{cloud}},
  \code{\link[lattice]{panel.cloud}}
}

\examples{

library(lattice)

cloud(VADeaths, panel.3d.cloud = panel.3dbars,
      col.facet = "grey", xbase = 0.4, ybase = 0.4,
      screen = list(z = 40, x = -30))

cloud(VADeaths, panel.3d.cloud = panel.3dbars,
      xbase = 0.4, ybase = 0.4, zlim = c(0, max(VADeaths)),
      scales = list(arrows = FALSE, just = "right"), xlab = NULL, ylab = NULL,
      col.facet = level.colors(VADeaths, at = do.breaks(range(VADeaths), 20),
                               col.regions = terrain.colors,
                               colors = TRUE),
      colorkey = list(col = terrain.colors, at = do.breaks(range(VADeaths), 20)),
      screen = list(z = 40, x = -30))


cloud(as.table(prop.table(Titanic, margin = 1:3)[,,,2]),
      type = c("p", "h"),
      zlab = "Proportion\nSurvived",
      panel.3d.cloud = panel.3dbars,
      xbase = 0.4, ybase = 0.4,
      aspect = c(1, 0.3),
      scales = list(distance = 2),
      panel.aspect = 0.5)

}

\keyword{dplot}

\name{tileplot}
\alias{tileplot}
\title{ Plot a spatial mosaic from irregular 2D points }
\description{
  Represents an irregular set of (x, y) points with a color covariate.
  Polygons are drawn enclosing the area closest to each point.
  This is known variously as a Voronoi mosaic, a Dirichlet tesselation,
  or Thiessen polygons.
}
\usage{
tileplot(x, data = NULL, aspect = "iso",
         prepanel = "prepanel.default.xyplot",
         panel = "panel.voronoi", ...)
}
\arguments{
  \item{x, data}{ formula and data as in
    \code{\link[lattice]{levelplot}}, except that it expects irregularly
    spaced points rather than a regular grid. }
  \item{aspect}{ aspect ratio: "iso" is recommended as it reproduces the
    distances used in the triangulation calculations. }
  \item{panel, prepanel}{ see \code{\link[lattice]{xyplot}}. }
  \item{\dots}{ further arguments to the panel function, which defaults to
    \code{\link{panel.voronoi}}. }
}
\details{
  See \code{\link{panel.voronoi}} for further options and details.
}
\author{ Felix Andrews \email{felix@nfrac.org} }
\seealso{ \code{\link{panel.voronoi}}, \code{\link[lattice]{levelplot}} }
\examples{
xyz <- data.frame(x = rnorm(100), y = rnorm(100), z = rnorm(100))
tileplot(z ~ x * y, xyz)

## Alternative backend using 'deldir' package
\dontrun{
tileplot(z ~ x * y, xyz, backend = "deldir")
}

## showing rectangular window boundary
tileplot(z ~ x * y, xyz, xlim = c(-2, 4), ylim = c(-2, 4))

## insert some missing values
xyz$z[1:10] <- NA
## the default na.rm = FALSE shows missing polygons
tileplot(z ~ x * y, xyz, border = "black",
  col.regions = grey.colors(100),
  pch = ifelse(is.na(xyz$z), 4, 21),
  panel = function(...) {
    panel.fill("hotpink")
    panel.voronoi(...)
  })
## use na.rm = TRUE to ignore points with missing values
update(trellis.last.object(), na.rm = TRUE)

## a quick and dirty approximation to US state boundaries
tmp <- state.center
tmp$Income <- state.x77[,"Income"]
tileplot(Income ~ x * y, tmp, border = "black",
  panel = function(x, y, ...) {
    panel.voronoi(x, y, ..., points = FALSE)
    panel.text(x, y, state.abb, cex = 0.6)
  })
}
\keyword{ hplot }

\name{segplot}
\alias{segplot}
\alias{segplot.formula}
\title{ Plot segments using the Trellis framework }

\description{ This function can be used to systematically draw segments
  using a formula interface to produce Trellis displays using the
  lattice package.  Segments can be drawn either as lines or bars, and
  can be color coded by the value of a covariate, with a suitable
  legend.}

\usage{
segplot(x, data, \dots)

\S3method{segplot}{formula}(x, data,
        level = NULL, centers = NULL,
        prepanel = prepanel.segplot,
        panel = panel.segplot,
        xlab = NULL, ylab = NULL,
        horizontal = TRUE,
        \dots,
        at, cuts = 30, colorkey = !is.null(level))

}

\arguments{
  \item{x}{ Argument on which argument dispatch is carried out.  For the
    \code{"formula"} method, a formula of the form \code{y ~ x1 + x2}
    (with further conditioning variables appended if necessary).  The
    terms in the formula must all be vectors of the same length.  Each
    element causes a line segment or rectangle to be drawn, with the
    vertical location determined by \code{y} and horizontal endpoints
    determined by \code{x1} and \code{x2}.  }
  \item{data}{ An optional data frame, list or environment where
    variables in the formula, as well as \code{level}, will be
    evaluated.
  }
  \item{level}{ An optional covariate that determines color coding of
    the segments}
  \item{centers}{ optional vector of \sQuote{centers} of the segments.
    If specified, points will be plotted at these \code{y}-locations.  }
  \item{prepanel}{ function determining range of the data rectangle from
    data to be used in a panel. }
  \item{panel}{ function to render the graphic given the data.  This is
    the function that actually implements the display.  }
  \item{xlab, ylab}{ Labels for the axes.  By default both are
    missing. }
  \item{horizontal}{ logical, whether the segments are to be drawn
    horizontally (the default) or vertically.  This essentially swaps
    the role of the x- and y-axes in each panel. }
  \item{\dots}{ further arguments.  Arguments to \code{levelplot} as
    well as to the default panel function \code{\link{panel.segplot}}
    can be supplied directly to \code{segplot}. }
  \item{colorkey}{ logical indicating whether a legend showing
    association of segment colors to values of \code{level} should be
    shown, or a list to control details of such a color key.  See
    details below. }
  \item{at, cuts}{ \code{at} specifies the values of \code{level} where
    the color code changes.  If \code{at} is missing, it defaults to
    \code{cuts} equispaced locations spanning the range of \code{levels}
  }
}

\details{
  The \code{levelplot} function from the lattice package is used to
  internally to implement this function.  In particular, the colorkey
  mechanism is used as it is, and documentation for
  \code{\link[lattice]{levelplot}} should be consulted to learn how to
  fine tune it.
}


\value{
  An object of class \code{"trellis"}. The
  \code{\link[lattice:update.trellis]{update}} method can be used to
  update components of the object and the
  \code{\link[lattice:print.trellis]{print}} method (usually called by
  default) will plot it on an appropriate plotting device.
}

% \references{ ~put references to the literature/web site here ~ }

\author{ Deepayan Sarkar \email{deepayan.sarkar@r-project.org} }

\note{ Currently only horizontal segments are supported.  Vertical
  segments can be obtained by modifying the prepanel and panel functions
  suitably. }

\seealso{
  \code{\link[lattice]{Lattice}},
  \code{\link{panel.segplot}},
  \code{\link[lattice]{levelplot}},
  \code{\link[lattice]{xyplot}}
}


\examples{

segplot(factor(1:10) ~ rnorm(10) + rnorm(10), level = runif(10))

data(USCancerRates)

segplot(reorder(factor(county), rate.male) ~ LCL95.male + UCL95.male,
        data = subset(USCancerRates, state == "Washington"))

segplot(reorder(factor(county), rate.male) ~ LCL95.male + UCL95.male,
        data = subset(USCancerRates, state == "Washington"),
        draw.bands = FALSE, centers = rate.male)

segplot(reorder(factor(county), rate.male) ~ LCL95.male + UCL95.male,
        data = subset(USCancerRates, state == "Washington"),
        level = rate.female, col.regions = terrain.colors)

segplot(reorder(factor(county), rate.male) ~ LCL95.male + UCL95.male,
        data = subset(USCancerRates, state == "Washington"),
        draw.bands = FALSE, centers = rate.male, 
        segments.fun = panel.arrows, ends = "both", 
        angle = 90, length = 1, unit = "mm")

}
\keyword{hplot}

\name{panel.qqmath.tails}
\alias{panel.qqmath.tails}
\title{
  Approximate distribution in qqmath but keep points on tails.
}
\description{
  Panel function for \code{\link[lattice]{qqmath}} to reduce the number of points
  plotted by sampling along the specified distribution. The usual method
  for such sampling is to use the \code{f.value} argument to
  \code{\link[lattice]{panel.qqmath}}. However, this panel function differs in
  two ways:
    (1) a specified number of data points are retained (not
    interpolated) on each tail of the distribution.
    (2) the sampling is evenly spaced along the specified
    distribution automatically (whereas \code{f.value = ppoints(100)} is
    evenly spaced along the uniform distribution only).


  \emph{
    This function is deprecated as of \pkg{lattice} 0.18-4 (available for
    R 2.11.0). Use the \code{tails.n} argument of
    \code{\link[lattice]{panel.qqmath}} instead.
  }
  
}
\usage{
panel.qqmath.tails(x, f.value = NULL, distribution = qnorm,
                    groups = NULL, ..., approx.n = 100, tails.n = 10)
}
\arguments{
  \item{x, f.value, distribution, groups}{
    see \code{\link[lattice]{panel.qqmath}}.
  }
  \item{\dots}{
    further arguments passed on to \code{\link[lattice]{panel.xyplot}}.
  }
  \item{approx.n}{
    number of points to use in approximating the distribution.
    Points will be equally spaced in the distribution space.
  }
  \item{tails.n}{
    number of points to retain (untouched) at both the high and low tails.
  }
}
\author{
  Felix Andrews \email{felix@nfrac.org}
}
\seealso{
  \code{\link[lattice]{panel.qqmath}} which should be used instead (as of \pkg{lattice} 0.18-4).
}
\examples{

## see ?panel.qqmath

}
\keyword{ dplot }

\name{USAge}
\alias{USAge}
\alias{USAge.table}
\alias{USAge.df}
\docType{data}
\title{ US national population estimates }
\description{
  US national population estimates by age and sex from 1900 to 1979.
  The data is available both as a (3-dimensional) table and a data
  frame.  The second form omits the 75+ age group to keep age numeric.
}

\usage{
data(USAge.table)
data(USAge.df)
}

\format{
  \code{USAge.table} is a 3-dimensional array with dimensions 
  \tabular{rll}{
    No \tab Name \tab Levels\cr
    1 \tab Age \tab 0, 1, 2, \dots, 74, 75+\cr
    2 \tab Sex \tab Male, Female\cr
    3 \tab Year \tab 1900, 1901, \dots, 1979 \cr
  }
  Cells contain raw counts of estimated population.

  \code{USAge.df} is a data frame with 12000 observations on the
  following 4 variables.
  \describe{
    \item{\code{Age}}{a numeric vector, giving age in years}
    \item{\code{Sex}}{a factor with levels \code{Male} \code{Female}}
    \item{\code{Year}}{a numeric vector, giving year}
    \item{\code{Population}}{a numeric vector, giving population in
      millions}
  }
}

\details{

  The data for 1900-1929 are rounded to thousands.  The data for
  1900-1939 exclude the Armed Forces overseas and the population
  residing in Alaska and Hawaii.  The data for 1940-1949 represent the
  resident population plus Armed Forces overseas, but exclude the
  population residing in Alaska and Hawaii.  The data for 1950-1979
  represent the resident population plus Armed Forces overseas, and
  also include the population residing in Alaska and Hawaii.

}

\source{

  U.S. Census Bureau website: \url{https://www.census.gov/}

  %% Original URL
  %% \url{http://www.census.gov/popest/data/national/asrh/pre-1980/PE-11.html} is no longer valid
  
  The data were available as individual files for each year, with
  varying levels for the margins.  The preprocessing steps used to
  reduce the data to the form given here are described in the scripts
  directory.
  
}

\examples{
data(USAge.df)
head(USAge.df)

## Figure 10.7 from Sarkar (2008)
xyplot(Population ~ Age | factor(Year), USAge.df,
       groups = Sex, type = c("l", "g"),
       auto.key = list(points = FALSE, lines = TRUE, columns = 2),
       aspect = "xy", ylab = "Population (millions)",
       subset = Year \%in\% seq(1905, 1975, by = 10))
}
\keyword{datasets}

\name{panel.2dsmoother}
\alias{panel.2dsmoother}
\title{
  Plot a smooth approximation of z over x and y.
}
\description{
  Plot a smooth approximation, using \code{\link{loess}} by default, of
  one variable (\code{z}) against two others (\code{x} and \code{y}).

  This panel function should be used with a \code{\link[lattice]{levelplot}}.
}
\usage{
panel.2dsmoother(x, y, z, subscripts = TRUE,
    form = z ~ x * y, method = "loess", ...,
    args = list(), n = 100)
}
\arguments{
  \item{x, y, z}{
    data points. If these are missing, they will be looked for in the
    environment of \code{form}. So in many cases you can skip these if
    passing \code{form}. In fact, for convenience, the formula
    can be passed as the first argument (i.e. \code{x}).
  }
  \item{form, method}{
    the smoothing model is constructed (approximately) as
    \code{method(form, data = list(x=x, y=y, z=z), {args})}.
    See the Examples section for common choices.
  }
  \item{subscripts}{
    data indices for the current packet, as passed in by \code{levelplot}.
  }
  \item{\dots}{
    further arguments passed on to \code{\link[lattice]{panel.levelplot}}.
  }
  \item{args}{
    a list of further arguments to the model function (\code{method}).
  }
  \item{n}{
    number of equi-spaced points along each of x and y on which to evaluate the smooth function.
  }
}
\details{
  This should work with any model function that takes a formula
  argument, and has a \code{predict} method argument.
}
\author{
  Felix Andrews \email{felix@nfrac.org}
}
\seealso{
  \code{\link{loess}},
  \code{\link{panel.smoother}}
}
\examples{
set.seed(1)
xyz <- data.frame(x = rnorm(100), y = rnorm(100))
xyz$z <- with(xyz, x * y + rnorm(100, sd = 1))

levelplot(z ~ x * y, xyz, panel = panel.2dsmoother)

## showing data points on the same color scale
levelplot(z ~ x * y, xyz,
          panel = panel.levelplot.points, cex = 1.2) +
  layer_(panel.2dsmoother(..., n = 200))

## simple linear regression model
levelplot(z ~ x * y, xyz,
          panel = panel.levelplot.points) +
  layer_(panel.2dsmoother(..., method = "lm"))

## GAM smoother with smoothness by cross validation
if (require("mgcv"))
  levelplot(z ~ x * y, xyz, panel = panel.2dsmoother,
            form = z ~ s(x, y), method = "gam")
}
\keyword{ dplot }

\name{SeatacWeather}
\alias{SeatacWeather}
\docType{data}
\title{ Daily Rainfall and Temperature at the Seattle-Tacoma Airport }
\description{
  Daily Rainfall and Temperature at the Seattle-Tacoma Airport between 
  January through March of 2007.
}
\usage{data(SeatacWeather)}
\format{
  A data frame with 90 observations on the following 14 variables.
  \describe{
    \item{\code{month}}{a factor with levels \code{January},
      \code{February}, and \code{March}}
    \item{\code{day}}{ day of the month }
    \item{\code{year}}{ year, all 2007 }
    \item{\code{max.temp}}{ maximum temperature (Fahrenheit) }
    \item{\code{record.max}}{ record maximum temperature }
    \item{\code{normal.max}}{ normal maximum temperature }
    \item{\code{min.temp}}{ minimum temperature }
    \item{\code{record.min}}{ record minimum temperature }
    \item{\code{normal.min}}{ normal minimum temperature }
    \item{\code{precip}}{ precipitation (inches) }
    \item{\code{record.precip}}{ record precipitation }
    \item{\code{normal.precip}}{ normal precipitation }
    \item{\code{time.max}}{ time of maximum temperature }
    \item{\code{time.min}}{ time of minimum temperature }
  }
}
\details{
  The time of minimum and maximum temperatures should be interpreted as
  follows: the least two significant digits denote minutes (out of 60)
  and the next two significant digits denote hout (out of 24).
}
\source{
  \url{https://www.atmos.washington.edu/cgi-bin/list_climate.cgi?clisea}
}
\keyword{datasets}

\name{panel.scaleArrow}
\alias{panel.scaleArrow}
\title{
  Draw a scale bar as an arrow, labelled with its length in plot units.
}
\description{
  Draw a scale bar as an arrow, labelled with its length in plot units.
}
\usage{
panel.scaleArrow(x = unit(0:1, "npc"), y = unit(0:1, "npc"),
                 default.units = "npc",
                 digits = 0, append = "", label = NULL,
                 angle = 30, length = 0.5, unit = "char",
                 type = "open", ends = "both",
                 \dots,
                 col = add.line$col, fill = col, alpha = add.line$alpha,
                 lty = add.line$lty, lwd = add.line$lwd,
                 col.text = add.text$col, alpha.text = add.text$alpha)
}
\arguments{
  \item{x, y, default.units}{
    coordinates of the line ends as \pkg{grid} \code{\link{unit}}s or
    otherwise interpreted in \code{default.units}.
  }
  \item{digits}{
    number of decimal places to keep for the distance measure.
  }
  \item{append}{
    a string to append to the distance for the label.
  }
  \item{label}{
    label to place on the mid point of the scale, over-riding the default.
  }
  \item{angle, length, unit, type, ends}{
    specification of the arrow style; see \code{\link[lattice]{panel.arrows}}.
  }
  \item{\dots}{
    further arguments passed to \code{\link[lattice]{panel.text}}.
    You will need at least the \code{pos} or \code{adj} arguments.
  }
  \item{col, fill, alpha, lty, lwd}{
    graphical parameters relevant to the line.
  }
  \item{col.text, alpha.text}{
    graphical parameters relevant to the text label. Others like
    \code{cex} and \code{font} can be passed though \code{\dots}.
  }
}
\author{
  Felix Andrews \email{felix@nfrac.org}
}
\seealso{
  \code{\link[lattice]{panel.abline}},
  \code{\link{grid.text}}
}
\examples{
xyplot(EuStockMarkets) +
  layer(panel.scaleArrow(x = 0.99, append = " units",
          col = "grey", srt = 90, cex = 0.8))
}
\keyword{ aplot }

\name{scale.components}
\alias{scale.components}
\alias{xscale.components.logpower}
\alias{yscale.components.logpower}
\alias{xscale.components.fractions}
\alias{yscale.components.fractions}
\alias{xscale.components.log10ticks}
\alias{yscale.components.log10ticks}
\alias{xscale.components.log}
\alias{yscale.components.log}
\alias{xscale.components.log10.3}
\alias{yscale.components.log10.3}
\alias{xscale.components.subticks}
\alias{yscale.components.subticks}

\title{ Custom lattice axis scales }
\description{
  Convenience functions for drawing axes with various non-default tick
  positions and labels. 
}
\usage{
xscale.components.logpower(lim, ...)
yscale.components.logpower(lim, ...)

xscale.components.fractions(lim, logsc = FALSE, ...)
yscale.components.fractions(lim, logsc = FALSE, ...)

xscale.components.log10ticks(lim, logsc = FALSE, at = NULL, ...)
yscale.components.log10ticks(lim, logsc = FALSE, at = NULL, ...)

xscale.components.log(lim, logsc = FALSE, at = NULL, loc = NULL, ...)
yscale.components.log(lim, logsc = FALSE, at = NULL, loc = NULL, ...)

xscale.components.log10.3(lim, logsc = FALSE, at = NULL, ...)
yscale.components.log10.3(lim, logsc = FALSE, at = NULL, ...)

xscale.components.subticks(lim, ..., n = 5, n2 = n * 5, min.n2 = n + 5)
yscale.components.subticks(lim, ..., n = 5, n2 = n * 5, min.n2 = n + 5)
}
\arguments{
  \item{lim}{ scale limits. }
  \item{\dots}{
    passed on to \code{\link[lattice]{xscale.components.default}} or
    \code{yscale.components.default}. 
  }
  \item{logsc}{
    \code{log} base, typically specified in the \code{scales}
    argument to a high-level lattice plot.
  }
  \item{at}{
    this is ignored unless it is \code{NULL}, in which case nothing is
    drawn. 
  }
  \item{loc}{
    Ignored if \code{NULL}.  Otherwise, vector of values between 1 and
    9 that form the template for tick mark locations after being scaled
    appropriately.  For example, \code{loc = c(1, 3)} leads to tick
    marks at \code{c(0.1, 0.3, 1, 3, 10, 30)} and so on.
  }
  \item{n}{
    desired number of intervals between major axis ticks (passed to
    \code{\link{pretty}}.
  }
  \item{n2, min.n2}{
    desired, and minimum, number of intervals between minor axis ticks
    (passed to \code{\link{pretty}}.
  }
}
\details{
  These functions are intended to be passed to the
  \code{xscale.components} or \code{yscale.components} arguments of
  high-level lattice plots. See \code{\link[lattice]{xscale.components.default}}.

  \code{xscale.components.logpower} draws tickmarks at the same
  locations as the default, but labels them more smartly using
  superscripts for the power (using expressions).

  \code{xscale.components.fractions} labels the tickmarks as fractions.

  \code{xscale.components.log10ticks} puts major tick marks at powers of
  10, and minor tickmarks in between.

}
\references{
  Sarkar, Deepayan (2008) \dQuote{Lattice: Multivariate Data
    Visualization with R}, Springer.  ISBN: 978-0-387-75968-5
  \url{http://lmdvr.r-forge.r-project.org/figures/figures.html?chapter=08;figure=08_04}
}
\examples{
xyplot((1:200)/20 ~ (1:200)/20, type = c("p", "g"),
  scales = list(x = list(log = 2), y = list(log = 10)),
  xscale.components = xscale.components.fractions,
  yscale.components = yscale.components.log10ticks)

xyplot((1:200)/20 ~ (1:200)/20, type = c("p", "g"),
  scales = list(x = list(log = 2), y = list(log = 10)),
  xscale.components = xscale.components.logpower,
  yscale.components = yscale.components.log10.3)

dd <- as.Date("2000-01-01") + 0:365
xyplot(0:365 ~ dd, type = c("p", "g"),
  xscale.components = xscale.components.subticks,
  yscale.components = yscale.components.subticks)
}
\keyword{dplot}

\name{panel.segplot}
\alias{prepanel.segplot}
\alias{panel.segplot}
\title{ Default prepanel and panel functions for segplot  }
\description{
  Draws line segments or rectangles.  Mainly intended to be used in
  conjunction with the \code{segplot} function.
}
\usage{
prepanel.segplot(x, y, z, subscripts, horizontal = TRUE, ...)

panel.segplot(x, y, z, level = NULL, subscripts, 
              at,
              draw.bands = is.factor(z),
              col, alpha, 
              lty, lwd,
              border,
              col.symbol = col, alpha.symbol = alpha,
              col.regions = regions$col,
              band.height = 0.6,
              horizontal = TRUE, 
              ...,
              segments.fun = panel.segments,
              centers = NULL,
              pch = 16)
}

\arguments{
  \item{x, y, z}{ Vectors corresponding to \code{x1}, \code{x2} and
    \code{y} respectively in the \code{segplot} formula.  The names are
    different for compatibility with \code{panel.levelplot}.  These are
    all the original vectors in \code{data}, not subsetted for
    particular panels.  }
  \item{level}{ optional vector controlling color of segments }
  \item{centers}{ optional vector of \sQuote{centers} of the segments.
    If specified, points will be plotted at these \code{y}-locations.  }
  \item{pch}{ plotting character used for \code{centers}. }
  \item{subscripts}{ integer subscript to be used as an indexing vector
    for \code{x}, \code{y}, \code{z} and \code{level}, giving the packet
    for the current panel. }
  \item{horizontal}{ logical, whether the segments are to be drawn
    horizontally (the default) or vertically.  This essentially swaps
    the role of the x- and y-axes in each panel. }
  \item{at}{ values of \code{level} where color code changes }
  \item{draw.bands}{ logical, whether to draw rectangles instead of
    lines }
  \item{col, alpha, lty, lwd, border}{ Graphical parameters for the
    segment.  Defaults to parameter settings for \code{"plot.line"} or
    \code{"plot.polygon"} for segments and rectangles respectively.
    \code{col} is overridden by \code{col.regions} if \code{level} is
    not null. }
  \item{col.symbol, alpha.symbol}{ Graphical parameters for the point if
    \code{centers} are plotted.  Defaults to the corresponding
    parameters for the segment.
  }
  \item{col.regions}{ vector of colors as in \code{\link[lattice]{levelplot}} }
  \item{band.height}{ height of rectangles (applicable if
    \code{draw.bands} is \code{TRUE} }

  \item{\dots}{ Other arguments, passed on to \code{panel.rect} (when
    \code{draw.bands=TRUE}), \code{segments.fun} (otherwise),
    \code{panel.points} (if \code{centers} is not \code{NULL}), etc. as
    appropriate. }

  \item{segments.fun}{ function used to plot segments when
    \code{draw.bands} is \code{FALSE}.  The default is to use
    \code{\link[lattice]{panel.segments}}, but \code{\link[lattice]{panel.arrows}} is a
    useful alternative (arguments to \code{segments.fun} can be provided
    via the \code{\dots} argument, see example for
    \code{\link{segplot}}).}

}
\value{
  For \code{prepanel.segplot} a list with components \code{xlim} and
  \code{ylim}.
}

\author{ Deepayan Sarkar \email{deepayan.sarkar@r-project.org} }
\seealso{ \code{\link{segplot}} }
\keyword{ dplot }

\name{panel.voronoi}
\alias{panel.voronoi}
\alias{panel.levelplot.points}
\title{ Panel functions for level-coded irregular points }
\description{
  These panel functions for \code{\link[lattice]{levelplot}}
  can represent irregular (x, y) points with a color covariate.
  \code{panel.levelplot.points} simply draws color-coded points.
  \code{panel.voronoi} uses the \pkg{interp} or \pkg{deldir} package to
  calculate the spatial extension of a set of points in 2 dimensions.
  This is known variously as a Voronoi mosaic, a Dirichlet tesselation,
  or Thiessen polygons.
}
\usage{
panel.voronoi(x, y, z, subscripts = TRUE, at = pretty(z),
   points = TRUE, border = "transparent",
   na.rm = FALSE, win.expand = 0.07, use.tripack = FALSE,
   backend = c("interp", "deldir"),
   ...,
   col.regions = regions$col, alpha.regions = regions$alpha)

panel.levelplot.points(x, y, z, subscripts = TRUE, at = pretty(z),
   shrink, labels, label.style, contour, region,
   pch = 21, col.symbol = "#00000044",
   ...,
   col.regions = regions$col, fill = NULL)
}
\arguments{
  \item{x, y, z}{ an irregular set of points at locations (x, y) with
    value z. }
  \item{subscripts}{ integer vector indicating what subset of x, y and z
    to draw. Typically passed by \code{\link[lattice]{levelplot}}. }
  \item{at, col.regions, alpha.regions}{ color scale definition;
    see \code{\link[lattice]{panel.levelplot}}. }
  \item{points}{ whether to draw the (x, y) points. }
  \item{border}{ color for polygon borders. }
  \item{na.rm}{ if \code{TRUE}, points with missing \code{z} values will
    be excluded from the calculation of polygons. If \code{FALSE},
    those polygons are calculated but are not drawn (i.e. are
    transparent). }
  \item{win.expand}{
    Defines the rectangular window bounding the polygons. This is a
    factor by which to expand the range of the data. Set to 0 to limit
    drawing at the furthest data point locations. Applies only for the
    \code{"deldir"} backend (see Details).
  }
  \item{use.tripack}{ Ignored, with a warning if a value is
    specified. See Details. }
  \item{backend}{ Character string specifying backend package to use to
    calculate the Voronoi mosaic. Defaults to \code{"interp"}.}
  \item{\dots}{Additional arguments; passed to
    \code{\link[lattice]{panel.xyplot}} if \code{points = TRUE}. }
  \item{pch, col.symbol}{ symbol and border color for points.
    A filled symbol should be used, i.e. in the range 21-25. }
  \item{shrink}{ Ignored. }
  \item{labels}{ Ignored. }
  \item{label.style}{ Ignored. }
  \item{contour}{ Ignored. }
  \item{region}{ Ignored. }
  \item{fill}{ Ignored. }
}
\details{
  Up to version 0.6-29, latticeExtra used the \pkg{deldir} package to
  compute the Voronoi mosaic by default, while offering the choice of
  using the faster but non-free ACM licensed implementation in the
  \pkg{tripack} package by setting \code{use.tripack = TRUE}. Later
  versions use the FOSS replacements in \pkg{interp} by default, and
  deprecates the \code{use.tripack} argument. The \pkg{deldir}
  implementation, which allows polygons to be clipped to a rectangular
  window (the \code{win.expand} argument), can still be used using
  \code{backend = "deldir"}.
}
\author{ Felix Andrews \email{felix@nfrac.org} }
\seealso{ \code{\link{tileplot}},
  \code{\link[lattice]{panel.levelplot}},
  \code{\link[deldir]{deldir}}  }
\examples{
## a variant of Figure 5.6 from Sarkar (2008)
## http://lmdvr.r-forge.r-project.org/figures/figures.html?chapter=05;figure=05_06

depth.ord <- rev(order(quakes$depth))
quakes$Magnitude <- equal.count(quakes$mag, 4)
quakes.ordered <- quakes[depth.ord, ]

levelplot(depth ~ long + lat | Magnitude, data = quakes.ordered,
         panel = panel.levelplot.points, type = c("p", "g"),
         aspect = "iso", prepanel = prepanel.default.xyplot)

## a levelplot with jittered cells

xyz <- expand.grid(x = 0:9, y = 0:9)
xyz[] <- jitter(as.matrix(xyz))
xyz$z <- with(xyz, sqrt((x - 5)^2 + (y - 5)^2))
levelplot(z ~ x * y, xyz, panel = panel.voronoi, points = FALSE)

## hexagonal cells

xyz$y <- xyz$y + c(0, 0.5)
levelplot(z ~ x * y, xyz, panel = panel.voronoi, points = FALSE)
}
\keyword{ hplot }

\name{useOuterStrips}
\alias{useOuterStrips}
\title{ Put Strips on the Boundary of a Lattice Display }
\description{
  Try to update a \code{"trellis"} object so that strips are only
  shown on the top and left boundaries when printed, instead of in every
  panel as is usual.  This is only meaningful when there are exactly two
  conditioning variables.

}
\usage{

useOuterStrips(x,
               strip = strip.default,
               strip.left = strip.custom(horizontal = FALSE),
               strip.lines = 1,
               strip.left.lines = strip.lines)

}

\arguments{
  \item{x}{ An object of class \code{"trellis"}. }
  \item{strip, strip.left}{ A function, character string or logical that
    would be appropriate \code{strip} and \code{strip.left} arguments
    respectively in a high level lattice function call (see
    \code{\link[lattice]{xyplot}}) }
  \item{strip.lines, strip.left.lines}{ height of strips in number of
    lines; helpful for multi-line text or mathematical annotation in
    strips. }
}

\details{
  \code{useOuterStrips} modifies a \code{"trellis"} object with
  \code{length(dim(x)) == 2} so that when plotted, strips are only shown
  on the top and left boundaries of the panel layout, rather than on top
  of every panel, as is the usual behaviour.

  If the original \code{"trellis"} object \code{x} includes non-default
  \code{strip} and \code{strip.left} arguments, they will be ignored.
  To provide customized strip behaviour, specify the custom strip
  functions directly as arguments to \code{useOuterStrips}.
  
}

\value{

  An object of class \code{"trellis"}; essentially the same as \code{x},
  but with certain properties modified.

}
\author{ Deepayan Sarkar }

\seealso{ \code{\link[lattice]{Lattice}}, \code{\link[lattice]{xyplot}} }

\examples{

library(lattice)

mtcars$HP <- equal.count(mtcars$hp)

useOuterStrips(xyplot(mpg ~ disp | HP + factor(cyl), mtcars))

useOuterStrips(xyplot(mpg ~ disp | factor(cyl) + HP, mtcars),
               strip.left = FALSE,
               strip = strip.custom(style = 4))

}

\keyword{dplot}

\name{ecdfplot}
\alias{ecdfplot}
\alias{ecdfplot.formula}
\alias{ecdfplot.numeric}
\alias{panel.ecdfplot}
\alias{prepanel.ecdfplot}
\title{Trellis Displays of Empirical CDF}
\description{
  Conditional displays of Empirical Cumulative Distribution Functions
}

\usage{
ecdfplot(x, data, \dots)

\method{ecdfplot}{formula}(x, data,
         prepanel = "prepanel.ecdfplot", 
         panel = "panel.ecdfplot",
         ylab,
         \dots)
\method{ecdfplot}{numeric}(x, data = NULL, xlab, \dots)

prepanel.ecdfplot(x, f.value = NULL, \dots)

panel.ecdfplot(x, f.value = NULL, type = "s",
               groups = NULL, qtype = 7,
               ref = TRUE,
               \dots)

}
\arguments{
  \item{x}{ For \code{ecdfplot}, \code{x} is the object on which
    method dispatch is carried out.  For the \code{"formula"} method,
    \code{x} is a formula describing the form of conditioning plot, and
    has to be of the form \code{~x}, where \code{x} is assumed to be a
    numeric vector.  Further conditioning variables are allowed as
    usual.

    A similar interpretation holds for \code{x} in the \code{"numeric"}
    method as well as \code{prepanel.ecdfplot} and
    \code{panel.ecdfplot}.
  }

  \item{data}{ For the \code{"formula"} method, a data frame containing
    values for any variables in the formula, as well as those in
    \code{groups} and \code{subset} if applicable. }

  \item{prepanel, panel}{ panel and prepanel function used to create the
    display.  }

  \item{xlab, ylab}{ axis labels; typically a character string or an
    expression. }

  \item{groups}{ a grouing variable of the same length as \code{x}.  If
    specified, ECDF plots are computed for each subset defined by unique
    values of \code{groups} and the resulting functions superposed
    within each panel.
  }
  \item{f.value, qtype}{ Defines how quantiles are calculated. See
    \code{\link[lattice]{panel.qqmath}}. }
  \item{ref}{ logical, whether a reference line should be drawn at 0 and
    1}
  \item{type}{ how the plot is rendered; see
    \code{\link[lattice]{panel.xyplot}} }

  \item{\dots}{ extra arguments, passed on as appropriate.  Standard
    lattice arguments as well as arguments to \code{panel.ecdfplot}
    can be supplied directly in the high level \code{ecdfplot} call.
  }
}

\value{
  \code{ecdfplot} produces an object of class \code{"trellis"}. The
  \code{update} method can be used to update components of the object and
  the \code{print} method (usually called by default) will plot it on an
  appropriate plotting device.

}
\author{ Deepayan Sarkar \email{deepayan.sarkar@r-project.org}}
\seealso{
  \code{\link[lattice]{qqmath}} for Quantile plots which are
  more generally useful, especially when comparing with a theoretical
  distribution other than uniform.  An ECDF plot is essentially a
  transposed version (i.e., with axes switched) of a uniform quantile
  plot.
}

\examples{

data(singer, package = "lattice")
ecdfplot(~height | voice.part, data = singer)

}

\keyword{dplot}

\name{panel.xyimage}
\alias{panel.xyimage}
\title{
  Scatter plots using emoji-like images
}
\description{
  Panel function that create scatter plots with emoji-like images for
  plotting character. Images can be local files or URLs. Only PNG and
  JPEG images are currently supported.
}
\usage{
panel.xyimage(x, y, 
              subscripts,
              groups = NULL,
              pch = NULL,
              cex = 1,
              ...,
              grid = FALSE, abline = NULL)
}
\arguments{
  \item{x, y}{
    Data vectors to be plotted.
  }
  \item{subscripts}{
    An integer vector of subscripts giving indices of the \code{x} and
    \code{y} values in the original data source.  See the corresponding
    entry in \code{\link[lattice]{xyplot}} for details.
  }
  \item{groups}{
    A factor defining groups.
  }
  \item{pch}{
    A character vector giving path(s) or URL(s) of PNG or JPEG files. If 
    \code{groups} is specified, there should be one value for each
    level, otherwise this should have length 1. Recycled as necessary.
  }
  \item{cex}{
    A numeric multiplier for the size of the symbols. As with
    \code{pch}, this can be a vector corresponding to levels of
    \code{groups}.
  }
  \item{\dots}{
    Further arguments are accepted but ignored.
  }
  \item{grid}{
    A logical flag, character string, or list specifying whether and how
    a background grid should be drawn. See \code{\link[lattice]{panel.xyplot}}
    for details.
  }
  \item{abline}{
    A numeric vector or list, specifying arguments arguments for
    \code{\link[lattice]{panel.abline}}, which is called with those
    arguments. See \code{\link[lattice]{panel.xyplot}}
    for details.
  }
}
\details{
  The image sources given by \code{pch} are downloaded if necessary and
  read in every time the panel function is called. If the same images
  are to be used in multiple panels, it may be more efficient to
  download them once and provide the file paths rather than provide
  URLs.
}
\author{
  Deepayan Sarkar
}
\seealso{
  \code{\link[lattice]{panel.xyplot}}
}
\examples{
\dontrun{
alive <- "https://twemoji.maxcdn.com/72x72/1f60a.png"
dead <- "https://twemoji.maxcdn.com/72x72/1f480.png"

dotplot(Titanic,
        scales = list(x = "free"),
        between = list(x = 1),
        panel = panel.xyimage,
        pch = c(dead, alive), grid = "h", 
        main = "Survival on the Titanic",
        xlab = "Number of persons")
}
}
\keyword{ dplot }

\name{marginal.plot}
\alias{marginal.plot}
\title{ Display marginal distributions }
\description{
  Display marginal distributions of several variables,
  which may be numeric and/or categorical, on one plot.
}
\usage{
marginal.plot(x,
              data = NULL,
              groups = NULL,
              reorder = !is.table(x),
              plot.points = FALSE,
              ref = TRUE, cut = 0,
              origin = 0, %ylim = c(0, NA),
              xlab = NULL, ylab = NULL,
              type = c("p", if (is.null(groups)) "h"),
              ...,
              subset = TRUE,
              as.table = TRUE,
              subscripts = TRUE,
              default.scales = list(
                relation = "free",
                abbreviate = TRUE, minlength = 5,
                rot = 30, cex = 0.75, tick.number = 3,
                y = list(draw = FALSE)),
             layout = NULL,
             lattice.options = list(
               layout.heights = list(
                 axis.xlab.padding = list(x = 0),
                 xlab.key.padding = list(x = 0))))
}
\arguments{
  \item{x}{ a data frame or table, or a formula of which the first term
    is a data frame or table. Otherwise coerced with
    \code{as.data.frame}. }
  \item{data}{ an optional data source in which groups and subset may be
    be evaluated. }
  \item{groups}{ term, to be evaluated in \code{data}, that is used as a
    grouping variable. }
  \item{reorder}{ whether to reorder factor variables by frequency. }
  \item{subset}{ data subset expression, evaluated in \code{data}. }
  \item{plot.points, ref, cut}{ passed to \code{panel.densityplot}. }
  \item{origin, type}{ passed to \code{panel.dotplot}. }
  \item{xlab, ylab, as.table, subscripts}{
    see \code{\link[lattice]{xyplot}}.
  }
  \item{default.scales, layout, lattice.options}{
    see \code{\link[lattice]{xyplot}}.
  }
  \item{\dots}{ passed to \code{\link[lattice]{panel.densityplot}} and/or
    \code{\link[lattice]{panel.dotplot}}. }
}
\details{
  In the case of mixed numeric and categorical variables,
  the trellis objects from \code{dotplot()} and \code{densityplot()}
  are merged.
}
\value{
  a trellis object.
}
\author{ Felix Andrews \email{felix@nfrac.org} }
\seealso{ \code{\link[lattice]{panel.dotplot}},
  \code{\link[lattice]{panel.densityplot}} }
\examples{
enviro <- environmental
## make an ordered factor (so it will not be reordered)
enviro$smell <- cut(enviro$ozone, breaks = c(0, 30, 50, Inf),
    labels = c("ok", "hmmm", "yuck"), ordered = TRUE)
marginal.plot(enviro)

## using groups
enviro$is.windy <- factor(enviro$wind > 10,
    levels = c(TRUE, FALSE), labels = c("windy", "calm"))
marginal.plot(enviro[,1:5], data = enviro, groups = is.windy,
    auto.key = list(lines = TRUE))

## support for tables
marginal.plot(Titanic)
## table with groups
marginal.plot(~ Titanic, data = Titanic, groups = Survived,
    type = "b", auto.key = list(title = "Survived?"))
}
\keyword{ hplot }

\name{custom.theme}
\alias{custom.theme}
\alias{custom.theme.2}
\title{ Create a lattice theme based on specified colors }
\description{
  Creates a lattice theme given a few colors.  Non-color settings
  are not included.  The colors are typically used to define the
  standard grouping (superposition) colors, and the first color is used
  for ungrouped displays.
}
\usage{
custom.theme(symbol = RColorBrewer::brewer.pal(n = 8, name = "Dark2"),
             fill = RColorBrewer::brewer.pal(n = 12, name = "Set3"),
             region = RColorBrewer::brewer.pal(n = 11, name = "Spectral"),
             reference = "#e8e8e8",
             bg = "transparent",
             fg = "black",
             \dots)

## different defaults ("Set1", "Accent", "RdBu"):
custom.theme.2(\dots)
}
\arguments{
  \item{symbol}{ a vector of symbol colors.   }
  \item{fill}{ a vector of fill colors (for barcharts, etc.) }
  \item{region}{ a vector of colors that is used to define a continuous
    color gradient using \code{\link{colorRampPalette}}}
  \item{reference}{ a color for reference lines and such  }
  \item{bg}{ a background color }
  \item{fg}{ a foreground color, primarily for annotation }
  \item{\dots}{ further arguments passed to \code{\link[lattice]{simpleTheme}}
    and used to modify the theme. }
}
\value{

  A list that can be supplied to
  \code{\link[lattice]{trellis.par.get}} or as the
  \code{theme} argument to
  \code{\link[lattice]{trellis.device}}.

}
\author{ Deepayan Sarkar }
\examples{
set.seed(0)

## create a plot to demonstrate graphical settings
obj <-
xyplot(Sepal.Length + Sepal.Width ~ Petal.Length + Petal.Width, iris,
       type = c("p", "r"), jitter.x = TRUE, jitter.y = TRUE, factor = 5,
       auto.key = list(lines = TRUE, rectangles = TRUE))
obj <- update(obj, legend = list(right =
         list(fun = "draw.colorkey", args = list(list(at = 0:100)))))

%% NOTE: used in website for "style example"
## draw with default theme
obj

## draw with custom.theme()
update(obj, par.settings = custom.theme())

## create a theme with paired colours, filled points, etc
update(obj, par.settings =
  custom.theme(symbol = RColorBrewer::brewer.pal(12, "Paired"),
               fill = RColorBrewer::brewer.pal(12, "Paired"),
               region = RColorBrewer::brewer.pal(9, "Blues"),
               bg = "grey90", fg = "grey20", pch = 16))

## draw with custom.theme.2()
update(obj, par.settings = custom.theme.2())
}
\keyword{dplot}
